@@ -1,11 +1,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-
 import warnings
 
-import numpy as np
+import os
+import sys
+import glob
+import ctypes
 from functools import partial
 
+import numpy as np
+from numpy.ctypeslib import ndpointer, load_library
+
 from .core import Kernel, Kernel1D, Kernel2D, MAX_NORMALIZATION
 from ..utils.exceptions import AstropyUserWarning
 from ..utils.console import human_file_size
@@ -14,8 +19,34 @@ from .. import units as u
 from ..nddata import support_nddata
 from ..modeling.core import _make_arithmetic_operator, BINARY_OPERATORS
 from ..modeling.core import _CompoundModelMeta
+from .utils import KernelSizeError, has_even_axis, raise_even_kernel_exception
 
+LIBRARY_PATH = os.path.dirname(__file__)
 
+try:
+    _convolve = load_library("_convolve", LIBRARY_PATH)
+except Exception:
+    raise ImportError("Convolution C extension is missing. Try re-building astropy.")
+
+# The GIL is automatically released by default when calling functions imported
+# from libaries loaded by ctypes.cdll.LoadLibrary(<path>)
+
+# Declare prototypes
+# Boundary None
+_convolveNd_c = _convolve.convolveNd_c
+_convolveNd_c.restype = None
+_convolveNd_c.argtypes = [ndpointer(ctypes.c_double, flags={"C_CONTIGUOUS", "WRITEABLE"}), # return array
+            ndpointer(ctypes.c_double, flags="C_CONTIGUOUS"), # input array
+            ctypes.c_uint, # N dim
+            ndpointer(ctypes.c_size_t, flags="C_CONTIGUOUS"), # size array for input and result unless
+                                                              # embed_result_within_padded_region is False,
+                                                              # in which case the result array is assumed to be
+                                                              # input.shape - 2*(kernel.shape//2). Note: integer division.
+            ndpointer(ctypes.c_double, flags="C_CONTIGUOUS"), # kernel array
+            ndpointer(ctypes.c_size_t, flags="C_CONTIGUOUS"), # size array for kernel
+            ctypes.c_bool, # nan_interpolate
+            ctypes.c_bool, # embed_result_within_padded_region
+            ctypes.c_uint] # n_threads
 
 # Disabling all doctests in this module until a better way of handling warnings
 # in doctests can be determined
@@ -23,6 +54,43 @@ __doctest_skip__ = ['*']
 
 BOUNDARY_OPTIONS = [None, 'fill', 'wrap', 'extend']
 
+def _copy_input_if_needed(input, dtype=float, order='C', nan_treatment=None, mask=None, fill_value=None):
+    # Alias input
+    input = input.array if isinstance(input, Kernel) else input
+    output = input
+    # Copy input
+    try:
+        # Anything that's masked must be turned into NaNs for the interpolation.
+        # This requires copying. A copy is also needed for nan_treatment == 'fill'
+        # A copy prevents possible function side-effects of the input array.
+        if nan_treatment == 'fill' or np.ma.is_masked(input) or mask is not None:
+            if np.ma.is_masked(input):
+                # ``np.ma.maskedarray.filled()`` returns a copy, however there is no way to specify the return type
+                # or order etc.
+                # In addition ``np.nan`` is a ``float`` and there is no conversion to an ``int`` type.
+                # Therefore, a pre-fill copy is needed for non ``float`` masked arrays.
+                # ``subok=True`` is needed to retain ``np.ma.maskedarray.filled()``.
+                # ``copy=False`` allows the fill to act as the copy if type and order are already correct.
+                output = np.array(input, dtype=dtype, copy=False, order=order, subok=True)
+                output = output.filled(fill_value)
+            else:
+                # Since we're making a copy, we might as well use `subok=False` to save,
+                # what is probably, a negligible amount of memory.
+                output = np.array(input, dtype=dtype, copy=True, order=order, subok=False)
+
+            if mask is not None:
+                # mask != 0 yields a bool mask for all ints/floats/bool
+                output[mask != 0] = fill_value
+        else:
+            # The call below is synonymous with np.asanyarray(array, ftype=float, order='C')
+            # The advantage of `subok=True` is that it won't copy when array is an ndarray subclass. If it
+            # is and `subok=False` (default), then it will copy even if `copy=False`. This uses less memory
+            # when ndarray subclasses are passed in.
+            output = np.array(input, dtype=dtype, copy=False, order=order, subok=True)
+    except (TypeError, ValueError) as e:
+        raise TypeError('input should be a Numpy array or something '
+                        'convertible into a float array', e)
+    return output
 
 @support_nddata(data='array')
 def convolve(array, kernel, boundary='fill', fill_value=0.,
@@ -40,7 +108,7 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
 
     Parameters
     ----------
-    array : `numpy.ndarray` or `~astropy.nddata.NDData`
+    array : `~astropy.nddata.NDData` or `numpy.ndarray` or array-like
         The array to convolve. This should be a 1, 2, or 3-dimensional array
         or a list or a set of nested lists representing a 1, 2, or
         3-dimensional array.  If an `~astropy.nddata.NDData`, the ``mask`` of
@@ -64,8 +132,7 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
     fill_value : float, optional
         The value to use outside the array when using ``boundary='fill'``
     normalize_kernel : bool, optional
-        Whether to normalize the kernel to have a sum of one prior to
-        convolving
+        Whether to normalize the kernel to have a sum of one.
     nan_treatment : 'interpolate', 'fill'
         interpolate will result in renormalization of the kernel at each
         position ignoring (pixels that are NaN in the image) in both the image
@@ -73,7 +140,7 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
         'fill' will replace the NaN pixels with a fixed numerical value (default
         zero, see ``fill_value``) prior to convolution
         Note that if the kernel has a sum equal to zero, NaN interpolation
-        is not possible and will raise an exception
+        is not possible and will raise an exception.
     preserve_nan : bool
         After performing convolution, should pixels that were originally NaN
         again become NaN?
@@ -102,21 +169,6 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
     For masked arrays, masked values are treated as NaNs.  The convolution
     is always done at ``numpy.float`` precision.
     '''
-    from .boundary_none import (convolve1d_boundary_none,
-                                convolve2d_boundary_none,
-                                convolve3d_boundary_none)
-
-    from .boundary_extend import (convolve1d_boundary_extend,
-                                  convolve2d_boundary_extend,
-                                  convolve3d_boundary_extend)
-
-    from .boundary_fill import (convolve1d_boundary_fill,
-                                convolve2d_boundary_fill,
-                                convolve3d_boundary_fill)
-
-    from .boundary_wrap import (convolve1d_boundary_wrap,
-                                convolve2d_boundary_wrap,
-                                convolve3d_boundary_wrap)
 
     if boundary not in BOUNDARY_OPTIONS:
         raise ValueError("Invalid boundary option: must be one of {0}"
@@ -125,7 +177,15 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
     if nan_treatment not in ('interpolate', 'fill'):
         raise ValueError("nan_treatment must be one of 'interpolate','fill'")
 
-    # The cython routines all need float type inputs (so, a particular
+    # OpenMP support is disabled at the C src code level, changing this will have
+    # no effect.
+    n_threads = 1
+
+    # Keep refs to originals
+    passed_kernel = kernel
+    passed_array = array
+
+    # The C routines all need float type inputs (so, a particular
     # bit size, endianness, etc.).  So we have to convert, which also
     # has the effect of making copies so we don't modify the inputs.
     # After this, the variables we work with will be array_internal, and
@@ -136,171 +196,172 @@ def convolve(array, kernel, boundary='fill', fill_value=0.,
     # It is always necessary to make a copy of kernel (since it is modified),
     # but, if we just so happen to be lucky enough to have the input array
     # have exactly the desired type, we just alias to array_internal
+    # Convert kernel to ndarray if not already
 
-    # Check if kernel is kernel instance
-    if isinstance(kernel, Kernel):
-        # Check if array is also kernel instance, if so convolve and
-        # return new kernel instance
-        if isinstance(array, Kernel):
-            if isinstance(array, Kernel1D) and isinstance(kernel, Kernel1D):
-                new_array = convolve1d_boundary_fill(array.array, kernel.array,
-                                                     0, True)
-                new_kernel = Kernel1D(array=new_array)
-            elif isinstance(array, Kernel2D) and isinstance(kernel, Kernel2D):
-                new_array = convolve2d_boundary_fill(array.array, kernel.array,
-                                                     0, True)
-                new_kernel = Kernel2D(array=new_array)
-            else:
-                raise Exception("Can't convolve 1D and 2D kernel.")
-            new_kernel._separable = kernel._separable and array._separable
-            new_kernel._is_bool = False
-            return new_kernel
-        kernel = kernel.array
+    # Copy or alias array to array_internal
+    array_internal = _copy_input_if_needed(passed_array, dtype=float, order='C',
+                                      nan_treatment=nan_treatment, mask=mask,
+                                      fill_value=np.nan)
+    array_dtype = getattr(passed_array, 'dtype', array_internal.dtype)
+    # Copy or alias kernel to kernel_internal
+    kernel_internal = _copy_input_if_needed(passed_kernel, dtype=float, order='C',
+                                        nan_treatment=None, mask=None,
+                                        fill_value=fill_value)
 
-    # Check that the arguments are lists or Numpy arrays
+    # Make sure kernel has all odd axes
+    if has_even_axis(kernel_internal):
+        raise_even_kernel_exception()
 
-    if isinstance(array, list):
-        array_internal = np.array(array, dtype=float)
-        array_dtype = array_internal.dtype
-    elif isinstance(array, np.ndarray):
-        # Note this won't copy if it doesn't have to -- which is okay
-        # because none of what follows modifies array_internal.
-        array_dtype = array.dtype
-        array_internal = array.astype(float, copy=False)
-    else:
-        raise TypeError("array should be a list or a Numpy array")
+    # If both image array and kernel are Kernel instances
+    # constrain convolution method
+    # This must occur before the main alias/copy of ``passed_kernel`` to
+    # ``kernel_internal`` as it is used for filling masked kernels.
+    if isinstance(passed_array, Kernel) and isinstance(passed_kernel, Kernel):
+        warnings.warn("Both array and kernel are Kernel instances, hardwiring the following parameters: "
+                              "boundary='fill', fill_value=0, normalize_Kernel=True, "
+                              "nan_treatment='interpolate'",
+                              AstropyUserWarning)
+        boundary = 'fill'
+        fill_value = 0
+        normalize_kernel = True
+        nan_treatment='interpolate'
 
-    if isinstance(kernel, list):
-        kernel_internal = np.array(kernel, dtype=float)
-    elif isinstance(kernel, np.ndarray):
-        # Note this always makes a copy, since we will be modifying it
-        kernel_internal = kernel.astype(float)
-    else:
-        raise TypeError("kernel should be a list or a Numpy array")
+    #-----------------------------------------------------------------------
+    # From this point onwards refer only to ``array_internal`` and
+    # ``kernel_internal``.
+    # Assume both are base np.ndarrays and NOT subclasses e.g. NOT
+    # ``Kernel`` nor ``np.ma.maskedarray`` classes.
+    #-----------------------------------------------------------------------
 
-    # Check that the number of dimensions is compatible
-    if array_internal.ndim != kernel_internal.ndim:
+    # Check dimensionality
+    if array_internal.ndim == 0:
+        raise Exception("cannot convolve 0-dimensional arrays")
+    elif array_internal.ndim > 3:
+        raise NotImplementedError('convolve only supports 1, 2, and 3-dimensional '
+                                  'arrays at this time')
+    elif array_internal.ndim != kernel_internal.ndim:
         raise Exception('array and kernel have differing number of '
                         'dimensions.')
 
-    # anything that's masked must be turned into NaNs for the interpolation.
-    # This requires copying the array_internal
-    array_internal_copied = False
-    if np.ma.is_masked(array):
-        array_internal = array_internal.filled(np.nan)
-        array_internal_copied = True
-    if mask is not None:
-        if not array_internal_copied:
-            array_internal = array_internal.copy()
-            array_internal_copied = True
-        # mask != 0 yields a bool mask for all ints/floats/bool
-        array_internal[mask != 0] = np.nan
-    if np.ma.is_masked(kernel):
-        # *kernel* doesn't support NaN interpolation, so instead we just fill it
-        kernel_internal = kernel_internal.filled(fill_value)
+    array_shape = np.array(array_internal.shape)
+    kernel_shape = np.array(kernel_internal.shape)
+    pad_width = kernel_shape//2
+
+    # For boundary=None only the center space is convolved. All array indices within a
+    # distance kernel.shape//2 from the edge are completely ignored (zeroed).
+    # E.g. (1D list) only the indices len(kernel)//2 : len(array)-len(kernel)//2
+    # are convolved. It is therefore not possible to use this method to convolve an
+    # array by a kernel that is larger (see note below) than the array - as ALL pixels would be ignored
+    # leaving an array of only zeros.
+    # Note: For even kernels the correctness condition is array_shape > kernel_shape.
+    # For odd kernels it is:
+    # array_shape >= kernel_shape OR array_shape > kernel_shape-1 OR array_shape > 2*(kernel_shape//2).
+    # Since the latter is equal to the former two for even lengths, the latter condition is complete.
+    if boundary == None and not np.all(array_shape > 2*pad_width):
+        raise KernelSizeError("for boundary=None all kernel axes must be smaller than array's - "
+                              "use boundary in ['fill', 'extend', 'wrap'] instead.")
+
+    # NaN interpolation significantly slows down the C convolution
+    # computation. Since nan_treatment = 'interpolate', is the default
+    # check whether it is even needed, if not, don't interpolate.
+    # NB: np.isnan(array_internal.sum()) is faster than np.isnan(array_internal).any()
+    nan_interpolate = (nan_treatment == 'interpolate') and np.isnan(array_internal.sum())
+
+    # Check if kernel is normalizable
+    if normalize_kernel or nan_interpolate:
+        kernel_sum = kernel_internal.sum()
+        kernel_sums_to_zero = np.isclose(kernel_sum, 0, atol=normalization_zero_tol)
+
+        if kernel_sum < 1. / MAX_NORMALIZATION or kernel_sums_to_zero:
+            raise ValueError("The kernel can't be normalized, because its sum is "
+                            "close to zero. The sum of the given kernel is < {0}"
+                            .format(1. / MAX_NORMALIZATION))
 
     # Mark the NaN values so we can replace them later if interpolate_nan is
     # not set
-    if preserve_nan:
-        badvals = np.isnan(array_internal)
-
-    if nan_treatment == 'fill':
+    if preserve_nan or nan_treatment == 'fill':
         initially_nan = np.isnan(array_internal)
-        array_internal[initially_nan] = fill_value
+        if nan_treatment == 'fill':
+            array_internal[initially_nan] = fill_value
 
-    # Because the Cython routines have to normalize the kernel on the fly, we
-    # explicitly normalize the kernel here, and then scale the image at the
-    # end if normalization was not requested.
-    kernel_sum = kernel_internal.sum()
-    kernel_sums_to_zero = np.isclose(kernel_sum, 0, atol=normalization_zero_tol)
+    # Avoid any memory allocation within the C code. Allocate output array
+    # here and pass through instead.
+    result = np.zeros(array_internal.shape, dtype=float, order='C')
 
-    if (kernel_sum < 1. / MAX_NORMALIZATION or kernel_sums_to_zero) and normalize_kernel:
-        raise Exception("The kernel can't be normalized, because its sum is "
-                        "close to zero. The sum of the given kernel is < {0}"
-                        .format(1. / MAX_NORMALIZATION))
+    embed_result_within_padded_region = True
+    array_to_convolve = array_internal
+    if boundary in ('fill', 'extend', 'wrap'):
+        embed_result_within_padded_region = False
+        if boundary == 'fill':
+            # This method is faster than using numpy.pad(..., mode='constant')
+            array_to_convolve = np.full(array_shape + 2*pad_width, fill_value=fill_value, dtype=float, order='C')
+            # Use bounds [pad_width[0]:array_shape[0]+pad_width[0]] instead of [pad_width[0]:-pad_width[0]]
+            # to account for when the kernel has size of 1 making pad_width = 0.
+            if array_internal.ndim == 1:
+                array_to_convolve[pad_width[0]:array_shape[0]+pad_width[0]] = array_internal
+            elif array_internal.ndim == 2:
+                array_to_convolve[pad_width[0]:array_shape[0]+pad_width[0],
+                             pad_width[1]:array_shape[1]+pad_width[1]] = array_internal
+            elif array_internal.ndim == 3:
+                array_to_convolve[pad_width[0]:array_shape[0]+pad_width[0],
+                             pad_width[1]:array_shape[1]+pad_width[1],
+                             pad_width[2]:array_shape[2]+pad_width[2]] = array_internal
+        else:
+            np_pad_mode_dict = {'fill':'constant', 'extend':'edge', 'wrap':'wrap'}
+            np_pad_mode = np_pad_mode_dict[boundary]
+            pad_width = kernel_shape//2
 
-    if not kernel_sums_to_zero:
-        kernel_internal /= kernel_sum
+            if array_internal.ndim == 1:
+                np_pad_width = (pad_width[0],)
+            elif array_internal.ndim == 2:
+                np_pad_width = ( (pad_width[0],), (pad_width[1],) )
+            elif array_internal.ndim == 3:
+                np_pad_width = ( (pad_width[0],), (pad_width[1],), (pad_width[2],) )
 
-    renormalize_by_kernel = not kernel_sums_to_zero
+            array_to_convolve = np.pad(array_internal, pad_width=np_pad_width,
+                                  mode=np_pad_mode)
 
-    if array_internal.ndim == 0:
-        raise Exception("cannot convolve 0-dimensional arrays")
-    elif array_internal.ndim == 1:
-        if boundary == 'extend':
-            result = convolve1d_boundary_extend(array_internal,
-                                                kernel_internal,
-                                                renormalize_by_kernel)
-        elif boundary == 'fill':
-            result = convolve1d_boundary_fill(array_internal,
-                                              kernel_internal,
-                                              float(fill_value),
-                                              renormalize_by_kernel)
-        elif boundary == 'wrap':
-            result = convolve1d_boundary_wrap(array_internal,
-                                              kernel_internal,
-                                              renormalize_by_kernel)
-        elif boundary is None:
-            result = convolve1d_boundary_none(array_internal,
-                                              kernel_internal,
-                                              renormalize_by_kernel)
-    elif array_internal.ndim == 2:
-        if boundary == 'extend':
-            result = convolve2d_boundary_extend(array_internal,
-                                                kernel_internal,
-                                                renormalize_by_kernel,
-                                               )
-        elif boundary == 'fill':
-            result = convolve2d_boundary_fill(array_internal,
-                                              kernel_internal,
-                                              float(fill_value),
-                                              renormalize_by_kernel,
-                                             )
-        elif boundary == 'wrap':
-            result = convolve2d_boundary_wrap(array_internal,
-                                              kernel_internal,
-                                              renormalize_by_kernel,
-                                             )
-        elif boundary is None:
-            result = convolve2d_boundary_none(array_internal,
-                                              kernel_internal,
-                                              renormalize_by_kernel,
-                                             )
-    elif array_internal.ndim == 3:
-        if boundary == 'extend':
-            result = convolve3d_boundary_extend(array_internal,
-                                                kernel_internal,
-                                                renormalize_by_kernel)
-        elif boundary == 'fill':
-            result = convolve3d_boundary_fill(array_internal,
-                                              kernel_internal,
-                                              float(fill_value),
-                                              renormalize_by_kernel)
-        elif boundary == 'wrap':
-            result = convolve3d_boundary_wrap(array_internal,
-                                              kernel_internal,
-                                              renormalize_by_kernel)
-        elif boundary is None:
-            result = convolve3d_boundary_none(array_internal,
-                                              kernel_internal,
-                                              renormalize_by_kernel)
+    _convolveNd_c(result, array_to_convolve,
+                  array_to_convolve.ndim,
+                  np.array(array_to_convolve.shape, dtype=ctypes.c_size_t, order='C'),
+                  kernel_internal,
+                  np.array(kernel_shape, dtype=ctypes.c_size_t, order='C'),
+                  nan_interpolate, embed_result_within_padded_region,
+                  n_threads
+                  )
+
+    # So far, normalization has only occured for nan_treatment == 'interpolate'
+    # because this had to happen within the C extension so as to ignore
+    # any NaNs
+    if normalize_kernel:
+        if not nan_interpolate:
+            result /= kernel_sum
     else:
-        raise NotImplementedError('convolve only supports 1, 2, and 3-dimensional '
-                                  'arrays at this time')
+        if nan_interpolate:
+            result *= kernel_sum
 
-    # If normalization was not requested, we need to scale the array (since
-    # the kernel is effectively normalized within the cython functions)
-    if not normalize_kernel and not kernel_sums_to_zero:
-        result *= kernel_sum
+    if nan_interpolate and not preserve_nan and np.isnan(result.sum()):
+        warnings.warn("nan_treatment='interpolate', however, NaN values detected "
+                     "post convolution. A contiguous region of NaN values, larger "
+                     "than the kernel size, are present in the input array. "
+                     "Increase the kernel size to avoid this.", AstropyUserWarning)
 
     if preserve_nan:
-        result[badvals] = np.nan
+        result[initially_nan] = np.nan
 
-    if nan_treatment == 'fill':
-        array_internal[initially_nan] = np.nan
-
-    # Try to preserve the input type if it's a floating point type
-    if array_dtype.kind == 'f':
+    # Convert result to original data type
+    if isinstance(passed_array, Kernel):
+        if isinstance(passed_array, Kernel1D):
+            new_result = Kernel1D(array=result)
+        elif isinstance(passed_array, Kernel2D):
+            new_result = Kernel2D(array=result)
+        new_result._is_bool = False
+        new_result._separable = passed_array._separable
+        if isinstance(passed_kernel, Kernel):
+            new_result._separable = new_result._separable and passed_kernel._separable
+        return new_result
+    elif array_dtype.kind == 'f':
+        # Try to preserve the input type if it's a floating point type
         # Avoid making another copy if possible
         try:
             return result.astype(array_dtype, copy=False)
@@ -510,8 +571,12 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0.,
 
     # Convert array dtype to complex
     # and ensure that list inputs become arrays
-    array = np.asarray(array, dtype=complex)
-    kernel = np.asarray(kernel, dtype=complex)
+    array = _copy_input_if_needed(array, dtype=complex, order='C',
+                             nan_treatment=nan_treatment, mask=mask,
+                             fill_value=np.nan)
+    kernel = _copy_input_if_needed(kernel, dtype=complex, order='C',
+                                  nan_treatment=None, mask=None,
+                                  fill_value=0)
 
     # Check that the number of dimensions is compatible
     if array.ndim != kernel.ndim:
@@ -528,22 +593,6 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0.,
                          "allow_huge=True to override this exception."
                          .format(human_file_size(array_size_B.to_value(u.byte))))
 
-    # mask catching - masks must be turned into NaNs for use later in the image
-    if np.ma.is_masked(array):
-        mamask = array.mask
-        array = np.array(array)
-        array[mamask] = np.nan
-    elif mask is not None:
-        # copying here because we have to mask it below.  But no need to copy
-        # if mask is None because we won't modify it.
-        array = np.array(array)
-    if mask is not None:
-        # mask != 0 yields a bool mask for all ints/floats/bool
-        array[mask != 0] = np.nan
-    # the *kernel* doesn't support NaN interpolation, so instead we just fill it
-    if np.ma.is_masked(kernel):
-        kernel = kernel.filled(0)
-
     # NaN and inf catching
     nanmaskarray = np.isnan(array) | np.isinf(array)
     array[nanmaskarray] = 0
@@ -707,19 +756,16 @@ def convolve_fft(array, kernel, boundary='fill', fill_value=0.,
         # this check should be unnecessary; call it an insanity check
         raise ValueError("Encountered NaNs in convolve.  This is disallowed.")
 
-    # restore NaNs in original image (they were modified inplace earlier)
-    # We don't have to worry about masked arrays - if input was masked, it was
-    # copied
-    array[nanmaskarray] = np.nan
-    kernel[nanmaskkernel] = np.nan
-
     fftmult *= kernel_scale
 
     if return_fft:
         return fftmult
 
     if interpolate_nan:
-        rifft = (ifftn(fftmult)) / bigimwt
+        with np.errstate(divide='ignore'):
+            # divide by zeros are expected here; if the weight is zero, we want
+            # the output to be nan or inf
+            rifft = (ifftn(fftmult)) / bigimwt
         if not np.isscalar(bigimwt):
             if min_wt > 0.:
                 rifft[bigimwt < min_wt] = np.nan
@@ -773,7 +819,7 @@ def interpolate_replace_nans(array, kernel, convolve=convolve, **kwargs):
     newarray = array.copy()
 
     convolved = convolve(array, kernel, nan_treatment='interpolate',
-                         normalize_kernel=True, **kwargs)
+                         normalize_kernel=True, preserve_nan=False, **kwargs)
 
     isnan = np.isnan(array)
     newarray[isnan] = convolved[isnan]
