@@ -5,15 +5,17 @@ import numpy as np
 import numpy.ma as ma
 
 from ..convolve import convolve, convolve_fft
+from ..kernels import Gaussian2DKernel
+from ...utils.exceptions import AstropyUserWarning
 
-from numpy.testing import assert_array_almost_equal_nulp, assert_array_almost_equal
+from numpy.testing import (assert_array_almost_equal_nulp,
+                           assert_array_almost_equal,
+                           assert_allclose)
 
 import itertools
 
-VALID_DTYPES = []
-for dtype_array in ['>f4', '<f4', '>f8', '<f8']:
-    for dtype_kernel in ['>f4', '<f4', '>f8', '<f8']:
-        VALID_DTYPES.append((dtype_array, dtype_kernel))
+VALID_DTYPES = ('>f4', '<f4', '>f8', '<f8')
+VALID_DTYPE_MATRIX = list(itertools.product(VALID_DTYPES, VALID_DTYPES))
 
 BOUNDARY_OPTIONS = [None, 'fill', 'wrap', 'extend']
 NANHANDLING_OPTIONS = ['interpolate', 'fill']
@@ -31,6 +33,12 @@ try:
 except ImportError:
     HAS_SCIPY = False
 
+HAS_PANDAS = True
+try:
+    import pandas
+except ImportError:
+    HAS_PANDAS = False
+
 
 class TestConvolve1D:
     def test_list(self):
@@ -44,19 +52,87 @@ class TestConvolve1D:
         assert_array_almost_equal_nulp(z,
             np.array([0., 3.6, 5., 5.6, 5.6, 6.8, 0.]), 10)
 
-    def test_input_unmodified(self):
+    def test_tuple(self):
+        """
+        Test that convolve works correctly when inputs are tuples
+        """
+
+        x = (1, 4, 5, 6, 5, 7, 8)
+        y = (0.2, 0.6, 0.2)
+        z = convolve(x, y, boundary=None)
+        assert_array_almost_equal_nulp(z,
+            np.array([0., 3.6, 5., 5.6, 5.6, 6.8, 0.]), 10)
+
+    @pytest.mark.parametrize(('boundary', 'nan_treatment',
+                              'normalize_kernel', 'preserve_nan', 'dtype'),
+                             itertools.product(BOUNDARY_OPTIONS,
+                                               NANHANDLING_OPTIONS,
+                                               NORMALIZE_OPTIONS,
+                                               PRESERVE_NAN_OPTIONS,
+                                               VALID_DTYPES))
+    def test_input_unmodified(self, boundary, nan_treatment,
+                              normalize_kernel, preserve_nan, dtype):
         """
         Test that convolve works correctly when inputs are lists
         """
 
-        inlist = [1, 4, 5, 6, 5, 7, 8]
-        x = np.array(inlist)
-        y = [0.2, 0.6, 0.2]
-        z = convolve(x, y, boundary=None)
+        array = [1., 4., 5., 6., 5., 7., 8.]
+        kernel = [0.2, 0.6, 0.2]
+        x = np.array(array, dtype=dtype)
+        y = np.array(kernel, dtype=dtype)
 
-        assert np.all(np.array(inlist) == x)
+        # Make pseudoimmutable
+        x.flags.writeable = False
+        y.flags.writeable = False
 
-    @pytest.mark.parametrize(('dtype_array', 'dtype_kernel'), VALID_DTYPES)
+        z = convolve(x, y, boundary=boundary, nan_treatment=nan_treatment,
+                          normalize_kernel=normalize_kernel, preserve_nan=preserve_nan)
+
+        assert np.all(np.array(array, dtype=dtype) == x)
+        assert np.all(np.array(kernel, dtype=dtype) == y)
+
+    @pytest.mark.parametrize(('boundary', 'nan_treatment',
+                              'normalize_kernel', 'preserve_nan', 'dtype'),
+                             itertools.product(BOUNDARY_OPTIONS,
+                                               NANHANDLING_OPTIONS,
+                                               NORMALIZE_OPTIONS,
+                                               PRESERVE_NAN_OPTIONS,
+                                               VALID_DTYPES))
+    def test_input_unmodified_with_nan(self, boundary, nan_treatment,
+                                       normalize_kernel, preserve_nan, dtype):
+        """
+        Test that convolve doesn't modify the input data
+        """
+
+        array = [1., 4., 5., np.nan, 5., 7., 8.]
+        kernel = [0.2, 0.6, 0.2]
+        x = np.array(array, dtype=dtype)
+        y = np.array(kernel, dtype=dtype)
+
+        # Make pseudoimmutable
+        x.flags.writeable = False
+        y.flags.writeable = False
+
+        # make copies for post call comparison
+        x_copy = x.copy()
+        y_copy = y.copy()
+
+        z = convolve(x, y, boundary=boundary, nan_treatment=nan_treatment,
+                     normalize_kernel=normalize_kernel, preserve_nan=preserve_nan)
+
+        # ( NaN == NaN ) = False
+        # Only compare non NaN values for canonical equivalance
+        # and then check NaN explicitly with np.isnan()
+        array_is_nan = np.isnan(array)
+        kernel_is_nan = np.isnan(kernel)
+        array_not_nan = ~array_is_nan
+        kernel_not_nan = ~kernel_is_nan
+        assert np.all(x_copy[array_not_nan] == x[array_not_nan])
+        assert np.all(y_copy[kernel_not_nan] == y[kernel_not_nan])
+        assert np.all(np.isnan(x[array_is_nan]))
+        assert np.all(np.isnan(y[kernel_is_nan]))
+
+    @pytest.mark.parametrize(('dtype_array', 'dtype_kernel'), VALID_DTYPE_MATRIX)
     def test_dtype(self, dtype_array, dtype_kernel):
         '''
         Test that 32- and 64-bit floats are correctly handled
@@ -261,6 +337,22 @@ class TestConvolve1D:
 
         assert_array_almost_equal_nulp(z, np.array(rslt, dtype='>f8'), 10)
 
+    @pytest.mark.parametrize('preserve_nan', PRESERVE_NAN_OPTIONS)
+    def test_int_masked_array(self, preserve_nan):
+        """
+        Test that convolve works correctly with integer masked arrays.
+        """
+
+        x = ma.array([3, 5, 7, 11, 13], mask=[0, 0, 1, 0, 0], fill_value=0.)
+        y = np.array([1., 1., 1.], dtype='>f8')
+
+        z = convolve(x, y, preserve_nan=preserve_nan)
+
+        if preserve_nan:
+            assert np.isnan(z[2])
+            z[2] = 8
+
+        assert_array_almost_equal_nulp(z, (8/3., 4, 8, 12, 8), 10)
 
 class TestConvolve2D:
     def test_list(self):
@@ -276,7 +368,7 @@ class TestConvolve2D:
         z = convolve(x, x, boundary='fill', fill_value=1, normalize_kernel=False)
         assert_array_almost_equal_nulp(z, np.array(x, float)*9, 10)
 
-    @pytest.mark.parametrize(('dtype_array', 'dtype_kernel'), VALID_DTYPES)
+    @pytest.mark.parametrize(('dtype_array', 'dtype_kernel'), VALID_DTYPE_MATRIX)
     def test_dtype(self, dtype_array, dtype_kernel):
         '''
         Test that 32- and 64-bit floats are correctly handled
@@ -526,7 +618,7 @@ class TestConvolve3D:
         z = convolve(x, x, boundary='fill', fill_value=1, normalize_kernel=False)
         assert_array_almost_equal_nulp(z / 27, x, 10)
 
-    @pytest.mark.parametrize(('dtype_array', 'dtype_kernel'), VALID_DTYPES)
+    @pytest.mark.parametrize(('dtype_array', 'dtype_kernel'), VALID_DTYPE_MATRIX)
     def test_dtype(self, dtype_array, dtype_kernel):
         '''
         Test that 32- and 64-bit floats are correctly handled
@@ -805,3 +897,64 @@ def test_astropy_convolution_against_scipy():
                               convolve(y, x, normalize_kernel=False))
     assert_array_almost_equal(fftconvolve(y, x, 'same'),
                               convolve_fft(y, x, normalize_kernel=False))
+
+
+@pytest.mark.skipif('not HAS_PANDAS')
+def test_regression_6099():
+    wave = np.array((np.linspace(5000, 5100, 10)))
+    boxcar = 3
+    nonseries_result = convolve(wave, np.ones((boxcar,))/boxcar)
+
+    wave_series = pandas.Series(wave)
+    series_result  = convolve(wave_series, np.ones((boxcar,))/boxcar)
+
+    assert_array_almost_equal(nonseries_result, series_result)
+
+
+def test_invalid_array_convolve():
+    kernel = np.ones(3)/3.
+
+    with pytest.raises(TypeError):
+        convolve('glork', kernel)
+
+
+@pytest.mark.parametrize(('boundary'), BOUNDARY_OPTIONS)
+def test_non_square_kernel_asymmetric(boundary):
+    # Regression test for a bug that occurred when using non-square kernels in
+    # 2D when using boundary=None
+    kernel = np.array([[1, 2, 3, 2, 1], [0, 1, 2, 1, 0], [0, 0, 0, 0, 0]])
+    image = np.zeros((13, 13))
+    image[6, 6] = 1
+    result = convolve(image, kernel, normalize_kernel=False, boundary=boundary)
+    assert_allclose(result[5:8, 4:9], kernel)
+
+
+@pytest.mark.parametrize(('boundary', 'normalize_kernel'),
+                         itertools.product(BOUNDARY_OPTIONS,
+                                           NORMALIZE_OPTIONS))
+def test_uninterpolated_nan_regions(boundary, normalize_kernel):
+    #8086
+    # Test NaN interpolation of contiguous NaN regions with kernels of size
+    # identical and greater than that of the region of NaN values.
+
+    # Test case: kernel.shape == NaN_region.shape
+    kernel = Gaussian2DKernel(1, 5, 5)
+    nan_centroid = np.full(kernel.shape, np.nan)
+    image = np.pad(nan_centroid, pad_width=kernel.shape[0]*2, mode='constant',
+                   constant_values=1)
+    with pytest.warns(AstropyUserWarning,
+                      match="nan_treatment='interpolate', however, NaN values detected "
+                      "post convolution. A contiguous region of NaN values, larger "
+                      "than the kernel size, are present in the input array. "
+                      "Increase the kernel size to avoid this."):
+        result = convolve(image, kernel, boundary=boundary, nan_treatment='interpolate',
+                          normalize_kernel=normalize_kernel)
+        assert(np.any(np.isnan(result)))
+
+    # Test case: kernel.shape > NaN_region.shape
+    nan_centroid = np.full((kernel.shape[0]-1, kernel.shape[1]-1), np.nan) # 1 smaller than kerenel
+    image = np.pad(nan_centroid, pad_width=kernel.shape[0]*2, mode='constant',
+                   constant_values=1)
+    result = convolve(image, kernel, boundary=boundary, nan_treatment='interpolate',
+                      normalize_kernel=normalize_kernel)
+    assert(~np.any(np.isnan(result))) # Note: negation
