@@ -12,12 +12,13 @@ from ..kernels import Moffat2DKernel
 
 
 SHAPES_ODD = [[15, 15], [31, 31]]
-SHAPES_EVEN = [[8, 8], [16, 16], [32, 32]]
+SHAPES_EVEN = [[8, 8], [16, 16], [32, 32]]  # FIXME: not used ?!
+NOSHAPE = [[None, None]]
 WIDTHS = [2, 3, 4, 5]
 
 KERNELS = []
 
-for shape in SHAPES_ODD:
+for shape in SHAPES_ODD + NOSHAPE:
     for width in WIDTHS:
 
         KERNELS.append(Gaussian2DKernel(width,
@@ -55,7 +56,7 @@ class Test2DConvolutions:
         shape = kernel.array.shape
 
         x = np.zeros(shape)
-        xslice = [slice(sh // 2, sh // 2 + 1) for sh in shape]
+        xslice = tuple([slice(sh // 2, sh // 2 + 1) for sh in shape])
         x[xslice] = 1.0
 
         c2 = convolve_fft(x, kernel, boundary='fill')
@@ -94,7 +95,7 @@ class Test2DConvolutions:
         kernel = np.ones([width, width])
 
         x = np.zeros(shape)
-        xslice = [slice(sh // 2, sh // 2 + 1) for sh in shape]
+        xslice = tuple([slice(sh // 2, sh // 2 + 1) for sh in shape])
         x[xslice] = 1.0
 
         c2 = convolve_fft(x, kernel, boundary='fill')
@@ -114,7 +115,7 @@ class Test2DConvolutions:
         kernel2 = Box2DKernel(width, mode='oversample', factor=10)
 
         x = np.zeros(shape)
-        xslice = [slice(sh // 2, sh // 2 + 1) for sh in shape]
+        xslice = tuple([slice(sh // 2, sh // 2 + 1) for sh in shape])
         x[xslice] = 1.0
 
         c2 = convolve_fft(x, kernel2, boundary='fill')
