@@ -15,8 +15,7 @@ from ..kernels import (
 
 from ..utils import KernelSizeError
 from ...modeling.models import Box2D, Gaussian1D, Gaussian2D
-from ...utils.exceptions import AstropyDeprecationWarning
-from ...tests.helper import catch_warnings
+from ...utils.exceptions import AstropyDeprecationWarning, AstropyUserWarning
 
 try:
     from scipy.ndimage import filters
@@ -30,7 +29,8 @@ MODES = ['center', 'linear_interp', 'oversample', 'integrate']
 KERNEL_TYPES = [Gaussian1DKernel, Gaussian2DKernel,
                 Box1DKernel, Box2DKernel,
                 Trapezoid1DKernel, TrapezoidDisk2DKernel,
-                MexicanHat1DKernel, Tophat2DKernel, AiryDisk2DKernel, Ring2DKernel]
+                MexicanHat1DKernel, Tophat2DKernel, AiryDisk2DKernel,
+                Ring2DKernel]
 
 
 NUMS = [1, 1., np.float32(1.), np.float64(1.)]
@@ -178,7 +178,10 @@ class TestKernels:
         gauss_2 = Gaussian1DKernel(4)
         test_gauss_3 = Gaussian1DKernel(5)
 
-        gauss_3 = convolve(gauss_1, gauss_2)
+        with pytest.warns(AstropyUserWarning, match='Both array and kernel '
+                          'are Kernel instances'):
+            gauss_3 = convolve(gauss_1, gauss_2)
+
         assert np.all(np.abs((gauss_3 - test_gauss_3).array) < 0.01)
 
     def test_convolve_2D_kernels(self):
@@ -189,7 +192,10 @@ class TestKernels:
         gauss_2 = Gaussian2DKernel(4)
         test_gauss_3 = Gaussian2DKernel(5)
 
-        gauss_3 = convolve(gauss_1, gauss_2)
+        with pytest.warns(AstropyUserWarning, match='Both array and kernel '
+                          'are Kernel instances'):
+            gauss_3 = convolve(gauss_1, gauss_2)
+
         assert np.all(np.abs((gauss_3 - test_gauss_3).array) < 0.01)
 
     @pytest.mark.parametrize(('number'), NUMS)
@@ -316,8 +322,13 @@ class TestKernels:
         sums to zero.
         """
         array = [-2, -1, 0, 1, 2]
+
         custom = CustomKernel(array)
-        custom.normalize()
+
+        with pytest.warns(AstropyUserWarning, match='kernel cannot be '
+                          'normalized because it sums to zero'):
+            custom.normalize()
+
         assert custom.truncation == 0.
         assert custom._kernel_sum == 0.
 
@@ -327,8 +338,13 @@ class TestKernels:
         sums to zero.
         """
         array = [[0, -1, 0], [-1, 4, -1], [0, -1, 0]]
+
         custom = CustomKernel(array)
-        custom.normalize()
+
+        with pytest.warns(AstropyUserWarning, match='kernel cannot be '
+                          'normalized because it sums to zero'):
+            custom.normalize()
+
         assert custom.truncation == 0.
         assert custom._kernel_sum == 0.
 
@@ -371,8 +387,8 @@ class TestKernels:
                [1 / 9., 1 + 1 / 9., 1 / 9.],
                [1 / 9., 1 / 9., 1 / 9.]]
         ref_1 = [[1 / 9., 1 / 9., 1 / 9.],
-               [1 / 9., 1 / 9., 1 / 9.],
-               [1 / 9., 1 / 9., 1 / 9.]]
+                 [1 / 9., 1 / 9., 1 / 9.],
+                 [1 / 9., 1 / 9., 1 / 9.]]
         assert_almost_equal(box_2.array, [[1]], decimal=12)
         assert_almost_equal(box_1.array, ref_1, decimal=12)
         assert_almost_equal(box_sum_1.array, ref, decimal=12)
@@ -394,7 +410,7 @@ class TestKernels:
 
     # https://github.com/astropy/astropy/issues/3605
     def test_Gaussian2DKernel_rotated(self):
-        with catch_warnings(AstropyDeprecationWarning) as w:
+        with pytest.warns(AstropyDeprecationWarning) as w:
             Gaussian2DKernel(stddev=10)
         assert len(w) == 1
 
