@@ -1,5 +1,5 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
+import ctypes
 import numpy as np
 
 from ..modeling.core import FittableModel, custom_model
@@ -18,6 +18,14 @@ class KernelSizeError(Exception):
     Called when size of kernels is even.
     """
 
+def has_even_axis(array):
+    if isinstance(array, (list, tuple)):
+        return not len(array) % 2
+    else:
+        return any(not axes_size % 2 for axes_size in array.shape)
+
+def raise_even_kernel_exception():
+    raise KernelSizeError("Kernel size must be odd in all axes.")
 
 def add_kernel_arrays_1D(array_1, array_2):
     """
