@@ -186,6 +186,10 @@ class TimeAttribute(Attribute):
                                                                value, err))
             converted = True
 
+        # Set attribute as read-only for arrays (not allowed by numpy
+        # for array scalars)
+        if out.shape:
+            out.writeable = False
         return out, converted
 
 
@@ -314,6 +318,7 @@ class QuantityAttribute(Attribute):
             converted = oldvalue is not value
             return value, converted
 
+
 class EarthLocationAttribute(Attribute):
     """
     A frame attribute that can act as a `~astropy.coordinates.EarthLocation`.
@@ -494,6 +499,7 @@ class FrameAttribute(Attribute):
                       AstropyDeprecationWarning)
         super().__init__(*args, **kwargs)
 
+
 class TimeFrameAttribute(TimeAttribute):
 
     def __init__(self, *args, **kwargs):
@@ -501,6 +507,7 @@ class TimeFrameAttribute(TimeAttribute):
                       AstropyDeprecationWarning)
         super().__init__(*args, **kwargs)
 
+
 class QuantityFrameAttribute(QuantityAttribute):
 
     def __init__(self, *args, **kwargs):
@@ -508,6 +515,7 @@ class QuantityFrameAttribute(QuantityAttribute):
                       "QuantityAttribute.", AstropyDeprecationWarning)
         super().__init__(*args, **kwargs)
 
+
 class CartesianRepresentationFrameAttribute(CartesianRepresentationAttribute):
 
     def __init__(self, *args, **kwargs):
