@@ -121,17 +121,16 @@ def icrs_to_gcrs(icrs_coo, gcrs_frame):
     # get the position and velocity arrays for the observatory.  Need to
     # have xyz in last dimension, and pos/vel in one-but-last.
     # (Note could use np.stack once our minimum numpy version is >=1.10.)
-    pv = np.concatenate(
-        (gcrs_frame.obsgeoloc.get_xyz(xyz_axis=-1).value[..., np.newaxis, :],
-         gcrs_frame.obsgeovel.get_xyz(xyz_axis=-1).value[..., np.newaxis, :]),
-        axis=-2)
+    obs_pv = erfa.pav2pv(
+        gcrs_frame.obsgeoloc.get_xyz(xyz_axis=-1).to_value(u.m),
+        gcrs_frame.obsgeovel.get_xyz(xyz_axis=-1).to_value(u.m/u.s))
 
     # find the position and velocity of earth
     jd1, jd2 = get_jd12(gcrs_frame.obstime, 'tdb')
     earth_pv, earth_heliocentric = prepare_earth_position_vel(gcrs_frame.obstime)
 
     # get astrometry context object, astrom.
-    astrom = erfa.apcs(jd1, jd2, pv, earth_pv, earth_heliocentric)
+    astrom = erfa.apcs(jd1, jd2, obs_pv, earth_pv, earth_heliocentric)
 
     if icrs_coo.data.get_name() == 'unitspherical' or icrs_coo.data.to_cartesian().x.unit == u.one:
         # if no distance, just do the infinite-distance/no parallax calculation
@@ -173,15 +172,14 @@ def gcrs_to_icrs(gcrs_coo, icrs_frame):
 
     # set up the astrometry context for ICRS<->GCRS and then convert to BCRS
     # coordinate direction
-    pv = np.concatenate(
-        (gcrs_coo.obsgeoloc.get_xyz(xyz_axis=-1).value[..., np.newaxis, :],
-         gcrs_coo.obsgeovel.get_xyz(xyz_axis=-1).value[..., np.newaxis, :]),
-        axis=-2)
+    obs_pv = erfa.pav2pv(
+        gcrs_coo.obsgeoloc.get_xyz(xyz_axis=-1).to_value(u.m),
+        gcrs_coo.obsgeovel.get_xyz(xyz_axis=-1).to_value(u.m/u.s))
 
     jd1, jd2 = get_jd12(gcrs_coo.obstime, 'tdb')
 
     earth_pv, earth_heliocentric = prepare_earth_position_vel(gcrs_coo.obstime)
-    astrom = erfa.apcs(jd1, jd2, pv, earth_pv, earth_heliocentric)
+    astrom = erfa.apcs(jd1, jd2, obs_pv, earth_pv, earth_heliocentric)
 
     i_ra, i_dec = aticq(gcrs_ra, gcrs_dec, astrom)
 
@@ -235,14 +233,13 @@ def gcrs_to_hcrs(gcrs_coo, hcrs_frame):
 
     # set up the astrometry context for ICRS<->GCRS and then convert to ICRS
     # coordinate direction
-    pv = np.concatenate(
-        (gcrs_coo.obsgeoloc.get_xyz(xyz_axis=-1).value[..., np.newaxis, :],
-         gcrs_coo.obsgeovel.get_xyz(xyz_axis=-1).value[..., np.newaxis, :]),
-        axis=-2)
+    obs_pv = erfa.pav2pv(
+        gcrs_coo.obsgeoloc.get_xyz(xyz_axis=-1).to_value(u.m),
+        gcrs_coo.obsgeovel.get_xyz(xyz_axis=-1).to_value(u.m/u.s))
 
     jd1, jd2 = get_jd12(hcrs_frame.obstime, 'tdb')
     earth_pv, earth_heliocentric = prepare_earth_position_vel(gcrs_coo.obstime)
-    astrom = erfa.apcs(jd1, jd2, pv, earth_pv, earth_heliocentric)
+    astrom = erfa.apcs(jd1, jd2, obs_pv, earth_pv, earth_heliocentric)
 
     i_ra, i_dec = aticq(gcrs_ra, gcrs_dec, astrom)
 
