@@ -13,7 +13,6 @@ from .. import units as u
 from .. import constants as consts
 from ..units.quantity import QuantityInfoBase
 from ..utils.exceptions import AstropyUserWarning
-from ..utils.compat.numpycompat import NUMPY_LT_1_12
 from .angles import Longitude, Latitude
 from .representation import CartesianRepresentation, CartesianDifferential
 from .errors import UnknownSiteException
@@ -51,7 +50,8 @@ def _check_ellipsoid(ellipsoid=None, default='WGS84'):
     return ellipsoid
 
 
-def _get_json_result(url, err_str):
+def _get_json_result(url, err_str, use_google):
+
     # need to do this here to prevent a series of complicated circular imports
     from .name_resolve import NameResolveError
     try:
@@ -73,14 +73,19 @@ def _get_json_result(url, err_str):
         # working request
         raise NameResolveError(err_str.format(msg="connection timed out"))
 
-    results = resp_data.get('results', [])
+    if use_google:
+        results = resp_data.get('results', [])
+
+        if resp_data.get('status', None) != 'OK':
+            raise NameResolveError(err_str.format(msg="unknown failure with "
+                                                  "Google API"))
+
+    else: # OpenStreetMap returns a list
+        results = resp_data
 
     if not results:
         raise NameResolveError(err_str.format(msg="no results returned"))
 
-    if resp_data.get('status', None) != 'OK':
-        raise NameResolveError(err_str.format(msg="unknown failure with Google maps API"))
-
     return results
 
 
@@ -286,15 +291,13 @@ class EarthLocation(u.Quantity):
         # don't convert to m by default, so we can use the height unit below.
         if not isinstance(height, u.Quantity):
             height = u.Quantity(height, u.m, copy=False)
-        # convert to float in units required for erfa routine, and ensure
-        # all broadcast to same shape, and are at least 1-dimensional.
-        _lon, _lat, _height = np.broadcast_arrays(lon.to_value(u.radian),
-                                                  lat.to_value(u.radian),
-                                                  height.to_value(u.m))
         # get geocentric coordinates. Have to give one-dimensional array.
-        xyz = erfa.gd2gc(getattr(erfa, ellipsoid), _lon.ravel(),
-                                 _lat.ravel(), _height.ravel())
-        self = xyz.view(cls._location_dtype, cls).reshape(_lon.shape)
+        xyz = erfa.gd2gc(getattr(erfa, ellipsoid),
+                         lon.to_value(u.radian),
+                         lat.to_value(u.radian),
+                         height.to_value(u.m))
+        self = xyz.ravel().view(cls._location_dtype,
+                                cls).reshape(xyz.shape[:-1])
         self._unit = u.meter
         self._ellipsoid = ellipsoid
         return self.to(height.unit)
@@ -308,6 +311,9 @@ class EarthLocation(u.Quantity):
         information, not a fully-featured exhaustive registry of observatories
         and all their properties.
 
+        Additional information about the site is stored in the ``.info.meta``
+        dictionary of sites obtained using this method (see the examples below).
+
         .. note::
             When this function is called, it will attempt to download site
             information from the astropy data server. If you would like a site
@@ -331,6 +337,16 @@ class EarthLocation(u.Quantity):
         site : This class (a `~astropy.coordinates.EarthLocation` or subclass)
             The location of the observatory.
 
+        Examples
+        --------
+
+        >>> from astropy.coordinates import EarthLocation
+        >>> keck = EarthLocation.of_site('Keck Observatory')  # doctest: +REMOTE_DATA
+        >>> keck.geodetic  # doctest: +REMOTE_DATA +FLOAT_CMP
+        GeodeticLocation(lon=<Longitude -155.47833333 deg>, lat=<Latitude 19.82833333 deg>, height=<Quantity 4160. m>)
+        >>> keck.info.meta  # doctest: +REMOTE_DATA
+        {'source': 'IRAF Observatory Database', 'timezone': 'US/Aleutian'}
+
         See Also
         --------
         get_site_names : the list of sites that this function can access
@@ -349,31 +365,41 @@ class EarthLocation(u.Quantity):
             return newel
 
     @classmethod
-    def of_address(cls, address, get_height=False):
+    def of_address(cls, address, get_height=False, google_api_key=None):
         """
-        Return an object of this class for a given address by querying the Google
-        maps geocoding API.
+        Return an object of this class for a given address by querying either
+        the OpenStreetMap Nominatim tool [1]_ (default) or the Google geocoding
+        API [2]_, which requires a specified API key.
 
-        This is intended as a quick convenience function to get fast access to
-        locations. In the background, this just issues a query to the Google maps
-        geocoding API. It is not meant to be abused! Google uses IP-based query
-        limiting and will ban your IP if you send more than a few thousand queries
-        per hour [1]_.
+        This is intended as a quick convenience function to get easy access to
+        locations. If you need to specify a precise location, you should use the
+        initializer directly and pass in a longitude, latitude, and elevation.
+
+        In the background, this just issues a web query to either of
+        the APIs noted above. This is not meant to be abused! Both
+        OpenStreetMap and Google use IP-based query limiting and will ban your
+        IP if you send more than a few thousand queries per hour [2]_.
 
         .. warning::
             If the query returns more than one location (e.g., searching on
-            ``address='springfield'``), this function will use the **first** returned
-            location.
+            ``address='springfield'``), this function will use the **first**
+            returned location.
 
         Parameters
         ----------
         address : str
-            The address to get the location for. As per the Google maps API, this
-            can be a fully specified street address (e.g., 123 Main St., New York,
-            NY) or a city name (e.g., Danbury, CT), or etc.
+            The address to get the location for. As per the Google maps API,
+            this can be a fully specified street address (e.g., 123 Main St.,
+            New York, NY) or a city name (e.g., Danbury, CT), or etc.
         get_height : bool (optional)
-            Use the retrieved location to perform a second query to the Google maps
-            elevation API to retrieve the height of the input address [2]_.
+            This only works when using the Google API! See the ``google_api_key``
+            block below. Use the retrieved location to perform a second query to
+            the Google maps elevation API to retrieve the height of the input
+            address [3]_.
+        google_api_key : str (optional)
+            A Google API key with the Geocoding API and (optionally) the
+            elevation API enabled. See [4]_ for more information.
+
 
         Returns
         -------
@@ -382,37 +408,68 @@ class EarthLocation(u.Quantity):
 
         References
         ----------
-        .. [1] https://developers.google.com/maps/documentation/geocoding/intro
-        .. [2] https://developers.google.com/maps/documentation/elevation/intro
+        .. [1] https://nominatim.openstreetmap.org/
+        .. [2] https://developers.google.com/maps/documentation/geocoding/start
+        .. [3] https://developers.google.com/maps/documentation/elevation/
+        .. [4] https://developers.google.com/maps/documentation/geocoding/get-api-key
 
         """
 
-        pars = urllib.parse.urlencode({'address': address})
-        geo_url = "https://maps.googleapis.com/maps/api/geocode/json?{0}".format(pars)
+        use_google = google_api_key is not None
+
+        # Fail fast if invalid options are passed:
+        if not use_google and get_height:
+            raise ValueError('Currently, `get_height` only works when using '
+                             'the Google geocoding API, which requires passing '
+                             'a Google API key with `google_api_key`. See: '
+                             'https://developers.google.com/maps/documentation/geocoding/get-api-key '
+                             'for information on obtaining an API key.')
+
+        if use_google: # Google
+            pars = urllib.parse.urlencode({'address': address,
+                                           'key': google_api_key})
+            geo_url = ("https://maps.googleapis.com/maps/api/geocode/json?{0}"
+                       .format(pars))
+
+        else: # OpenStreetMap
+            pars = urllib.parse.urlencode({'q': address,
+                                           'format': 'json'})
+            geo_url = ("https://nominatim.openstreetmap.org/search?{0}"
+                       .format(pars))
 
         # get longitude and latitude location
-        err_str = ("Unable to retrieve coordinates for address '{address}'; {{msg}}"
-                   .format(address=address))
-        geo_result = _get_json_result(geo_url, err_str=err_str)
-        loc = geo_result[0]['geometry']['location']
+        err_str = ("Unable to retrieve coordinates for address '{address}'; "
+                   "{{msg}}".format(address=address))
+        geo_result = _get_json_result(geo_url, err_str=err_str,
+                                      use_google=use_google)
+
+        if use_google:
+            loc = geo_result[0]['geometry']['location']
+            lat = loc['lat']
+            lon = loc['lng']
+
+        else:
+            loc = geo_result[0]
+            lat = float(loc['lat']) # strings are returned by OpenStreetMap
+            lon = float(loc['lon'])
 
         if get_height:
-            pars = {'locations': '{lat:.8f},{lng:.8f}'.format(lat=loc['lat'],
-                                                              lng=loc['lng'])}
+            pars = {'locations': '{lat:.8f},{lng:.8f}'.format(lat=lat, lng=lon),
+                    'key': google_api_key}
             pars = urllib.parse.urlencode(pars)
-            ele_url = "https://maps.googleapis.com/maps/api/elevation/json?{0}".format(pars)
+            ele_url = ("https://maps.googleapis.com/maps/api/elevation/json?{0}"
+                       .format(pars))
 
-            err_str = ("Unable to retrieve elevation for address '{address}'; {{msg}}"
-                       .format(address=address))
-            ele_result = _get_json_result(ele_url, err_str=err_str)
+            err_str = ("Unable to retrieve elevation for address '{address}'; "
+                       "{{msg}}".format(address=address))
+            ele_result = _get_json_result(ele_url, err_str=err_str,
+                                          use_google=use_google)
             height = ele_result[0]['elevation']*u.meter
 
         else:
             height = 0.
 
-        return cls.from_geodetic(lon=loc['lng']*u.degree,
-                                 lat=loc['lat']*u.degree,
-                                 height=height)
+        return cls.from_geodetic(lon=lon*u.deg, lat=lat*u.deg, height=height)
 
     @classmethod
     def get_site_names(cls):
@@ -762,14 +819,6 @@ class EarthLocation(u.Quantity):
         new_array = self.unit.to(unit, array_view, equivalencies=equivalencies)
         return new_array.view(self.dtype).reshape(self.shape)
 
-    if NUMPY_LT_1_12:
-        def __repr__(self):
-            # Use the numpy >=1.12 way to format structured arrays.
-            from .representation import _array2string
-            prefixstr = '<' + self.__class__.__name__ + ' '
-            arrstr = _array2string(self.view(np.ndarray), prefix=prefixstr)
-            return '{0}{1}{2:s}>'.format(prefixstr, arrstr, self._unitstr)
-
 
 # need to do this here at the bottom to avoid circular dependencies
 from .sites import get_builtin_sites, get_downloaded_sites
