@@ -1,7 +1,5 @@
 # -*- coding: utf-8 -*-
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-
 """
 This includes tests for the Distance class and related calculations
 """
@@ -14,6 +12,8 @@ from ... import units as u
 from ...units import allclose as quantity_allclose
 from .. import Longitude, Latitude, Distance, CartesianRepresentation
 from ..builtin_frames import ICRS, Galactic
+from ...tests.helper import catch_warnings
+from ...utils.exceptions import AstropyWarning
 
 try:
     import scipy  # pylint: disable=W0611
@@ -127,10 +127,10 @@ def test_distances_scipy():
     npt.assert_allclose(d6.value, 3.5417046898762366e+22)
 
     with pytest.raises(ValueError):
-        d7 = Distance(cosmology=WMAP5, unit=u.km)
+        Distance(cosmology=WMAP5, unit=u.km)
 
     with pytest.raises(ValueError):
-        d8 = Distance()
+        Distance()
 
 
 def test_distance_change():
@@ -226,6 +226,22 @@ def test_parallax():
     assert quantity_allclose(d.pc, [1000., 100., 10.])
     assert quantity_allclose(plx, d.parallax)
 
+    # check behavior for negative parallax
+    with pytest.raises(ValueError):
+        Distance(parallax=-1 * u.mas)
+
+    with pytest.raises(ValueError):
+        Distance(parallax=[10, 1, -1] * u.mas)
+
+    with catch_warnings(AstropyWarning) as w:
+        Distance(parallax=-1 * u.mas, allow_negative=True)
+    assert len(w) > 0
+
+    with catch_warnings(AstropyWarning) as w:
+        Distance(parallax=[10, 1, -1] * u.mas, allow_negative=True)
+    assert len(w) > 0
+
+
 def test_distance_in_coordinates():
     """
     test that distances can be created from quantities and that cartesian
