@@ -41,8 +41,8 @@ def test_faux_lsr(dt, symmetric):
                                      LSR2, ICRS, finite_difference_dt=dt,
                                      symmetric_finite_difference=symmetric)
     def lsr_to_icrs(lsr_coo, icrs_frame):
-        dt = lsr_frame.obstime - J2000
-        offset = lsr_frame.v_bary * dt.to(u.second)
+        dt = lsr_coo.obstime - J2000
+        offset = lsr_coo.v_bary * dt.to(u.second)
         return icrs_frame.realize_frame(lsr_coo.data - offset)
 
     ic = ICRS(ra=12.3*u.deg, dec=45.6*u.deg, distance=7.8*u.au,
@@ -59,13 +59,17 @@ def test_faux_lsr(dt, symmetric):
     assert quantity_allclose(totchange, np.sum(lsrc.v_bary.d_xyz**2)**0.5)
 
     ic2 = ICRS(ra=120.3*u.deg, dec=45.6*u.deg, distance=7.8*u.au,
-              pm_ra_cosdec=0*u.marcsec/u.yr, pm_dec=10*u.marcsec/u.yr,
-              radial_velocity=1000*u.km/u.s)
+               pm_ra_cosdec=0*u.marcsec/u.yr, pm_dec=10*u.marcsec/u.yr,
+               radial_velocity=1000*u.km/u.s)
     lsrc2 = ic2.transform_to(LSR2())
+    ic2_roundtrip = lsrc2.transform_to(ICRS)
 
     tot = np.sum(lsrc2.cartesian.differentials['s'].d_xyz**2)**0.5
     assert np.abs(tot.to('km/s') - 1000*u.km/u.s) < 20*u.km/u.s
 
+    assert quantity_allclose(ic2.cartesian.xyz,
+                             ic2_roundtrip.cartesian.xyz)
+
 
 def test_faux_fk5_galactic():
 
