@@ -148,8 +148,19 @@ def test_header_from_readme():
                 assert val == Q[i]
 
 
+def test_cds_units():
+    from .... import units
+    data_and_readme = 't/cds.dat'
+    reader = ascii.get_reader(ascii.Cds)
+    table = reader.read(data_and_readme)
+    # column unit is GMsun (giga solar masses)
+    # make sure this is parsed correctly, not as a "string" unit
+    assert table['Fit'].to(units.solMass).unit == units.solMass
+
+
 if __name__ == "__main__":  # run from main directory; not from test/
     test_header_from_readme()
     test_multi_header()
     test_glob_header()
     test_description()
+    test_cds_units()
