@@ -34,6 +34,8 @@ except ImportError:
 else:
     HAS_BZ2 = True
 
+asciiIO = lambda x: BytesIO(x.encode('ascii'))
+
 
 @pytest.mark.parametrize('fast_reader', [True, False, {'use_fast_converter': False},
                                          {'use_fast_converter': True}, 'force'])
@@ -105,6 +107,24 @@ def test_guess_with_format_arg():
     assert dat.colnames == ['a', 'b']
 
 
+def test_guess_with_delimiter_arg():
+    """
+    When the delimiter is explicitly given then do not try others in guessing.
+    """
+    fields = ['10.1E+19', '3.14', '2048', '-23']
+    values = [1.01e20, 3.14, 2048, -23]
+
+    # Default guess should recognise CSV with optional spaces
+    t0 = ascii.read(asciiIO(', '.join(fields)), guess=True)
+    for n, v in zip(t0.colnames, values):
+        assert t0[n][0] == v
+
+    # Forcing space as delimiter produces type str columns ('10.1E+19,')
+    t1 = ascii.read(asciiIO(', '.join(fields)), guess=True, delimiter=' ')
+    for n, v in zip(t1.colnames[:-1], fields[:-1]):
+        assert t1[n][0] == v+','
+
+
 def test_reading_mixed_delimiter_tabs_spaces():
     # Regression test for https://github.com/astropy/astropy/issues/6770
     dat = ascii.read('1 2\t3\n1 2\t3', format='no_header', names=list('abc'))
@@ -120,7 +140,8 @@ def test_read_with_names_arg(fast_reader):
     """
     Test that a bad value of `names` raises an exception.
     """
-    with pytest.raises(ValueError):
+    # CParser only uses columns in `names` and thus reports mismach in num_col
+    with pytest.raises(ascii.InconsistentTableError):
         ascii.read(['c d', 'e f'], names=('a', ), guess=False, fast_reader=fast_reader)
 
 
@@ -135,8 +156,8 @@ def test_read_all_files(fast_reader):
             test_opts = testfile['opts'].copy()
             if 'guess' not in test_opts:
                 test_opts['guess'] = guess
-            if 'Reader' in test_opts and 'fast_{0}'.format(test_opts['Reader']._format_name) \
-                in core.FAST_CLASSES:  # has fast version
+            if ('Reader' in test_opts and 'fast_{0}'.format(test_opts['Reader']._format_name)
+                    in core.FAST_CLASSES):  # has fast version
                 if 'Inputter' not in test_opts:  # fast reader doesn't allow this
                     test_opts['fast_reader'] = fast_reader
             table = ascii.read(testfile['name'], **test_opts)
@@ -282,7 +303,7 @@ def test_set_include_names(fast_reader):
     names = ('c1', 'c2', 'c3', 'c4', 'c5', 'c6')
     include_names = ('c1', 'c3')
     data = ascii.read('t/simple3.txt', names=names, include_names=include_names,
-                           delimiter='|', fast_reader=fast_reader)
+                      delimiter='|', fast_reader=fast_reader)
     assert_equal(data.dtype.names, include_names)
 
 
@@ -428,7 +449,7 @@ def test_fill_values_include_names(fast_reader):
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
     data = ascii.read(f, fill_values=('a', '1'), fast_reader=fast_reader,
-                           fill_include_names=['b'], **testfile['opts'])
+                      fill_include_names=['b'], **testfile['opts'])
     check_fill_values(data)
 
 
@@ -437,7 +458,7 @@ def test_fill_values_exclude_names(fast_reader):
     f = 't/fill_values.txt'
     testfile = get_testfiles(f)
     data = ascii.read(f, fill_values=('a', '1'), fast_reader=fast_reader,
-                           fill_exclude_names=['a'], **testfile['opts'])
+                      fill_exclude_names=['a'], **testfile['opts'])
     check_fill_values(data)
 
 
@@ -503,7 +524,7 @@ def test_Ipac_meta():
 def test_set_guess_kwarg():
     """Read a file using guess with one of the typical guess_kwargs explicitly set."""
     data = ascii.read('t/space_delim_no_header.dat',
-                           delimiter=',', guess=True)
+                      delimiter=',', guess=True)
     assert(data.dtype.names == ('1 3.4 hello',))
     assert(len(data) == 1)
 
@@ -832,7 +853,8 @@ def test_header_start_exception():
     This was implemented in response to issue #885.
     '''
     for readerclass in [ascii.NoHeader, ascii.SExtractor, ascii.Ipac,
-                   ascii.BaseReader, ascii.FixedWidthNoHeader, ascii.Cds, ascii.Daophot]:
+                        ascii.BaseReader, ascii.FixedWidthNoHeader,
+                        ascii.Cds, ascii.Daophot]:
         with pytest.raises(ValueError):
             reader = ascii.core._get_reader(readerclass, header_start=5)
 
@@ -865,8 +887,8 @@ def test_sextractor_units():
     """
     table = ascii.read('t/sextractor2.dat', Reader=ascii.SExtractor, guess=False)
     expected_units = [None, Unit('pix'), Unit('pix'), Unit('mag'),
-                Unit('mag'), None, Unit('pix**2'), Unit('m**(-6)'),
-                Unit('mag * arcsec**(-2)')]
+                      Unit('mag'), None, Unit('pix**2'), Unit('m**(-6)'),
+                      Unit('mag * arcsec**(-2)')]
     expected_descrs = ['Running object number',
                        'Windowed position estimate along x',
                        'Windowed position estimate along y',
@@ -1073,7 +1095,7 @@ def test_probably_html():
                   'junk < table baz> <tr foo > <td bar> </td> </tr> </table> junk',
                   ['junk < table baz>', ' <tr foo >', ' <td bar> ', '</td> </tr>', '</table> junk'],
                   (' <! doctype html > ', ' hello world'),
-    ):
+                   ):
         assert _probably_html(table) is True
 
     for table in ('t/html.htms',
@@ -1085,7 +1107,7 @@ def test_probably_html():
                   ['junk < table baz>', ' <t foo >', ' <td bar> ', '</td> </tr>', '</table> junk'],
                   (' <! doctype htm > ', ' hello world'),
                   [[1, 2, 3]],
-    ):
+                   ):
         assert _probably_html(table) is False
 
 
@@ -1099,8 +1121,8 @@ def test_data_header_start(fast_reader):
                '1 2'],  # line 2
               [{'header_start': 1},
                {'header_start': 1, 'data_start': 2}
-           ]
-           ),
+               ]
+               ),
 
              (['# comment',
                '',
@@ -1409,3 +1431,16 @@ def test_read_non_ascii():
     table = Table.read(['col1, col2', '\u2119, \u01b4', '1, 2'], format='csv')
     assert np.all(table['col1'] == ['\u2119', '1'])
     assert np.all(table['col2'] == ['\u01b4', '2'])
+
+
+@pytest.mark.parametrize('enable', [True, False, 'force'])
+def test_kwargs_dict_guess(enable):
+    """Test that fast_reader dictionary is preserved through guessing sequence.
+    """
+    # Fails for enable=(True, 'force') - #5578
+    ascii.read('a\tb\n 1\t2\n3\t 4.0', fast_reader=dict(enable=enable))
+    assert get_read_trace()[-1]['kwargs']['Reader'] is (
+        ascii.Tab if (enable is False) else ascii.FastTab)
+    for k in get_read_trace():
+        if not k.get('status', 'Disabled').startswith('Disabled'):
+            assert k.get('kwargs').get('fast_reader').get('enable') is enable
