@@ -69,9 +69,6 @@ from .hdu.image import PrimaryHDU, ImageHDU
 from .hdu.table import BinTableHDU
 from .header import Header
 from .util import fileobj_closed, fileobj_name, fileobj_mode, _is_int
-from ...units import Unit
-from ...units.format.fits import UnitScaleError
-from ...units import Quantity
 from ...utils.exceptions import AstropyUserWarning
 from ...utils.decorators import deprecated_renamed_argument
 
@@ -444,6 +441,7 @@ def table_to_hdu(table, character_as_bytes=False):
     """
     # Avoid circular imports
     from .connect import is_column_keyword, REMOVE_KEYWORDS
+    from .column import python_to_tdisp
 
     # Header to store Time related metadata
     hdr = None
@@ -452,8 +450,9 @@ def table_to_hdu(table, character_as_bytes=False):
     if table.has_mixin_columns:
         # Import is done here, in order to avoid it at build time as erfa is not
         # yet available then.
-        from ...table.column import BaseColumn, Column
+        from ...table.column import BaseColumn
         from ...time import Time
+        from ...units import Quantity
         from .fitstime import time_to_fits
 
         # Only those columns which are instances of BaseColumn, Quantity or Time can
@@ -497,10 +496,24 @@ def table_to_hdu(table, character_as_bytes=False):
     else:
         table_hdu = BinTableHDU.from_columns(np.array(table.filled()), header=hdr, character_as_bytes=character_as_bytes)
 
-    # Set units for output HDU
+    # Set units and format display for output HDU
     for col in table_hdu.columns:
+
+        if table[col.name].info.format is not None:
+            # check for boolean types, special format case
+            logical = table[col.name].info.dtype == bool
+
+            tdisp_format = python_to_tdisp(table[col.name].info.format,
+                                           logical_dtype=logical)
+            if tdisp_format is not None:
+                col.disp = tdisp_format
+
         unit = table[col.name].unit
         if unit is not None:
+            # Local imports to avoid importing units when it is not required,
+            # e.g. for command-line scripts
+            from ...units import Unit
+            from ...units.format.fits import UnitScaleError
             try:
                 col.unit = unit.to_string(format='fits')
             except UnitScaleError:
