@@ -450,6 +450,11 @@ def fits_to_time(hdr, table):
             time_columns[int(idx)][base] = value
             hcopy.remove(key)
 
+        elif (value in ('OBSGEO-X', 'OBSGEO-Y', 'OBSGEO-Z') and
+              re.match('TTYPE[0-9]+', key)):
+
+            global_info[value] = table[value]
+
     # Verify and get the global time reference frame information
     _verify_global_info(global_info)
     _convert_global_time(table, global_info)
@@ -550,14 +555,17 @@ def time_to_fits(table):
                     'Earth Location "TOPOCENTER" for Time Column "{}" is incompatabile '
                     'with scale "{}".'.format(col.info.name, col.scale.upper()),
                     AstropyUserWarning)
-            if col.location.size > 1:
-                raise ValueError('Vectorized Location of Time Column "{}" cannot be '
-                                 'written, as it is not supported.'.format(col.info.name))
+
             if location is None:
                 # Set global geocentric location
                 location = col.location
-                hdr.extend([Card(keyword='OBSGEO-{}'.format(dim.upper()),
-                                 value=getattr(location, dim).to_value(u.m))
+                if location.size > 1:
+                    for dim in ('x', 'y', 'z'):
+                        newtable.add_column(Column(getattr(location, dim).to_value(u.m)),
+                                            name='OBSGEO-{}'.format(dim.upper()))
+                else:
+                    hdr.extend([Card(keyword='OBSGEO-{}'.format(dim.upper()),
+                                     value=getattr(location, dim).to_value(u.m))
                             for dim in ('x', 'y', 'z')])
             elif location != col.location:
                 raise ValueError('Multiple Time Columns with different geocentric '
