@@ -18,18 +18,17 @@ from .base import DELAYED, _ValidHDU, ExtensionHDU
 # This module may have many dependencies on astropy.io.fits.column, but
 # astropy.io.fits.column has fewer dependencies overall, so it's easier to
 # keep table/column-related utilities in astropy.io.fits.column
-from .. import _numpy_hacks as nh
 from ..column import (FITS2NUMPY, KEYWORD_NAMES, KEYWORD_TO_ATTRIBUTE,
                       ATTRIBUTE_TO_KEYWORD, TDEF_RE, Column, ColDefs,
                       _AsciiColDefs, _FormatP, _FormatQ, _makep,
                       _parse_tformat, _scalar_to_format, _convert_format,
-                      _cmp_recformats, _get_index)
+                      _cmp_recformats)
 from ..fitsrec import FITS_rec, _get_recarray_field, _has_unicode_fields
 from ..header import Header, _pad_length
 from ..util import _is_int, _str_to_num
 
 from ....utils import lazyproperty
-from ....utils.exceptions import AstropyUserWarning, AstropyDeprecationWarning
+from ....utils.exceptions import AstropyDeprecationWarning
 from ....utils.decorators import deprecated_renamed_argument
 
 
@@ -1088,9 +1087,6 @@ class BinTableHDU(_TableBaseHDU):
             if isinstance(f, str):
                 if os.path.exists(f) and os.path.getsize(f) != 0:
                     if overwrite:
-                        warnings.warn(
-                            "Overwriting existing file '{}'.".format(f),
-                            AstropyUserWarning)
                         os.remove(f)
                     else:
                         exist.append(f)
@@ -1152,7 +1148,7 @@ class BinTableHDU(_TableBaseHDU):
         header : Header object
             When the cdfile and hfile are missing, use this Header object in
             the creation of the new table and HDU.  Otherwise this Header
-            supercedes the keywords from hfile, which is only used to update
+            supersedes the keywords from hfile, which is only used to update
             values not present in this Header, unless ``replace=True`` in which
             this Header's values are completely replaced with the values from
             hfile.
@@ -1530,10 +1526,9 @@ def _binary_table_byte_swap(data):
     for arr in reversed(to_swap):
         arr.byteswap(True)
 
-    new_dtype = nh.realign_dtype(np.dtype(list(zip(names, formats))),
-                                 offsets)
-
-    data.dtype = new_dtype
+    data.dtype = np.dtype({'names': names,
+                           'formats': formats,
+                           'offsets': offsets})
 
     yield data
 
