@@ -90,6 +90,8 @@
 
 /* Include the Python C API */
 
+#include <float.h>
+#include <limits.h>
 #include <math.h>
 #include <string.h>
 
@@ -99,17 +101,6 @@
 #include <fitsio2.h>
 #include "compressionmodule.h"
 
-/* Some defines for Python3 support--bytes objects should be used where */
-/* strings were previously used                                         */
-#if PY_MAJOR_VERSION >= 3
-#define IS_PY3K
-#endif
-
-#ifdef IS_PY3K
-#define PyString_FromString PyUnicode_FromString
-#define PyInt_AsLong PyLong_AsLong
-#endif
-
 
 /* These defaults mirror the defaults in astropy.io.fits.hdu.compressed */
 #define DEFAULT_COMPRESSION_TYPE "RICE_1"
@@ -119,6 +110,13 @@
 #define DEFAULT_BLOCK_SIZE 32
 #define DEFAULT_BYTE_PIX 4
 
+/* Flags to pass to get_header_* functions to control error messages. */
+typedef enum {
+    HDR_NOFLAG = 0,
+    HDR_FAIL_KEY_MISSING = 1 << 0,
+    HDR_FAIL_VAL_NEGATIVE = 1 << 1,
+} HeaderGetFlags;
+
 
 /* Report any error based on the status returned from cfitsio. */
 void process_status_err(int status)
@@ -130,8 +128,7 @@ void process_status_err(int status)
    err_msg[0] = '\0';
    def_err_msg[0] = '\0';
 
-   switch (status)
-   {
+   switch (status) {
       case MEMORY_ALLOCATION:
          except_type = PyExc_MemoryError;
          break;
@@ -160,19 +157,15 @@ void process_status_err(int status)
          break;
       default:
          except_type = PyExc_RuntimeError;
+         break;
    }
 
-   if (fits_read_errmsg(err_msg))
-   {
+   if (fits_read_errmsg(err_msg)) {
       PyErr_SetString(except_type, err_msg);
-   }
-   else if (*def_err_msg)
-   {
+   } else if (*def_err_msg) {
       PyErr_SetString(except_type, def_err_msg);
-   }
-   else
-   {
-      PyErr_SetString(except_type, "unknown error.");
+   } else {
+      PyErr_Format(except_type, "unknown error %i.", status);
    }
 }
 
@@ -185,7 +178,7 @@ void bitpix_to_datatypes(int bitpix, int* datatype, int* npdatatype) {
     switch (bitpix) {
         case BYTE_IMG:
             *datatype = TBYTE;
-            *npdatatype = NPY_UINT8;
+            *npdatatype = NPY_INT8;
             break;
         case SHORT_IMG:
             *datatype = TSHORT;
@@ -210,6 +203,7 @@ void bitpix_to_datatypes(int bitpix, int* datatype, int* npdatatype) {
         default:
             PyErr_Format(PyExc_ValueError, "Invalid value for BITPIX: %d",
                          bitpix);
+            break;
    }
 
    return;
@@ -244,165 +238,158 @@ int compress_type_from_string(char* zcmptype) {
 }
 
 
+PyObject *
+get_header_value(PyObject* header, const char* key, HeaderGetFlags flags) {
+    PyObject* hdrkey;
+    PyObject* hdrval;
+    hdrkey = PyUnicode_FromString(key);
+    if (hdrkey == NULL) {
+        return NULL;
+    }
+    hdrval = PyObject_GetItem(header, hdrkey);
+    Py_DECREF(hdrkey);
+    if ((flags & HDR_FAIL_KEY_MISSING) == 0) {
+        /* Normally we have a default so we want to ignore the exception in
+           any case. But if the flag was given this step must be skipped. */
+        PyErr_Clear();
+    }
+    return hdrval;
+}
+
+
 // TODO: It might be possible to simplify these further by making the
 // conversion function (eg. PyString_AsString) an argument to a macro or
 // something, but I'm not sure yet how easy it is to generalize the error
 // handling
-int get_header_string(PyObject* header, char* keyword, char* val, char* def) {
-    PyObject* keystr;
-    PyObject* keyval;
-#ifdef IS_PY3K
-    PyObject* tmp;  // Temp pointer to decoded bytes object
-#endif
-    int retval;
+/* The get_header_* functions resemble "Header.get" where "def" is the default
+   value, "keyword" is a string representing the header-key and the result is
+   stored in "val".
+   The function returns 0 on success, 1 if the header didn't have the keyword
+   and the default was applied and -1 (with an exception set) if an Exception
+   happened (like a MemoryError or Overflow).
+*/
+#define GET_HEADER_SUCCESS 0
+#define GET_HEADER_DEFAULT_USED 1
+#define GET_HEADER_FAILED -1
+int get_header_string(PyObject* header, const char* keyword, char* val,
+                      const char* def, HeaderGetFlags flags) {
+    /* nonnegative doesn't make sense for strings*/
+    assert(!(flags & HDR_FAIL_VAL_NEGATIVE));
+    PyObject* keyval = get_header_value(header, keyword, flags);
 
-    keystr = PyString_FromString(keyword);
-    keyval = PyObject_GetItem(header, keystr);
-
-    if (keyval != NULL) {
-#ifdef IS_PY3K
-        // FITS header values should always be ASCII, but Latin1 is on the
-        // safe side
-        tmp = PyUnicode_AsLatin1String(keyval);
-        strncpy(val, PyBytes_AsString(tmp), 72);
-        Py_DECREF(tmp);
-#else
-        strncpy(val, PyString_AsString(keyval), 72);
-#endif
-        retval = 0;
-    }
-    else {
-        PyErr_Clear();
+    if (keyval == NULL) {
         strncpy(val, def, 72);
-        retval = 1;
+        return PyErr_Occurred() ? GET_HEADER_FAILED : GET_HEADER_DEFAULT_USED;
     }
-
-    Py_DECREF(keystr);
-    Py_XDECREF(keyval);
-    return retval;
+    PyObject* tmp = PyUnicode_AsLatin1String(keyval);
+    // FITS header values should always be ASCII, but Latin1 is on the
+    // safe side
+    Py_DECREF(keyval);
+    if (tmp == NULL) {
+        /* could always fail to allocate the memory or such like. */
+        return GET_HEADER_FAILED;
+    }
+    strncpy(val, PyBytes_AsString(tmp), 72);
+    Py_DECREF(tmp);
+    return GET_HEADER_SUCCESS;
 }
 
 
-int get_header_int(PyObject* header, char* keyword, int* val, int def) {
-    PyObject* keystr;
-    PyObject* keyval;
-    int retval;
+int get_header_long(PyObject* header, const char* keyword, long* val, long def,
+                    HeaderGetFlags flags) {
+    PyObject* keyval = get_header_value(header, keyword, flags);
 
-    keystr = PyString_FromString(keyword);
-    keyval = PyObject_GetItem(header, keystr);
-
-    if (keyval != NULL) {
-        *val = (int) PyInt_AsLong(keyval);
-        retval = 0;
-    }
-    else {
-        PyErr_Clear();
+    if (keyval == NULL) {
         *val = def;
-        retval = 1;
+        return PyErr_Occurred() ? GET_HEADER_FAILED : GET_HEADER_DEFAULT_USED;
     }
-
-    Py_DECREF(keystr);
-    Py_XDECREF(keyval);
-    return retval;
+    long tmp = PyLong_AsLong(keyval);
+    Py_DECREF(keyval);
+    if (PyErr_Occurred()) {
+        return GET_HEADER_FAILED;
+    }
+    if ((flags & HDR_FAIL_VAL_NEGATIVE) && (tmp < 0)) {
+        PyErr_Format(PyExc_ValueError, "%s should not be negative.", keyword);
+        return GET_HEADER_FAILED;
+    }
+    *val = tmp;
+    return GET_HEADER_SUCCESS;
 }
 
 
-int get_header_long(PyObject* header, char* keyword, long* val, long def) {
-    PyObject* keystr;
-    PyObject* keyval;
-    int retval;
-
-    keystr = PyString_FromString(keyword);
-    keyval = PyObject_GetItem(header, keystr);
-
-    if (keyval != NULL) {
-        *val = PyLong_AsLong(keyval);
-        retval = 0;
+int get_header_int(PyObject* header, const char* keyword, int* val, int def,
+                   HeaderGetFlags flags) {
+    long tmp;
+    int ret = get_header_long(header, keyword, &tmp, def, flags);
+    if (ret == GET_HEADER_SUCCESS) {
+        if (tmp >= INT_MIN && tmp <= INT_MAX) {
+            *val = (int) tmp;
+        } else {
+            PyErr_Format(PyExc_OverflowError, "Cannot convert %ld to C 'int'", tmp);
+            ret = GET_HEADER_FAILED;
+        }
     }
-    else {
-        PyErr_Clear();
-        *val = def;
-        retval = 1;
-    }
-
-    Py_DECREF(keystr);
-    Py_XDECREF(keyval);
-    return retval;
+    return ret;
 }
 
 
-int get_header_float(PyObject* header, char* keyword, float* val,
-                     float def) {
-    PyObject* keystr;
-    PyObject* keyval;
-    int retval;
+int get_header_double(PyObject* header, const char* keyword, double* val,
+                      double def, HeaderGetFlags flags) {
+    /* nonnegative isn't currently used for doubles/floats. But if needed one
+       could simply remove the assert again and implement the negative check. */
+    assert(!(flags & HDR_FAIL_VAL_NEGATIVE));
+    PyObject* keyval = get_header_value(header, keyword, flags);
 
-    keystr = PyString_FromString(keyword);
-    keyval = PyObject_GetItem(header, keystr);
-
-    if (keyval != NULL) {
-        *val = (float) PyFloat_AsDouble(keyval);
-        retval = 0;
-    }
-    else {
-        PyErr_Clear();
+    if (keyval == NULL) {
         *val = def;
-        retval = 1;
+        return PyErr_Occurred() ? GET_HEADER_FAILED : GET_HEADER_DEFAULT_USED;
     }
-
-    Py_DECREF(keystr);
-    Py_XDECREF(keyval);
-    return retval;
+    double tmp = PyFloat_AsDouble(keyval);
+    Py_DECREF(keyval);
+    if (PyErr_Occurred()) {
+        return GET_HEADER_FAILED;
+    }
+    *val = tmp;
+    return GET_HEADER_SUCCESS;
 }
 
 
-int get_header_double(PyObject* header, char* keyword, double* val,
-                      double def) {
-    PyObject* keystr;
-    PyObject* keyval;
-    int retval;
-
-    keystr = PyString_FromString(keyword);
-    keyval = PyObject_GetItem(header, keystr);
-
-    if (keyval != NULL) {
-        *val = PyFloat_AsDouble(keyval);
-        retval = 0;
+int get_header_float(PyObject* header, const char* keyword, float* val,
+                     float def, HeaderGetFlags flags) {
+    double tmp;
+    int ret = get_header_double(header, keyword, &tmp, def, flags);
+    if (ret == GET_HEADER_SUCCESS) {
+        if (tmp == 0.0 || (fabs(tmp) >= FLT_MIN && fabs(tmp) <= FLT_MAX)) {
+            *val = (float) tmp;
+        } else {
+            PyErr_SetString(PyExc_OverflowError,
+                            "Cannot convert 'double' to 'float'");
+            ret = GET_HEADER_FAILED;
+        }
     }
-    else {
-        PyErr_Clear();
-        *val = def;
-        retval = 1;
-    }
-
-    Py_DECREF(keystr);
-    Py_XDECREF(keyval);
-    return retval;
+    return ret;
 }
 
 
-int get_header_longlong(PyObject* header, char* keyword, long long* val,
-                        long long def) {
-    PyObject* keystr;
-    PyObject* keyval;
-    int retval;
+int get_header_longlong(PyObject* header, const char* keyword, long long* val,
+                        long long def, HeaderGetFlags flags) {
+    PyObject* keyval = get_header_value(header, keyword, flags);
 
-    keystr = PyString_FromString(keyword);
-    keyval = PyObject_GetItem(header, keystr);
-
-    if (keyval != NULL) {
-        *val = PyLong_AsLongLong(keyval);
-        retval = 0;
-    }
-    else {
-        PyErr_Clear();
+    if (keyval == NULL) {
         *val = def;
-        retval = 1;
+        return PyErr_Occurred() ? GET_HEADER_FAILED : GET_HEADER_DEFAULT_USED;
     }
-
-    Py_DECREF(keystr);
-    Py_XDECREF(keyval);
-    return retval;
+    long long tmp = PyLong_AsLongLong(keyval);
+    Py_DECREF(keyval);
+    if (PyErr_Occurred()) {
+        return GET_HEADER_FAILED;
+    }
+    if ((flags & HDR_FAIL_VAL_NEGATIVE) && (tmp < 0)) {
+        PyErr_Format(PyExc_ValueError, "%s should not be negative.", keyword);
+        return GET_HEADER_FAILED;
+    }
+    *val = tmp;
+    return GET_HEADER_SUCCESS;
 }
 
 
@@ -414,7 +401,6 @@ void tcolumns_from_header(fitsfile* fileptr, PyObject* header,
 
     tcolumn* column;
     char tkw[9];
-    unsigned int idx;
 
     int tfields;
     char ttype[72];
@@ -424,14 +410,27 @@ void tcolumns_from_header(fitsfile* fileptr, PyObject* header,
     long twidth;
     long long totalwidth;
     int status = 0;
+    int idx;
 
-    get_header_int(header, "TFIELDS", &tfields, 0);
+    if (get_header_int(header, "TFIELDS", &tfields, 0, HDR_FAIL_VAL_NEGATIVE) == GET_HEADER_FAILED) {
+        return;
+    }
+    /* To avoid issues in the loop we need to limit the number of TFIELDs to
+       999. Otherwise we would exceed the maximum length of the keyword name of
+       8. This could lead to multiple accesses of the same header keyword with
+       snprintf because we limit it to 8 characters + null-termination. */
+    if (tfields > 999) {
+        PyErr_SetString(PyExc_ValueError, "The TFIELDS value exceeds 999.");
+        return;
+    }
 
     // This used to use PyMem_New, but don't do that; CFITSIO will later
     // free() this object when the file is closed, so just use malloc here
     // *columns = column = PyMem_New(tcolumn, (size_t) tfields);
     *columns = column = calloc((size_t) tfields, sizeof(tcolumn));
     if (column == NULL) {
+        PyErr_SetString(PyExc_MemoryError,
+                        "Couldn't allocate memory for columns.");
         return;
     }
 
@@ -447,12 +446,16 @@ void tcolumns_from_header(fitsfile* fileptr, PyObject* header,
         column->twidth = 0;
 
         snprintf(tkw, 9, "TTYPE%u", idx);
-        get_header_string(header, tkw, ttype, "");
+        if (get_header_string(header, tkw, ttype, "", HDR_NOFLAG) == GET_HEADER_FAILED) {
+            return;
+        }
         strncpy(column->ttype, ttype, 69);
         column->ttype[69] = '\0';
 
         snprintf(tkw, 9, "TFORM%u", idx);
-        get_header_string(header, tkw, tform, "");
+        if (get_header_string(header, tkw, tform, "", HDR_NOFLAG) == GET_HEADER_FAILED) {
+            return;
+        }
         strncpy(column->tform, tform, 9);
         column->tform[9] = '\0';
         fits_binary_tform(tform, &dtcode, &trepeat, &twidth, &status);
@@ -466,13 +469,19 @@ void tcolumns_from_header(fitsfile* fileptr, PyObject* header,
         column->twidth = twidth;
 
         snprintf(tkw, 9, "TSCAL%u", idx);
-        get_header_double(header, tkw, &(column->tscale), 1.0);
+        if (get_header_double(header, tkw, &(column->tscale), 1.0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+            return;
+        }
 
         snprintf(tkw, 9, "TZERO%u", idx);
-        get_header_double(header, tkw, &(column->tzero), 0.0);
+        if (get_header_double(header, tkw, &(column->tzero), 0.0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+            return;
+        }
 
         snprintf(tkw, 9, "TNULL%u", idx);
-        get_header_longlong(header, tkw, &(column->tnull), NULL_UNDEFINED);
+        if (get_header_longlong(header, tkw, &(column->tnull), NULL_UNDEFINED, HDR_NOFLAG) == GET_HEADER_FAILED) {
+            return;
+        }
     }
 
     fileptr->Fptr->tableptr = *columns;
@@ -505,14 +514,16 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
     char tmp[72];
     float version;
 
-    unsigned int idx;
+    int idx;
 
     Fptr = fileptr->Fptr;
     tfields = Fptr->tfield;
     columns = Fptr->tableptr;
 
+    int tmp_retval;
+
     // Get the ZBITPIX header value; if this is missing we're in trouble
-    if (0 != get_header_int(header, "ZBITPIX", &(Fptr->zbitpix), 0)) {
+    if (get_header_int(header, "ZBITPIX", &(Fptr->zbitpix), 0, HDR_FAIL_KEY_MISSING) != GET_HEADER_SUCCESS) {
         return;
     }
 
@@ -547,29 +558,51 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
     Fptr->zblank = 0;
     if (Fptr->cn_zblank < 1) {
         // No ZBLANK column--check the ZBLANK and BLANK heard keywords
-        if(0 != get_header_int(header, "ZBLANK", &(Fptr->zblank), 0)) {
+        switch (get_header_int(header, "ZBLANK", &(Fptr->zblank), 0, HDR_NOFLAG)) {
+          case GET_HEADER_FAILED:
+            return;
+          case GET_HEADER_DEFAULT_USED:
             // ZBLANK keyword not found
-            get_header_int(header, "BLANK", &(Fptr->zblank), 0);
+            if (get_header_int(header, "BLANK", &(Fptr->zblank), 0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+              return;
+            }
+            break;
+          default:
+            break;
         }
     }
 
     Fptr->zscale = 1.0;
     if (Fptr->cn_zscale < 1) {
-        if (0 != get_header_double(header, "ZSCALE", &(Fptr->zscale), 1.0)) {
+        switch (get_header_double(header, "ZSCALE", &(Fptr->zscale), 1.0, HDR_NOFLAG)) {
+          case GET_HEADER_FAILED:
+            return;
+          case GET_HEADER_DEFAULT_USED:
             Fptr->cn_zscale = 0;
+            break;
+          default:
+            break;
         }
     }
     Fptr->cn_bscale = Fptr->zscale;
 
     Fptr->zzero = 0.0;
     if (Fptr->cn_zzero < 1) {
-        if (0 != get_header_double(header, "ZZERO", &(Fptr->zzero), 0.0)) {
+        switch (get_header_double(header, "ZZERO", &(Fptr->zzero), 0.0, HDR_NOFLAG)) {
+          case GET_HEADER_FAILED:
+            return;
+          case GET_HEADER_DEFAULT_USED:
             Fptr->cn_zzero = 0;
+            break;
+          default:
+            break;
         }
     }
     Fptr->cn_bzero = Fptr->zzero;
 
-    get_header_string(header, "ZCMPTYPE", tmp, DEFAULT_COMPRESSION_TYPE);
+    if (get_header_string(header, "ZCMPTYPE", tmp, DEFAULT_COMPRESSION_TYPE, HDR_NOFLAG) == GET_HEADER_FAILED) {
+        return;
+    }
     strncpy(Fptr->zcmptype, tmp, 11);
     Fptr->zcmptype[strlen(tmp)] = '\0';
 
@@ -578,7 +611,9 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
         return;
     }
 
-    get_header_int(header, "ZNAXIS", &znaxis, 0);
+    if (get_header_int(header, "ZNAXIS", &znaxis, 0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+        return;
+    }
     Fptr->zndim = znaxis;
 
     if (znaxis > MAX_COMPRESS_DIM) {
@@ -591,9 +626,13 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
     Fptr->maxtilelen = 1;
     for (idx = 1; idx <= znaxis; idx++) {
         snprintf(keyword, 9, "ZNAXIS%u", idx);
-        get_header_long(header, keyword, Fptr->znaxis + idx - 1, 0);
+        if (get_header_long(header, keyword, Fptr->znaxis + idx - 1, 0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+            return;
+        }
         snprintf(keyword, 9, "ZTILE%u", idx);
-        get_header_long(header, keyword, Fptr->tilesize + idx - 1, 0);
+        if (get_header_long(header, keyword, Fptr->tilesize + idx - 1, 0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+            return;
+        }
         Fptr->maxtilelen *= Fptr->tilesize[idx - 1];
     }
 
@@ -611,34 +650,48 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
         // Assumes there are no gaps in the ZNAMEn keywords; this same
         // assumption was made in the Python code.  This could be done slightly
         // more flexibly by using a wildcard slice of the header
-        if (0 != get_header_string(header, keyword, zname, "")) {
+        tmp_retval = get_header_string(header, keyword, zname, "", HDR_NOFLAG);
+        if (tmp_retval == GET_HEADER_FAILED) {
+            return;
+        } else if (tmp_retval == 1) {
             break;
         }
+
         snprintf(keyword, 9, "ZVAL%u", idx);
         if (Fptr->compress_type == RICE_1) {
             if (0 == strcmp(zname, "BLOCKSIZE")) {
-                get_header_int(header, keyword, &(Fptr->rice_blocksize),
-                               DEFAULT_BLOCK_SIZE);
+                if (get_header_int(header, keyword, &(Fptr->rice_blocksize),
+                                   DEFAULT_BLOCK_SIZE, HDR_NOFLAG) == GET_HEADER_FAILED) {
+                    return;
+                }
             } else if (0 == strcmp(zname, "BYTEPIX")) {
-                get_header_int(header, keyword, &(Fptr->rice_bytepix),
-                               DEFAULT_BYTE_PIX);
+                if (get_header_int(header, keyword, &(Fptr->rice_bytepix),
+                                   DEFAULT_BYTE_PIX, HDR_NOFLAG) == GET_HEADER_FAILED) {
+                    return;
+                }
             }
         } else if (Fptr->compress_type == HCOMPRESS_1) {
             if (0 == strcmp(zname, "SMOOTH")) {
-                get_header_int(header, keyword, &(Fptr->hcomp_smooth),
-                               DEFAULT_HCOMP_SMOOTH);
+                if (get_header_int(header, keyword, &(Fptr->hcomp_smooth),
+                                   DEFAULT_HCOMP_SMOOTH, HDR_NOFLAG) == GET_HEADER_FAILED) {
+                    return;
+                }
             } else if (0 == strcmp(zname, "SCALE")) {
-                get_header_float(header, keyword, &(Fptr->hcomp_scale),
-                                 DEFAULT_HCOMP_SCALE);
+                if (get_header_float(header, keyword, &(Fptr->hcomp_scale),
+                                     DEFAULT_HCOMP_SCALE, HDR_NOFLAG) == GET_HEADER_FAILED) {
+                    return;
+                }
             }
         }
         if (Fptr->zbitpix < 0 && 0 == strcmp(zname, "NOISEBIT")) {
-             get_header_float(header, keyword, &(Fptr->quantize_level),
-                              DEFAULT_QUANTIZE_LEVEL);
-             if (Fptr->quantize_level == 0.0) {
-                 /* NOISEBIT == 0 is equivalent to no quantize */
-                 Fptr->quantize_level = NO_QUANTIZE;
-             }
+            if (get_header_float(header, keyword, &(Fptr->quantize_level),
+                                 DEFAULT_QUANTIZE_LEVEL, HDR_NOFLAG) == GET_HEADER_FAILED) {
+                return;
+            }
+            if (Fptr->quantize_level == 0.0) {
+                /* NOISEBIT == 0 is equivalent to no quantize */
+                Fptr->quantize_level = NO_QUANTIZE;
+            }
         }
 
         idx++;
@@ -646,7 +699,10 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
 
     /* The ZQUANTIZ keyword determines the quantization algorithm; NO_QUANTIZE
        implies lossless compression */
-    if (0 == get_header_string(header, "ZQUANTIZ", tmp, "")) {
+    tmp_retval = get_header_string(header, "ZQUANTIZ", tmp, "", HDR_NOFLAG);
+    if (tmp_retval == GET_HEADER_FAILED) {
+        return;
+    } else if (tmp_retval == GET_HEADER_SUCCESS) {
         /* Ugh; the fact that cfitsio defines its version as a float makes
            preprocessor comparison impossible */
         fits_get_version(&version);
@@ -668,10 +724,15 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
     }
 
     if (Fptr->quantize_method != NO_DITHER) {
-        if (0 != get_header_int(header, "ZDITHER0", &(Fptr->dither_seed), 0)) {
-            // ZDITHER0 keyword not found
+        switch (get_header_int(header, "ZDITHER0", &(Fptr->dither_seed), 0, HDR_NOFLAG)) {
+          case GET_HEADER_FAILED:
+            return;
+          case GET_HEADER_DEFAULT_USED: // ZDITHER0 keyword not found
             Fptr->dither_seed = 0;
             Fptr->request_dither_seed = 0;
+            break;
+          default:
+            break;
         }
     }
 #else
@@ -684,11 +745,16 @@ void configure_compression(fitsfile* fileptr, PyObject* header) {
     }
 
     if (Fptr->quantize_dither != NO_DITHER) {
-        if (0 != get_header_int(header, "ZDITHER0", &(Fptr->dither_offset),
-                                0)) {
-            // ZDITHER0 keyword no found
+        switch (get_header_int(header, "ZDITHER0", &(Fptr->dither_offset), 0, HDR_NOFLAG)) {
+          case GET_HEADER_FAILED:
+            return;
+          case GET_HEADER_DEFAULT_USED: // ZDITHER0 keyword no found
+            /* TODO: Find out if that's actually working and not invalid... */
             Fptr->dither_offset = 0;
             Fptr->request_dither_offset = 0;
+            break;
+          default:
+            break;
         }
     }
 #endif
@@ -712,7 +778,6 @@ void init_output_buffer(PyObject* hdu, void** buf, size_t* bufsize) {
     char keyword[9];
     char tmp[72];
     int znaxis;
-    int idx;
     int compress_type;
     int zbitpix;
     int rice_blocksize = 0;
@@ -721,37 +786,56 @@ void init_output_buffer(PyObject* hdu, void** buf, size_t* bufsize) {
     long maxelem;
     long tilelen;
     unsigned long maxtilelen = 1;
+    int idx;
 
     header = PyObject_GetAttrString(hdu, "_header");
     if (header == NULL) {
+        return;
+    }
+
+    if (get_header_int(header, "ZNAXIS", &znaxis, 0,
+                       HDR_FAIL_KEY_MISSING | HDR_FAIL_VAL_NEGATIVE) != GET_HEADER_SUCCESS) {
         goto fail;
     }
 
-    if (0 != get_header_int(header, "ZNAXIS", &znaxis, 0)) {
-        PyErr_SetString(PyExc_TypeError,
-                        "ZNAXIS keyword not present in header.");
+    if (znaxis > 999) {
+        PyErr_SetString(PyExc_ValueError, "ZNAXIS is greater than 999.");
         goto fail;
     }
 
     for (idx = 1; idx <= znaxis; idx++) {
         snprintf(keyword, 9, "ZTILE%u", idx);
-        get_header_long(header, keyword, &tilelen, 1);
+        if (get_header_long(header, keyword, &tilelen, 1, HDR_NOFLAG) == GET_HEADER_FAILED) {
+            goto fail;
+        }
         maxtilelen *= tilelen;
     }
 
-    get_header_string(header, "ZCMPTYPE", tmp, DEFAULT_COMPRESSION_TYPE);
+    if (get_header_string(header, "ZCMPTYPE", tmp, DEFAULT_COMPRESSION_TYPE, HDR_NOFLAG) == GET_HEADER_FAILED) {
+        goto fail;
+    }
     compress_type = compress_type_from_string(tmp);
+    if (PyErr_Occurred()) {
+        goto fail;
+    }
     if (compress_type == RICE_1) {
-        get_header_int(header, "ZVAL1", &rice_blocksize, 0);
+        if (get_header_int(header, "ZVAL1", &rice_blocksize, 0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+            goto fail;
+        }
     }
 
-    get_header_longlong(header, "NAXIS1", &rowlen, 0);
-    get_header_longlong(header, "NAXIS2", &nrows, 0);
+    /* Because we calculate the size of the buffer based on these values they
+       must not be negative. Otherwise it would wrap around during the casting
+       to size_t and give huge values. */
+    if (get_header_longlong(header, "NAXIS1", &rowlen, 0, HDR_FAIL_VAL_NEGATIVE) == GET_HEADER_FAILED) {
+        goto fail;
+    }
+    if (get_header_longlong(header, "NAXIS2", &nrows, 0, HDR_FAIL_VAL_NEGATIVE) == GET_HEADER_FAILED) {
+        goto fail;
+    }
 
     // Get the ZBITPIX header value; if this is missing we're in trouble
-    if (0 != get_header_int(header, "ZBITPIX", &zbitpix, 0)) {
-        PyErr_SetString(PyExc_TypeError,
-                        "ZBITPIX keyword not present in header.");
+    if (get_header_int(header, "ZBITPIX", &zbitpix, 0, HDR_FAIL_KEY_MISSING) != GET_HEADER_SUCCESS) {
         goto fail;
     }
 
@@ -779,7 +863,7 @@ void init_output_buffer(PyObject* hdu, void** buf, size_t* bufsize) {
     }
 
 fail:
-    Py_XDECREF(header);
+    Py_DECREF(header);
     return;
 }
 
@@ -826,6 +910,7 @@ fail:
 
 void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
                    PyObject* hdu, tcolumn** columns, int mode) {
+
     PyObject* header = NULL;
     FITSfile* Fptr;
 
@@ -840,19 +925,26 @@ void open_from_hdu(fitsfile** fileptr, void** buf, size_t* bufsize,
         goto fail;
     }
 
-    get_header_longlong(header, "NAXIS1", &rowlen, 0);
-    get_header_longlong(header, "NAXIS2", &nrows, 0);
+    if (get_header_longlong(header, "NAXIS1", &rowlen, 0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+        goto fail;
+    }
+    if (get_header_longlong(header, "NAXIS2", &nrows, 0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+        goto fail;
+    }
 
     // The PCOUNT keyword contains the number of bytes in the table heap
-    get_header_longlong(header, "PCOUNT", &heapsize, 0);
+    if (get_header_longlong(header, "PCOUNT", &heapsize, 0, HDR_FAIL_VAL_NEGATIVE) == GET_HEADER_FAILED) {
+        goto fail;
+    }
 
     // The THEAP keyword gives the offset of the heap from the beginning of
     // the HDU data portion; normally this offset is 0 but it can be set
     // to something else with THEAP
-    get_header_longlong(header, "THEAP", &theap, 0);
+    if (get_header_longlong(header, "THEAP", &theap, 0, HDR_NOFLAG) == GET_HEADER_FAILED) {
+        goto fail;
+    }
 
     fits_create_memfile(fileptr, buf, bufsize, 0, realloc, &status);
-
     if (status != 0) {
         process_status_err(status);
         goto fail;
@@ -895,7 +987,6 @@ fail:
 }
 
 
-
 PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
 {
     PyObject* hdu;
@@ -905,6 +996,7 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     void* outbuf = NULL;
     size_t outbufsize;
 
+    PyObject* tmp_indata;
     PyArrayObject* indata = NULL;
     PyArrayObject* tmp;
     npy_intp znaxis;
@@ -912,13 +1004,11 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     int npdatatype;
     unsigned long long heapsize;
 
-    fitsfile* fileptr;
+    fitsfile* fileptr = NULL;
     FITSfile* Fptr = NULL;
     int status = 0;
 
-    if (!PyArg_ParseTuple(args, "O:compression.compress_hdu", &hdu))
-    {
-        PyErr_SetString(PyExc_TypeError, "Couldn't parse arguments");
+    if (!PyArg_ParseTuple(args, "O:compression.compress_hdu", &hdu)) {
         return NULL;
     }
 
@@ -944,7 +1034,20 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
         goto fail;
     }
 
-    indata = (PyArrayObject*) PyObject_GetAttrString(hdu, "data");
+    /* The data attribute could be something different from an array, i.e. None */
+    tmp_indata = PyObject_GetAttrString(hdu, "data");
+    if (tmp_indata == NULL) {
+        goto fail;
+    }
+
+    if (!PyObject_TypeCheck(tmp_indata, &PyArray_Type)) {
+        PyErr_SetString(PyExc_TypeError,
+                        "CompImageHDU.data must be a numpy.ndarray");
+        Py_DECREF(tmp_indata);
+        goto fail;
+    }
+
+    indata = (PyArrayObject*) tmp_indata;
 
     fits_write_img(fileptr, datatype, 1, PyArray_SIZE(indata),
                    PyArray_DATA(indata), &status);
@@ -968,19 +1071,43 @@ PyObject* compression_compress_hdu(PyObject* self, PyObject* args)
     znaxis = (npy_intp) (Fptr->heapstart + heapsize);
 
     if (znaxis < outbufsize) {
+        void* tmp_outbuf = NULL;
         // Go ahead and truncate to the size in znaxis to free the
         // redundant allocation
-        // TODO: Add error handling
-        outbuf = realloc(outbuf, (size_t) znaxis);
+        if (znaxis == 0) {
+            /* This really shouldn't happen, but if it did, we would have a
+               problem because realloc would deallocate outbuf AND return NULL.
+               */
+            PyErr_SetString(PyExc_ValueError,
+                            "Calculated array size is zero. This shouldn't happen!");
+            goto fail;
+        }
+        tmp_outbuf = realloc(outbuf, (size_t) znaxis);
+        if (tmp_outbuf == NULL) {
+            PyErr_SetString(PyExc_MemoryError,
+                            "Couldn't resize the output-buffer.");
+            goto fail;
+        }
+        outbuf = tmp_outbuf;
     }
 
     tmp = (PyArrayObject*) PyArray_SimpleNewFromData(1, &znaxis, NPY_UBYTE,
                                                      outbuf);
-
+    if (tmp == NULL) {
+        /* Really not sure if it's always safe to free outbuf when
+           PyArray_SimpleNewFromData failed (which is unlikely but could happen)
+           but it seems like if it fails then the outbuf NEEDS to be freed... */
+        goto fail;
+    }
     PyArray_ENABLEFLAGS(tmp, NPY_ARRAY_OWNDATA);
+    /* From this point on outbuf MUST NOT BE FREED! */
 
     // Leaves refcount of tmp untouched, so its refcount should remain as 1
     retval = Py_BuildValue("KN", heapsize, tmp);
+    if (retval == NULL) {
+        Py_DECREF(tmp);
+        goto cleanup;
+    }
 
     goto cleanup;
 
@@ -992,10 +1119,13 @@ fail:
     }
 cleanup:
     if (columns != NULL) {
-        PyMem_Free(columns);
-	if (Fptr != NULL) {
-	    Fptr->tableptr = NULL;
-	}
+        free(columns);
+        /* See https://github.com/astropy/astropy/pull/4489
+           We can only set the tableptr to NULL if Fptr is actually not NULL.
+           */
+        if (fileptr != NULL && fileptr->Fptr != NULL) {
+            fileptr->Fptr->tableptr = NULL;
+        }
     }
 
     if (fileptr != NULL) {
@@ -1018,27 +1148,28 @@ cleanup:
 
 PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
 {
+
     PyObject* hdu;
     tcolumn* columns = NULL;
 
     void* inbuf;
     size_t inbufsize;
 
-    PyArrayObject* outdata;
+    PyArrayObject* outdata = NULL;
     int datatype;
     int npdatatype;
     npy_intp zndim;
-    npy_intp* znaxis;
+    npy_intp* znaxis = NULL;
     long arrsize;
-    unsigned int idx;
 
     fitsfile* fileptr = NULL;
     int anynul = 0;
     int status = 0;
+    int idx;
 
-    if (!PyArg_ParseTuple(args, "O:compression.decompress_hdu", &hdu))
-    {
-        PyErr_SetString(PyExc_TypeError, "Couldn't parse arguments");
+    int free_columns_manually = 1;
+
+    if (!PyArg_ParseTuple(args, "O:compression.decompress_hdu", &hdu)) {
         return NULL;
     }
 
@@ -1050,22 +1181,25 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
     } else if (inbufsize == 0) {
         // The compressed data buffer is empty (probably zero rows, for an
         // empty "compressed" image.  Just return None in this case.
-        Py_INCREF(Py_None);
-        return Py_None;
+        Py_RETURN_NONE;
     }
 
     open_from_hdu(&fileptr, &inbuf, &inbufsize, hdu, &columns, READONLY);
     if (PyErr_Occurred()) {
-        return NULL;
+        goto fail;
     }
 
     bitpix_to_datatypes(fileptr->Fptr->zbitpix, &datatype, &npdatatype);
     if (PyErr_Occurred()) {
-        return NULL;
+        goto fail;
     }
 
     zndim = (npy_intp)fileptr->Fptr->zndim;
-    znaxis = (npy_intp*) PyMem_Malloc(sizeof(npy_intp) * zndim);
+    znaxis = PyMem_Malloc(sizeof(npy_intp) * zndim);
+    if (znaxis == NULL) {
+        goto fail;
+    }
+
     arrsize = 1;
     for (idx = 0; idx < zndim; idx++) {
         znaxis[zndim - idx - 1] = fileptr->Fptr->znaxis[idx];
@@ -1074,25 +1208,32 @@ PyObject* compression_decompress_hdu(PyObject* self, PyObject* args)
 
     /* Create and allocate a new array for the decompressed data */
     outdata = (PyArrayObject*) PyArray_SimpleNew(zndim, znaxis, npdatatype);
+    if (outdata == NULL) {
+        goto fail;
+    }
 
     fits_read_img(fileptr, datatype, 1, arrsize, NULL, PyArray_DATA(outdata),
                   &anynul, &status);
+    /* At this point we need to let CFITSIO clean up the tableptr and the
+       compressed tile cache. */
+    free_columns_manually = 0;
     if (status != 0) {
         process_status_err(status);
+        Py_DECREF(outdata);
         outdata = NULL;
-        goto fail;
     }
 
 fail:
     // CFITSIO will free this object in the ffchdu function by way of
     // fits_close_file; we need to let CFITSIO handle this so that it also
-    // cleans up the compressed tile cache
-    /*
-    if (columns != NULL) {
-        PyMem_Free(columns);
-        fileptr->Fptr->tableptr = NULL;
+    // cleans up the compressed tile cache - but that's only necessary in case
+    // we called "fits_read_img"...
+    if (free_columns_manually && columns != NULL) {
+        free(columns);
+        if (fileptr != NULL && fileptr->Fptr != NULL) {
+            fileptr->Fptr->tableptr = NULL;
+        }
     }
-    */
 
     if (fileptr != NULL) {
         status = 1;// Disable header-related errors
@@ -1103,7 +1244,9 @@ fail:
         }
     }
 
-    PyMem_Free(znaxis);
+    if (znaxis != NULL) {
+        PyMem_Free(znaxis);
+    }
 
     // Clear any messages remaining in CFITSIO's error stack
     fits_clear_errmsg();
@@ -1116,11 +1259,13 @@ fail:
 static double cfitsio_version;
 
 
-void compression_module_init(PyObject* module) {
-    /* Python version-indendependent initialization routine for the
-       compression module */
+int compression_module_init(PyObject* module) {
+    /* Python version-independent initialization routine for the
+       compression module. Returns 0 on success and -1 (with exception set)
+       on failure. */
     PyObject* tmp;
     float version_tmp;
+    int ret;
 
     fits_get_version(&version_tmp);
     cfitsio_version = (double) version_tmp;
@@ -1131,10 +1276,12 @@ void compression_module_init(PyObject* module) {
     cfitsio_version = floor((1000 * version_tmp + 0.5)) / 1000;
 
     tmp = PyFloat_FromDouble(cfitsio_version);
-    PyObject_SetAttrString(module, "CFITSIO_VERSION", tmp);
-    Py_XDECREF(tmp);
-
-    return;
+    if (tmp == NULL) {
+        return -1;
+    }
+    ret = PyObject_SetAttrString(module, "CFITSIO_VERSION", tmp);
+    Py_DECREF(tmp);
+    return ret;
 }
 
 
@@ -1146,7 +1293,6 @@ static PyMethodDef compression_methods[] =
    {NULL, NULL}
 };
 
-#ifdef IS_PY3K
 static struct PyModuleDef compressionmodule = {
     PyModuleDef_HEAD_INIT,
     "compression",
@@ -1159,7 +1305,13 @@ PyObject *
 PyInit_compression(void)
 {
     PyObject* module = PyModule_Create(&compressionmodule);
-    compression_module_init(module);
+    if (module == NULL) {
+        return NULL;
+    }
+    if (compression_module_init(module)) {
+        Py_DECREF(module);
+        return NULL;
+    }
 
     /* Needed to use Numpy routines */
     /* Note -- import_array() is a macro that behaves differently in Python2.x
@@ -1169,13 +1321,3 @@ PyInit_compression(void)
     import_array();
     return module;
 }
-#else
-PyMODINIT_FUNC initcompression(void)
-{
-   PyObject* module = Py_InitModule4("compression", compression_methods,
-                                     "astropy.io.fits.compression module",
-                                     NULL, PYTHON_API_VERSION);
-   compression_module_init(module);
-   import_array();
-}
-#endif
