@@ -7,18 +7,22 @@ import pytest
 import numpy as np
 from numpy.testing import assert_allclose
 
+from ..column import (_parse_tdisp_format, _fortran_to_python_format,
+                      python_to_tdisp)
+
 from .. import HDUList, PrimaryHDU, BinTableHDU
 
 from ... import fits
 
 from .... import units as u
 from ....table import Table, QTable, NdarrayMixin, Column
+from ....table.table_helpers import simple_table
 from ....tests.helper import catch_warnings
 from ....units.format.fits import UnitScaleError
+from ....utils.exceptions import AstropyUserWarning
 
 from ....coordinates import SkyCoord, Latitude, Longitude, Angle, EarthLocation
 from ....time import Time, TimeDelta
-from ....units import allclose as quantity_allclose
 from ....units.quantity import QuantityInfo
 
 try:
@@ -109,6 +113,20 @@ class TestSingleTable:
         assert t2['a'].unit == u.m
         assert t2['c'].unit == u.km / u.s
 
+    @pytest.mark.parametrize('table_type', (Table, QTable))
+    def test_with_format(self, table_type, tmpdir):
+        filename = str(tmpdir.join('test_with_format.fits'))
+        t1 = table_type(self.data)
+        t1['a'].format = '{:5d}'
+        t1['b'].format = '{:>20}'
+        t1['c'].format = '{:6.2f}'
+        t1.write(filename, overwrite=True)
+        t2 = table_type.read(filename)
+        assert equal_data(t1, t2)
+        assert t2['a'].format == '{:5d}'
+        assert t2['b'].format == '{:>20}'
+        assert t2['c'].format == '{:6.2f}'
+
     def test_masked(self, tmpdir):
         filename = str(tmpdir.join('test_masked.fits'))
         t1 = Table(self.data, masked=True)
@@ -305,6 +323,43 @@ def test_scale_error():
     assert exc.value.args[0] == "The column 'a' could not be stored in FITS format because it has a scale '(1.2)' that is not recognized by the FITS standard. Either scale the data or change the units."
 
 
+@pytest.mark.parametrize('tdisp_str, format_return',
+                         [('EN10.5', ('EN', '10', '5', None)),
+                          ('F6.2', ('F', '6', '2', None)),
+                          ('B5.10', ('B', '5', '10', None)),
+                          ('E10.5E3', ('E', '10', '5', '3')),
+                          ('A21', ('A', '21', None, None))])
+def test_parse_tdisp_format(tdisp_str, format_return):
+    assert _parse_tdisp_format(tdisp_str) == format_return
+
+
+@pytest.mark.parametrize('tdisp_str, format_str_return',
+                         [('G15.4E2', '{:15.4g}'),
+                          ('Z5.10', '{:5x}'),
+                          ('I6.5', '{:6d}'),
+                          ('L8', '{:>8}'),
+                          ('E20.7', '{:20.7e}')])
+def test_fortran_to_python_format(tdisp_str, format_str_return):
+    assert _fortran_to_python_format(tdisp_str) == format_str_return
+
+
+@pytest.mark.parametrize('fmt_str, tdisp_str',
+                         [('{:3d}', 'I3'),
+                          ('3d', 'I3'),
+                          ('7.3f', 'F7.3'),
+                          ('{:>4}', 'A4'),
+                          ('{:7.4f}', 'F7.4'),
+                          ('%5.3g', 'G5.3'),
+                          ('%10s', 'A10'),
+                          ('%.4f', 'F13.4')])
+def test_python_to_tdisp(fmt_str, tdisp_str):
+    assert python_to_tdisp(fmt_str) == tdisp_str
+
+
+def test_logical_python_to_tdisp():
+    assert python_to_tdisp('{:>7}', logical_dtype=True) == 'L7'
+
+
 def test_bool_column(tmpdir):
     """
     Regression test for https://github.com/astropy/astropy/issues/1953
@@ -367,7 +422,9 @@ def test_convert_comment_convention(tmpdir):
     Regression test for https://github.com/astropy/astropy/issues/6079
     """
     filename = os.path.join(DATA, 'stddata.fits')
-    t = Table.read(filename)
+    with pytest.warns(AstropyUserWarning, catch='hdu= was not specified but '
+                      'multiple tables are present'):
+        t = Table.read(filename)
 
     assert t.meta['comments'] == [
         '',
@@ -416,7 +473,7 @@ el2 = EarthLocation(x=[1, 2] * u.km, y=[3, 4] * u.km, z=[5, 6] * u.km)
 sc = SkyCoord([1, 2], [3, 4], unit='deg,deg', frame='fk4',
               obstime='J1990.5')
 scc = sc.copy()
-scc.representation = 'cartesian'
+scc.representation_type = 'cartesian'
 tm = Time([2450814.5, 2450815.5], format='jd', scale='tai', location=el)
 
 
@@ -440,9 +497,9 @@ compare_attrs = {
     'c2': ['data'],
     'tm': time_attrs,
     'dt': ['shape', 'value', 'format', 'scale'],
-    'sc': ['ra', 'dec', 'representation', 'frame.name'],
-    'scc': ['x', 'y', 'z', 'representation', 'frame.name'],
-    'scd': ['ra', 'dec', 'distance', 'representation', 'frame.name'],
+    'sc': ['ra', 'dec', 'representation_type', 'frame.name'],
+    'scc': ['x', 'y', 'z', 'representation_type', 'frame.name'],
+    'scd': ['ra', 'dec', 'distance', 'representation_type', 'frame.name'],
     'q': ['value', 'unit'],
     'lon': ['value', 'unit', 'wrap_angle'],
     'lat': ['value', 'unit'],
@@ -525,8 +582,8 @@ def test_fits_mixins_as_one(table_cls, tmpdir):
     assert t.colnames == t2.colnames
 
     # Read directly via fits and confirm column names
-    hdus = fits.open(filename)
-    assert hdus[1].columns.names == serialized_names
+    with fits.open(filename) as hdus:
+        assert hdus[1].columns.names == serialized_names
 
 
 @pytest.mark.skipif('not HAS_YAML')
@@ -591,11 +648,45 @@ def test_info_attributes_with_no_mixins(tmpdir):
     filename = str(tmpdir.join('test.fits'))
     t = Table([[1.0, 2.0]])
     t['col0'].description = 'hello' * 40
-    t['col0'].format = '%8.4f'
+    t['col0'].format = '{:8.4f}'
     t['col0'].meta['a'] = {'b': 'c'}
     t.write(filename, overwrite=True)
 
     t2 = Table.read(filename)
     assert t2['col0'].description == 'hello' * 40
-    assert t2['col0'].format == '%8.4f'
+    assert t2['col0'].format == '{:8.4f}'
     assert t2['col0'].meta['a'] == {'b': 'c'}
+
+
+@pytest.mark.skipif('not HAS_YAML')
+@pytest.mark.parametrize('method', ['set_cols', 'names', 'class'])
+def test_round_trip_masked_table_serialize_mask(tmpdir, method):
+    """
+    Same as previous test but set the serialize_method to 'data_mask' so mask is
+    written out and the behavior is all correct.
+    """
+    filename = str(tmpdir.join('test.fits'))
+
+    t = simple_table(masked=True)  # int, float, and str cols with one masked element
+
+    if method == 'set_cols':
+        for col in t.itercols():
+            col.info.serialize_method['fits'] = 'data_mask'
+        t.write(filename)
+    elif method == 'names':
+        t.write(filename, serialize_method={'a': 'data_mask', 'b': 'data_mask',
+                                            'c': 'data_mask'})
+    elif method == 'class':
+        t.write(filename, serialize_method='data_mask')
+
+    t2 = Table.read(filename)
+    assert t2.masked is True
+    assert t2.colnames == t.colnames
+    for name in t2.colnames:
+        assert np.all(t2[name].mask == t[name].mask)
+        assert np.all(t2[name] == t[name])
+
+        # Data under the mask round-trips also (unmask data to show this).
+        t[name].mask = False
+        t2[name].mask = False
+        assert np.all(t2[name] == t[name])
