@@ -43,6 +43,8 @@ class TestConvenience(FitsTestCase):
         header = fits.getheader(f)
         assert not f.closed
 
+        f.close()  # Close it now
+
     def test_table_to_hdu(self):
         table = Table([[1, 2, 3], ['a', 'b', 'c'], [2.3, 4.5, 6.7]],
                       names=['a', 'b', 'c'], dtype=['i', 'U1', 'f'])
@@ -154,3 +156,10 @@ class TestConvenience(FitsTestCase):
         # test with datafile
         fits.tabledump(self.temp('tb.fits'), datafile=self.temp('test_tb.txt'))
         assert os.path.isfile(self.temp('test_tb.txt'))
+
+    @pytest.mark.parametrize('mode', ['wb', 'wb+', 'ab', 'ab+'])
+    def test_append_filehandle(self, tmpdir, mode):
+
+        append_file = tmpdir.join('append.fits')
+        with append_file.open(mode) as handle:
+            fits.append(filename=handle, data=np.ones((4, 4)))
