@@ -5,7 +5,7 @@ import numpy as np
 
 from astropy import table
 
-asdf = pytest.importorskip('asdf', minversion='2.0.0')
+asdf = pytest.importorskip('asdf', minversion='2.0.0.dev0')
 from asdf.tests import helpers
 
 
@@ -115,7 +115,7 @@ table: !core/table
     buff = helpers.yaml_to_asdf(yaml)
 
     with pytest.raises(ValueError):
-        with asdf.AsdfFile.open(buff) as ff:
+        with asdf.open(buff) as ff:
             pass
 
 
