@@ -9,7 +9,7 @@ from astropy.modeling import functional_models
 from ...types import AstropyAsdfType
 
 
-__all__ = ['TransformType', 'IdentityType', 'ConstantType', 'DomainType']
+__all__ = ['TransformType', 'IdentityType', 'ConstantType']
 
 
 class TransformType(AstropyAsdfType):
@@ -25,21 +25,11 @@ class TransformType(AstropyAsdfType):
         if 'name' in node:
             model = model.rename(node['name'])
 
-        # TODO: Remove domain in a later version.
-        if 'domain' in node:
-            model.bounding_box = cls._domain_to_bounding_box(node['domain'])
-        elif 'bounding_box' in node:
-            model.bounding_box = node['bounding_box']
+        if 'bounding_box' in node:
+            model.bounding_box = yamlutil.tagged_tree_to_custom_tree(node['bounding_box'], ctx)
 
         return model
 
-    @classmethod
-    def _domain_to_bounding_box(cls, domain):
-        bb = tuple([(item['lower'], item['upper']) for item in domain])
-        if len(bb) == 1:
-            bb = bb[0]
-        return bb
-
     @classmethod
     def from_tree_transform(cls, node, ctx):
         raise NotImplementedError(
@@ -70,7 +60,7 @@ class TransformType(AstropyAsdfType):
                 bb = list(bb)
             else:
                 bb = [list(item) for item in model.bounding_box]
-            node['bounding_box'] = bb
+            node['bounding_box'] = yamlutil.custom_tree_to_tagged_tree(bb, ctx)
 
 
     @classmethod
@@ -129,24 +119,10 @@ class ConstantType(TransformType):
         }
 
 
-class DomainType(AstropyAsdfType):
-    # TODO: Is this used anywhere? Can it be removed?
-    name = "transform/domain"
-    version = '1.0.0'
-
-    @classmethod
-    def from_tree(cls, node, ctx):
-        return node
-
-    @classmethod
-    def to_tree(cls, data, ctx):
-        return data
-
-
 class GenericModel(mappings.Mapping):
     def __init__(self, n_inputs, n_outputs):
         mapping = tuple(range(n_inputs))
-        super(GenericModel, self).__init__(mapping)
+        super().__init__(mapping)
         self._outputs = tuple('x' + str(idx) for idx in range(self.n_outputs + 1))
 
 
