@@ -6,7 +6,7 @@ import pytest
 
 from astropy import units as u
 
-asdf = pytest.importorskip('asdf', minversion='2.0.0')
+asdf = pytest.importorskip('asdf', minversion='2.0.0.dev0')
 from asdf.tests import helpers
 
 
@@ -18,12 +18,12 @@ unit: !unit/unit-1.0.0 "2.1798721  10-18kg m2 s-2"
     """
 
     buff = helpers.yaml_to_asdf(yaml)
-    with asdf.AsdfFile.open(buff) as ff:
+    with asdf.open(buff) as ff:
         assert ff.tree['unit'].is_equivalent(u.Ry)
 
         buff2 = io.BytesIO()
         ff.write_to(buff2)
 
     buff2.seek(0)
-    with asdf.AsdfFile.open(buff2) as ff:
+    with asdf.open(buff2) as ff:
         assert ff.tree['unit'].is_equivalent(u.Ry)
