@@ -429,9 +429,8 @@ class TestSubclass:
         assert mt.colnames == t.colnames
         assert type(t) is MTable
         assert t['a'].unit == u.m
+        assert t['a'].format == '{:13.4f}'
         if HAS_YAML:
-            assert t['a'].format == '.4f'
             assert t['a'].description == 'hello'
         else:
-            assert t['a'].format is None
             assert t['a'].description is None
