@@ -389,8 +389,7 @@ class Chebyshev1D(PolynomialModel):
         return np.rollaxis(v, 0, v.ndim)
 
     def prepare_inputs(self, x, **kwargs):
-        inputs, format_info = \
-                super(PolynomialModel, self).prepare_inputs(x, **kwargs)
+        inputs, format_info = super().prepare_inputs(x, **kwargs)
 
         x = inputs[0]
 
@@ -496,8 +495,7 @@ class Hermite1D(PolynomialModel):
         return np.rollaxis(v, 0, v.ndim)
 
     def prepare_inputs(self, x, **kwargs):
-        inputs, format_info = \
-                super(PolynomialModel, self).prepare_inputs(x, **kwargs)
+        inputs, format_info = super().prepare_inputs(x, **kwargs)
 
         x = inputs[0]
 
@@ -701,8 +699,7 @@ class Legendre1D(PolynomialModel):
             name=name, meta=meta, **params)
 
     def prepare_inputs(self, x, **kwargs):
-        inputs, format_info = \
-                super(PolynomialModel, self).prepare_inputs(x, **kwargs)
+        inputs, format_info = super().prepare_inputs(x, **kwargs)
 
         x = inputs[0]
 
