@@ -18,6 +18,18 @@ except ImportError:
     HAS_SCIPY = False
 
 
+def test_sigma_constant():
+    """
+    Test that the GAUSSIAN_SIGMA_TO_FWHM constant matches the
+    gaussian_sigma_to_fwhm constant in astropy.stats. We define
+    it manually in astropy.modeling to avoid importing from
+    astropy.stats.
+    """
+    from ...stats.funcs import gaussian_sigma_to_fwhm
+    from ..functional_models import GAUSSIAN_SIGMA_TO_FWHM
+    assert gaussian_sigma_to_fwhm == GAUSSIAN_SIGMA_TO_FWHM
+
+
 def test_Trapezoid1D():
     """Regression test for https://github.com/astropy/astropy/issues/1721"""
 
@@ -171,6 +183,11 @@ def test_Scale_inverse():
     assert_allclose(m.inverse(m(6.789)), 6.789)
 
 
+def test_Multiply_inverse():
+    m = models.Multiply(1.2345)
+    assert_allclose(m.inverse(m(6.789)), 6.789)
+
+
 def test_Shift_inverse():
     m = models.Shift(1.2345)
     assert_allclose(m.inverse(m(6.789)), 6.789)
@@ -205,10 +222,11 @@ def test_Shift_model_set_linear_fit():
     assert_allclose(fitted_model.parameters, [0.1, -0.2], atol=1e-15)
 
 
-def test_Scale_model_set_linear_fit():
+@pytest.mark.parametrize('Model', (models.Scale, models.Multiply))
+def test_Scale_model_set_linear_fit(Model):
     """Test linear fitting of Scale model (#6103)."""
 
-    init_model = models.Scale(factor=[0, 0], n_models=2)
+    init_model = Model(factor=[0, 0], n_models=2)
 
     x = np.arange(-3, 7)
     yy = np.array([1.15*x, 0.96*x])
