@@ -3,10 +3,9 @@
 This module tests fitting and model evaluation with various inputs
 """
 
-
 import pytest
 import numpy as np
-from numpy.testing.utils import assert_allclose
+from numpy.testing import assert_allclose
 
 from .. import models
 from .. import fitting
