@@ -2,7 +2,7 @@
 
 import pytest
 import numpy as np
-from numpy.testing.utils import assert_allclose, assert_array_equal
+from numpy.testing import assert_allclose, assert_array_equal
 
 from ..fitting import LevMarLSQFitter
 from ..models import Shift, Rotation2D, Gaussian1D, Identity, Mapping
