@@ -473,26 +473,40 @@ class _BoundingBox(tuple):
         nd = model.n_inputs
 
         if nd == 1:
-            if (not isiterable(bounding_box)
-                    or np.shape(bounding_box) not in ((2,), (1, 2))):
-                raise ValueError(
-                    "Bounding box for {0} model must be a sequence of length "
-                    "2 consisting of a lower and upper bound, or a 1-tuple "
-                    "containing such a sequence as its sole element.".format(
-                        model.name))
+            MESSAGE = "Bounding box for {0} model must be a sequence of length "
+            "2 consisting of a lower and upper bound, or a 1-tuple "
+            "containing such a sequence as its sole element.".format(model.name)
+
+            try:
+                valid_shape = np.shape(bounding_box) in ((2,), (1, 2))
+            except TypeError:
+                # np.shape does not work with lists of Quantities
+                valid_shape = np.shape([b.to_value() for b in bounding_box]) in ((2,), (1, 2))
+            except ValueError:
+                raise ValueError(MESSAGE)
+
+            if not isiterable(bounding_box) or not valid_shape:
+                raise ValueError(MESSAGE)
 
             if len(bounding_box) == 1:
                 return cls((tuple(bounding_box[0]),))
             else:
                 return cls(tuple(bounding_box))
         else:
-            if (not isiterable(bounding_box)
-                    or np.shape(bounding_box) != (nd, 2)):
-                raise ValueError(
-                    "Bounding box for {0} model must be a sequence of length "
-                    "{1} (the number of model inputs) consisting of pairs of "
-                    "lower and upper bounds for those inputs on which to "
-                    "evaluate the model.".format(model.name, nd))
+            MESSAGE = "Bounding box for {0} model must be a sequence of length "
+            "{1} (the number of model inputs) consisting of pairs of "
+            "lower and upper bounds for those inputs on which to "
+            "evaluate the model.".format(model.name, nd)
+
+            try:
+                valid_shape = all([len(i) == 2 for i in bounding_box])
+            except TypeError:
+                valid_shape = False
+            if len(bounding_box) != nd:
+                valid_shape = False
+
+            if not isiterable(bounding_box) or not valid_shape:
+                    raise ValueError(MESSAGE)
 
             return cls(tuple(bounds) for bounds in bounding_box)
 
