@@ -9,9 +9,22 @@ be easily provided by a single array.
 """
 
 from .nddata import *
+from .nddata_base import *
+from .nddata_withmixins import *
 from .nduncertainty import *
 from .flag_collection import *
 
+from .decorators import *
+
+from .mixins.ndarithmetic import *
+from .mixins.ndslicing import *
+from .mixins.ndio import *
+
+from .compat import *
+from .utils import *
+from .ccddata import *
+from .bitmask import *
+
 from .. import config as _config
 
 
@@ -34,4 +47,5 @@ class Conf(_config.ConfigNamespace):
         'that data values/uncertainties are not scaled with the unit change.'
     )
 
+
 conf = Conf()
