@@ -7,7 +7,9 @@ import numpy as np
 import pytest
 
 from ...io import fits
-from ..nduncertainty import StdDevUncertainty, MissingDataAssociationException
+from ..nduncertainty import (
+    StdDevUncertainty, MissingDataAssociationException, VarianceUncertainty,
+    InverseVariance)
 from ... import units as u
 from ... import log
 from ...wcs import WCS, FITSFixedWarning
@@ -34,11 +36,11 @@ DEFAULT_DATA_SCALE = 1.0
 
 
 def value_from_markers(key, request):
-    try:
-        val = request.keywords[key].args[0]
-    except KeyError:
-        val = DEFAULTS[key]
-    return val
+    m = request.node.get_closest_marker(key)
+    if m is not None:
+        return m.args[0]
+    else:
+        return DEFAULTS[key]
 
 
 @pytest.fixture
@@ -468,15 +470,19 @@ def test_arithmetic_overload_ccddata_operand(ccd_data):
     assert len(result.meta) == 0
     np.testing.assert_array_equal(result.data,
                                   2 * ccd_data.data)
-    np.testing.assert_array_equal(result.uncertainty.array,
-                                  np.sqrt(2) * ccd_data.uncertainty.array)
+    np.testing.assert_array_almost_equal_nulp(
+        result.uncertainty.array,
+        np.sqrt(2) * ccd_data.uncertainty.array
+    )
 
     result = ccd_data.subtract(operand)
     assert len(result.meta) == 0
     np.testing.assert_array_equal(result.data,
                                   0 * ccd_data.data)
-    np.testing.assert_array_equal(result.uncertainty.array,
-                                  np.sqrt(2) * ccd_data.uncertainty.array)
+    np.testing.assert_array_almost_equal_nulp(
+        result.uncertainty.array,
+        np.sqrt(2) * ccd_data.uncertainty.array
+    )
 
     result = ccd_data.multiply(operand)
     assert len(result.meta) == 0
@@ -809,13 +815,34 @@ def test_write_read_multiextensionfits_mask_default(ccd_data, tmpdir):
     np.testing.assert_array_equal(ccd_data.mask, ccd_after.mask)
 
 
-def test_write_read_multiextensionfits_uncertainty_default(ccd_data, tmpdir):
+@pytest.mark.parametrize(
+    'uncertainty_type',
+    [StdDevUncertainty, VarianceUncertainty, InverseVariance])
+def test_write_read_multiextensionfits_uncertainty_default(
+        ccd_data, tmpdir, uncertainty_type):
     # Test that if a uncertainty is present it is saved and loaded by default.
-    ccd_data.uncertainty = StdDevUncertainty(ccd_data.data * 10)
+    ccd_data.uncertainty = uncertainty_type(ccd_data.data * 10)
     filename = tmpdir.join('afile.fits').strpath
     ccd_data.write(filename)
     ccd_after = CCDData.read(filename)
     assert ccd_after.uncertainty is not None
+    assert type(ccd_after.uncertainty) is uncertainty_type
+    np.testing.assert_array_equal(ccd_data.uncertainty.array,
+                                  ccd_after.uncertainty.array)
+
+
+@pytest.mark.parametrize(
+    'uncertainty_type',
+    [StdDevUncertainty, VarianceUncertainty, InverseVariance])
+def test_write_read_multiextensionfits_uncertainty_different_uncertainty_key(
+        ccd_data, tmpdir, uncertainty_type):
+    # Test that if a uncertainty is present it is saved and loaded by default.
+    ccd_data.uncertainty = uncertainty_type(ccd_data.data * 10)
+    filename = tmpdir.join('afile.fits').strpath
+    ccd_data.write(filename, key_uncertainty_type='Blah')
+    ccd_after = CCDData.read(filename, key_uncertainty_type='Blah')
+    assert ccd_after.uncertainty is not None
+    assert type(ccd_after.uncertainty) is uncertainty_type
     np.testing.assert_array_equal(ccd_data.uncertainty.array,
                                   ccd_after.uncertainty.array)
 
