@@ -19,6 +19,7 @@ from warnings import warn
 
 from ..utils.decorators import deprecated_renamed_argument
 from ..utils import isiterable
+from . import _stats
 
 
 __all__ = ['gaussian_fwhm_to_sigma', 'gaussian_sigma_to_fwhm',
@@ -1265,6 +1266,12 @@ def kuiper_false_positive_probability(D, N):
     fpp : float
         The probability of a score this large arising from the null hypothesis.
 
+    Notes
+    -----
+    Eq 7 of Paltani 2004 appears to incorrectly quote the original formula
+    (Stephens 1965). This function implements the original formula, as it
+    produces a result closer to Monte Carlo simulations.
+
     References
     ----------
 
@@ -1272,6 +1279,9 @@ def kuiper_false_positive_probability(D, N):
            Kuiper's test. Application to the ROSAT PSPC archive",
            Astronomy and Astrophysics, v.240, p.789-790, 2004.
 
+    .. [2] Stephens, M. A., "The goodness-of-fit statistic VN: distribution
+           and significance points", Biometrika, v.52, p.309, 1965.
+
     """
     try:
         from scipy.special import factorial, comb
@@ -1287,42 +1297,29 @@ def kuiper_false_positive_probability(D, N):
         return 1. - factorial(N) * (D - 1. / N)**(N - 1)
     elif D < 3. / N:
         k = -(N * D - 1.) / 2.
-        r = np.sqrt(k**2 - (N * D - 2.) / 2.)
+        r = np.sqrt(k**2 - (N * D - 2.)**2 / 2.)
         a, b = -k + r, -k - r
-        return 1. - factorial(N - 1) * (b**(N - 1.) * (1. - a) -
-                                        a**(N - 1.) * (1. - b)) / float(N)**(N - 2) * (b - a)
+        return 1 - (factorial(N - 1) * (b**(N - 1) * (1 - a) - a**(N - 1) * (1 - b))
+                    / N**(N - 2) / (b - a))
     elif (D > 0.5 and N % 2 == 0) or (D > (N - 1.) / (2. * N) and N % 2 == 1):
-        def T(t):
-            y = D + t / float(N)
-            return y**(t - 3) * (y**3 * N - y**2 * t * (3. - 2. /
-                                                        N) / N - t * (t - 1) * (t - 2) / float(N)**2)
-        s = 0.
         # NOTE: the upper limit of this sum is taken from Stephens 1965
-        for t in range(int(np.floor(N * (1 - D))) + 1):
-            term = T(t) * comb(N, t) * (1 - D - t / float(N))**(N - t - 1)
-            s += term
-        return s
+        t = np.arange(np.floor(N * (1 - D)) + 1)
+        y = D + t / N
+        Tt = y**(t - 3) * (y**3 * N
+                           - y**2 * t * (3 - 2 / N)
+                           + y * t * (t - 1) * (3 - 2 / N) / N
+                           - t * (t - 1) * (t - 2) / N**2)
+        term = Tt * comb(N, t) * (1 - D - t / N)**(N - t - 1)
+        return term.sum()
     else:
         z = D * np.sqrt(N)
-        S1 = 0.
-        term_eps = 1e-12
-        abs_eps = 1e-100
-        for m in itertools.count(1):
-            T1 = 2. * (4. * m**2 * z**2 - 1.) * np.exp(-2. * m**2 * z**2)
-            so = S1
-            S1 += T1
-            if np.abs(S1 - so) / (np.abs(S1) + np.abs(so)
-                                  ) < term_eps or np.abs(S1 - so) < abs_eps:
-                break
-        S2 = 0.
-        for m in itertools.count(1):
-            T2 = m**2 * (4. * m**2 * z**2 - 3.) * np.exp(-2 * m**2 * z**2)
-            so = S2
-            S2 += T2
-            if np.abs(S2 - so) / (np.abs(S2) + np.abs(so)
-                                  ) < term_eps or np.abs(S1 - so) < abs_eps:
-                break
-        return S1 - 8 * D / (3. * np.sqrt(N)) * S2
+        # When m*z>18.82 (sqrt(-log(finfo(double))/2)), exp(-2m**2z**2)
+        # underflows.  Cutting off just before avoids triggering a (pointless)
+        # underflow warning if `under="warn"`.
+        ms = np.arange(1, 18.82 / z)
+        S1 = (2 * (4 * ms**2 * z**2 - 1) * np.exp(-2 * ms**2 * z**2)).sum()
+        S2 = (ms**2 * (4 * ms**2 * z**2 - 3) * np.exp(-2 * ms**2 * z**2)).sum()
+        return S1 - 8 * D / 3 * S2
 
 
 def kuiper(data, cdf=lambda x: x, args=()):
@@ -1421,18 +1418,19 @@ def kuiper_two(data1, data2):
         The fpp is quite approximate, especially for small samples.
 
     """
-    data1, data2 = np.sort(data1), np.sort(data2)
-
-    if len(data2) < len(data1):
-        data1, data2 = data2, data1
-
-    # this could be more efficient
-    cdfv1 = np.searchsorted(data2, data1) / float(len(data2))
-    # this could be more efficient
-    cdfv2 = np.searchsorted(data1, data2) / float(len(data1))
-    D = (np.amax(cdfv1 - np.arange(len(data1)) / float(len(data1))) +
-         np.amax(cdfv2 - np.arange(len(data2)) / float(len(data2))))
-
+    data1 = np.sort(data1)
+    data2 = np.sort(data2)
+    n1, = data1.shape
+    n2, = data2.shape
+    common_type = np.find_common_type([], [data1.dtype, data2.dtype])
+    if not (np.issubdtype(common_type, np.number)
+            and not np.issubdtype(common_type, np.complexfloating)):
+        raise ValueError('kuiper_two only accepts real inputs')
+    # nans, if any, are at the end after sorting.
+    if np.isnan(data1[-1]) or np.isnan(data2[-1]):
+        raise ValueError('kuiper_two only accepts non-nan inputs')
+    D = _stats.ks_2samp(np.asarray(data1, common_type),
+                        np.asarray(data2, common_type))
     Ne = len(data1) * len(data2) / float(len(data1) + len(data2))
     return D, kuiper_false_positive_probability(D, Ne)
 
