@@ -2,7 +2,6 @@
 
 import pytest
 import numpy as np
-from numpy.random import randn, normal
 from numpy.testing import assert_allclose, assert_array_almost_equal_nulp
 
 from ..biweight import (biweight_location, biweight_scale,
@@ -15,11 +14,45 @@ from ...utils.misc import NumpyRNGContext
 def test_biweight_location():
     with NumpyRNGContext(12345):
         # test that it runs
-        randvar = randn(10000)
+        randvar = np.random.randn(10000)
         cbl = biweight_location(randvar)
         assert abs(cbl - 0) < 1e-2
 
 
+def test_biweight_location_constant():
+    cbl = biweight_location(np.ones((10, 5)))
+    assert cbl == 1.
+
+
+def test_biweight_location_constant_axis_2d():
+    shape = (10, 5)
+    data = np.ones(shape)
+    cbl = biweight_location(data, axis=0)
+    assert_allclose(cbl, np.ones(shape[1]))
+    cbl = biweight_location(data, axis=1)
+    assert_allclose(cbl, np.ones(shape[0]))
+
+    val1 = 100.
+    val2 = 2.
+    data = np.arange(50).reshape(10, 5)
+    data[2] = val1
+    data[7] = val2
+    cbl = biweight_location(data, axis=1)
+    assert_allclose(cbl[2], val1)
+    assert_allclose(cbl[7], val2)
+
+
+def test_biweight_location_constant_axis_3d():
+    shape = (10, 5, 2)
+    data = np.ones(shape)
+    cbl = biweight_location(data, axis=0)
+    assert_allclose(cbl, np.ones((shape[1], shape[2])))
+    cbl = biweight_location(data, axis=1)
+    assert_allclose(cbl, np.ones((shape[0], shape[2])))
+    cbl = biweight_location(data, axis=2)
+    assert_allclose(cbl, np.ones((shape[0], shape[1])))
+
+
 def test_biweight_location_small():
     cbl = biweight_location([1, 3, 5, 500, 2])
     assert abs(cbl - 2.745) < 1e-3
@@ -30,7 +63,7 @@ def test_biweight_location_axis():
     with NumpyRNGContext(12345):
         ny = 100
         nx = 200
-        data = normal(5, 2, (ny, nx))
+        data = np.random.normal(5, 2, (ny, nx))
 
         bw = biweight_location(data, axis=0)
         bwi = []
@@ -53,7 +86,7 @@ def test_biweight_location_axis_3d():
         nz = 3
         ny = 4
         nx = 5
-        data = normal(5, 2, (nz, ny, nx))
+        data = np.random.normal(5, 2, (nz, ny, nx))
         bw = biweight_location(data, axis=0)
         assert bw.shape == (ny, nx)
 
@@ -76,7 +109,7 @@ def test_biweight_scale():
 def test_biweight_midvariance():
     with NumpyRNGContext(12345):
         # test that it runs
-        randvar = randn(10000)
+        randvar = np.random.randn(10000)
         var = biweight_midvariance(randvar)
         assert_allclose(var, 1.0, rtol=0.02)
 
@@ -103,7 +136,7 @@ def test_biweight_midvariance_axis():
     with NumpyRNGContext(12345):
         ny = 100
         nx = 200
-        data = normal(5, 2, (ny, nx))
+        data = np.random.normal(5, 2, (ny, nx))
 
         bw = biweight_midvariance(data, axis=0)
         bwi = []
@@ -126,7 +159,7 @@ def test_biweight_midvariance_axis_3d():
         nz = 3
         ny = 4
         nx = 5
-        data = normal(5, 2, (nz, ny, nx))
+        data = np.random.normal(5, 2, (nz, ny, nx))
         bw = biweight_midvariance(data, axis=0)
         assert bw.shape == (ny, nx)
 
@@ -138,6 +171,38 @@ def test_biweight_midvariance_axis_3d():
         assert_allclose(bw[y], bwi)
 
 
+def test_biweight_midvariance_constant_axis():
+    bw = biweight_midvariance(np.ones((10, 5)))
+    assert bw == 0.0
+
+
+def test_biweight_midvariance_constant_axis_2d():
+    shape = (10, 5)
+    data = np.ones(shape)
+    cbl = biweight_midvariance(data, axis=0)
+    assert_allclose(cbl, np.zeros(shape[1]))
+    cbl = biweight_midvariance(data, axis=1)
+    assert_allclose(cbl, np.zeros(shape[0]))
+
+    data = np.arange(50).reshape(10, 5)
+    data[2] = 100.
+    data[7] = 2.
+    bw = biweight_midvariance(data, axis=1)
+    assert_allclose(bw[2], 0.)
+    assert_allclose(bw[7], 0.)
+
+
+def test_biweight_midvariance_constant_axis_3d():
+    shape = (10, 5, 2)
+    data = np.ones(shape)
+    cbl = biweight_midvariance(data, axis=0)
+    assert_allclose(cbl, np.zeros((shape[1], shape[2])))
+    cbl = biweight_midvariance(data, axis=1)
+    assert_allclose(cbl, np.zeros((shape[0], shape[2])))
+    cbl = biweight_midvariance(data, axis=2)
+    assert_allclose(cbl, np.zeros((shape[0], shape[1])))
+
+
 def test_biweight_midcovariance_1d():
     d = [0, 1, 2]
     cov = biweight_midcovariance(d)
@@ -161,6 +226,12 @@ def test_biweight_midcovariance_2d():
                           [-5.19350838, 4.61391501]])
 
 
+def test_biweight_midcovariance_constant():
+    data = np.ones((3, 10))
+    cov = biweight_midcovariance(data)
+    assert_allclose(cov, np.zeros((3, 3)))
+
+
 def test_biweight_midcovariance_midvariance():
     """
     Test that biweight_midcovariance diagonal elements agree with
