@@ -25,7 +25,7 @@ def test_sigma_clip():
         # Amazing, I've got the same combination on my luggage!
         randvar = np.random.randn(10000)
 
-        filtered_data = sigma_clip(randvar, sigma=1, iters=2)
+        filtered_data = sigma_clip(randvar, sigma=1, maxiters=2)
 
         assert sum(filtered_data.mask) > 0
         assert sum(~filtered_data.mask) < randvar.size
@@ -33,22 +33,24 @@ def test_sigma_clip():
         # this is actually a silly thing to do, because it uses the
         # standard deviation as the variance, but it tests to make sure
         # these arguments are actually doing something
-        filtered_data2 = sigma_clip(randvar, sigma=1, iters=2, stdfunc=np.var)
+        filtered_data2 = sigma_clip(randvar, sigma=1, maxiters=2,
+                                    stdfunc=np.var)
         assert not np.all(filtered_data.mask == filtered_data2.mask)
 
-        filtered_data3 = sigma_clip(randvar, sigma=1, iters=2,
+        filtered_data3 = sigma_clip(randvar, sigma=1, maxiters=2,
                                     cenfunc=np.mean)
         assert not np.all(filtered_data.mask == filtered_data3.mask)
 
-        # make sure the iters=None method works at all.
-        filtered_data = sigma_clip(randvar, sigma=3, iters=None)
+        # make sure the maxiters=None method works at all.
+        filtered_data = sigma_clip(randvar, sigma=3, maxiters=None)
 
         # test copying
         assert filtered_data.data[0] == randvar[0]
         filtered_data.data[0] += 1.
         assert filtered_data.data[0] != randvar[0]
 
-        filtered_data = sigma_clip(randvar, sigma=3, iters=None, copy=False)
+        filtered_data = sigma_clip(randvar, sigma=3, maxiters=None,
+                                   copy=False)
         assert filtered_data.data[0] == randvar[0]
         filtered_data.data[0] += 1.
         assert filtered_data.data[0] == randvar[0]
@@ -71,7 +73,8 @@ def test_compare_to_scipy_sigmaclip():
 
         randvar = np.random.randn(10000)
 
-        astropyres = sigma_clip(randvar, sigma=3, iters=None, cenfunc=np.mean)
+        astropyres = sigma_clip(randvar, sigma=3, maxiters=None,
+                                cenfunc=np.mean)
         scipyres = stats.sigmaclip(randvar, 3, 3)[0]
 
         assert astropyres.count() == len(scipyres)
@@ -81,7 +84,7 @@ def test_compare_to_scipy_sigmaclip():
 def test_sigma_clip_scalar_mask():
     """Test that the returned mask is not a scalar."""
     data = np.arange(5)
-    result = sigma_clip(data, sigma=100., iters=1)
+    result = sigma_clip(data, sigma=100., maxiters=1)
     assert result.mask.shape != ()
 
 
@@ -89,11 +92,29 @@ def test_sigma_clip_class():
     with NumpyRNGContext(12345):
         data = np.random.randn(100)
         data[10] = 1.e5
-        sobj = SigmaClip(sigma=1, iters=2)
-        sfunc = sigma_clip(data, sigma=1, iters=2)
+        sobj = SigmaClip(sigma=1, maxiters=2)
+        sfunc = sigma_clip(data, sigma=1, maxiters=2)
         assert_equal(sobj(data), sfunc)
 
 
+def test_sigma_clip_mean():
+    with NumpyRNGContext(12345):
+        data = np.random.normal(0., 0.05, (10, 10))
+        data[2, 2] = 1.e5
+        sobj1 = SigmaClip(sigma=1, maxiters=2, cenfunc='mean')
+        sobj2 = SigmaClip(sigma=1, maxiters=2, cenfunc=np.nanmean)
+        assert_equal(sobj1(data), sobj2(data))
+        assert_equal(sobj1(data, axis=0), sobj2(data, axis=0))
+
+
+def test_sigma_clip_invalid_cenfunc_stdfunc():
+    with pytest.raises(ValueError):
+        SigmaClip(cenfunc='invalid')
+
+    with pytest.raises(ValueError):
+        SigmaClip(stdfunc='invalid')
+
+
 def test_sigma_clipped_stats():
     """Test list data with input mask or mask_value (#3268)."""
     # test list data with mask
@@ -104,6 +125,9 @@ def test_sigma_clipped_stats():
     assert isinstance(result[1], float)
     assert result == (1., 1., 0.)
 
+    result2 = sigma_clipped_stats(data, mask=mask, axis=0)
+    assert_equal(result, result2)
+
     # test list data with mask_value
     result = sigma_clipped_stats(data, mask_value=0.)
     assert isinstance(result[1], float)
@@ -150,11 +174,25 @@ def test_invalid_sigma_clip():
     # Pre #4051 if data contains any NaN or infs sigma_clip returns the
     # mask containing `False` only or TypeError if data also contains a
     # masked value.
-
     assert result.mask[2, 2]
     assert result.mask[3, 4]
     assert result.mask[1, 1]
 
+    result2 = sigma_clip(data, axis=0)
+    assert result2.mask[1, 1]
+    assert result2.mask[3, 4]
+
+    result3 = sigma_clip(data, axis=0, copy=False)
+    assert result3.mask[1, 1]
+    assert result3.mask[3, 4]
+
+    # stats along axis with all nans
+    data[0, :] = np.nan     # row of all nans
+    result4, minarr, maxarr = sigma_clip(data, axis=1, masked=False,
+                                         return_bounds=True)
+    assert np.isnan(minarr[0])
+    assert np.isnan(maxarr[0])
+
 
 def test_sigmaclip_negative_axis():
     """Test that dimensions are expanded correctly even if axis is negative."""
@@ -197,7 +235,7 @@ def test_sigma_clip_axis_tuple_3D():
     """Test sigma clipping over a subset of axes (issue #7227).
     """
 
-    data = np.sin(0.78 * np.arange(27)).reshape(3,3,3)
+    data = np.sin(0.78 * np.arange(27)).reshape(3, 3, 3)
     mask = np.zeros_like(data, dtype=np.bool)
 
     data_t = np.rollaxis(data, 1, 0)
@@ -212,6 +250,18 @@ def test_sigma_clip_axis_tuple_3D():
                                       data_plane > mean + maxdev)
 
     # Do the equivalent thing using sigma_clip:
-    result = sigma_clip(data, sigma=1.5, cenfunc=np.mean, iters=1, axis=(0,-1))
+    result = sigma_clip(data, sigma=1.5, cenfunc=np.mean, maxiters=1,
+                        axis=(0, -1))
 
     assert_equal(result.mask, mask)
+
+
+def test_sigmaclip_repr():
+    sigclip = SigmaClip()
+    sigclip_repr = ('SigmaClip(sigma=3.0, sigma_lower=3.0, sigma_upper=3.0,'
+                    ' maxiters=5, cenfunc=')
+    sigclip_str = ('<SigmaClip>\n    sigma: 3.0\n    sigma_lower: 3.0\n'
+                   '    sigma_upper: 3.0\n    maxiters: 5\n    cenfunc: ')
+
+    assert repr(sigclip).startswith(sigclip_repr)
+    assert str(sigclip).startswith(sigclip_str)
