@@ -2,7 +2,8 @@
 import numpy as np
 import pytest
 
-from numpy.testing.utils import assert_allclose
+from numpy.testing import assert_allclose
+
 from ..spatial import RipleysKEstimator
 from ...utils.misc import NumpyRNGContext
 
