@@ -4,7 +4,7 @@ import platform
 import warnings
 
 import numpy as np
-from .index import get_index
+from .index import get_index_by_names
 
 from ..utils.exceptions import AstropyUserWarning
 
@@ -34,6 +34,8 @@ def _table_group_by(table, keys):
     grouped_table : Table object with groups attr set accordingly
     """
     from .table import Table
+    from .serialize import _represent_mixins_as_columns
+
     # Pre-convert string to tuple of strings, or Table to the underlying structured array
     if isinstance(keys, str):
         keys = (keys,)
@@ -45,25 +47,33 @@ def _table_group_by(table, keys):
             if table.masked and np.any(table[name].mask):
                 raise ValueError('Missing values in key column {0!r} are not allowed'.format(name))
 
-        keys = tuple(keys)
-        table_keys = table[keys]
-        grouped_by_table_cols = True  # Grouping keys are columns from the table being grouped
+        # Make a column slice of the table without copying
+        table_keys = table.__class__([table[key] for key in keys], copy=False)
+
+        # If available get a pre-existing index for these columns
+        table_index = get_index_by_names(table, keys)
+        grouped_by_table_cols = True
 
     elif isinstance(keys, (np.ndarray, Table)):
         table_keys = keys
         if len(table_keys) != len(table):
             raise ValueError('Input keys array length {0} does not match table length {1}'
                              .format(len(table_keys), len(table)))
-        grouped_by_table_cols = False  # Grouping key(s) are external
+        table_index = None
+        grouped_by_table_cols = False
 
     else:
-        raise TypeError('Keys input must be string, list, tuple or numpy array, but got {0}'
+        raise TypeError('Keys input must be string, list, tuple, Table or numpy array, but got {0}'
                         .format(type(keys)))
 
+    # If there is not already an available index and table_keys is a Table then ensure
+    # that all cols (including mixins) are in a form that can sorted with the code below.
+    if not table_index and isinstance(table_keys, Table):
+        table_keys = _represent_mixins_as_columns(table_keys)
+
+    # Get the argsort index `idx_sort`, accounting for particulars
     try:
         # take advantage of index internal sort if possible
-        table_index = get_index(table, table_keys) if \
-                      isinstance(table_keys, Table) else None
         if table_index is not None:
             idx_sort = table_index.sorted_data()
         else:
@@ -75,6 +85,8 @@ def _table_group_by(table, keys):
         # sort by default, nor does Windows, while Linux does (or appears to).
         idx_sort = table_keys.argsort()
         stable_sort = platform.system() not in ('Darwin', 'Windows')
+
+    # Finally do the actual sort of table_keys values
     table_keys = table_keys[idx_sort]
 
     # Get all keys
@@ -114,8 +126,10 @@ def column_group_by(column, keys):
     grouped_column : Column object with groups attr set accordingly
     """
     from .table import Table
+    from .serialize import _represent_mixins_as_columns
 
     if isinstance(keys, Table):
+        keys = _represent_mixins_as_columns(keys)
         keys = keys.as_array()
 
     if not isinstance(keys, np.ndarray):
@@ -126,17 +140,7 @@ def column_group_by(column, keys):
         raise ValueError('Input keys array length {0} does not match column length {1}'
                          .format(len(keys), len(column)))
 
-    # take advantage of table or column indices, if possible
-    index = None
-    if isinstance(keys, Table):
-        index = get_index(keys)
-    elif hasattr(keys, 'indices') and keys.indices:
-        index = keys.indices[0]
-
-    if index is not None:
-        idx_sort = index.sorted_data()
-    else:
-        idx_sort = keys.argsort()
+    idx_sort = keys.argsort()
     keys = keys[idx_sort]
 
     # Get all keys
