@@ -59,7 +59,7 @@ class Index:
         create an empty index for purposes of deep copying.
     engine : type, instance, or None
         Indexing engine class to use (from among SortedArray, BST,
-        FastBST, and FastRBT) or actual engine instance.
+        FastBST, FastRBT, and SCEngine) or actual engine instance.
         If the supplied argument is None (by default), use SortedArray.
     unique : bool (defaults to False)
         Whether the values of the index must be unique
@@ -620,7 +620,6 @@ def get_index(table, table_copy):
         Subset of the columns in the table argument
     '''
     cols = set(table_copy.columns)
-    indices = set()
     for column in cols:
         for index in table[column].info.indices:
             if set([x.info.name for x in index.columns]) == cols:
@@ -628,6 +627,27 @@ def get_index(table, table_copy):
     return None
 
 
+def get_index_by_names(table, names):
+    '''
+    Returns an index in ``table`` corresponding to the ``names`` columns or None
+    if no such index exists.
+
+    Parameters
+    ----------
+    table : `Table`
+        Input table
+    nmaes : tuple, list
+        Column names
+    '''
+    names = list(names)
+    for index in table.indices:
+        index_names = [col.info.name for col in index.columns]
+        if index_names == names:
+            return index
+    else:
+        return None
+
+
 class _IndexModeContext:
     '''
     A context manager that allows for special indexing modes, which
