@@ -1,8 +1,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-
-
-from collections import OrderedDict, Mapping, UserDict
+from collections import OrderedDict, UserDict
+from collections.abc import Mapping
 
 import pytest
 import numpy as np
