@@ -19,7 +19,7 @@ try:
 except ImportError:
     pass
 
-from ..units import allclose as _quantity_allclose
+from ..units import allclose as quantity_allclose  # noqa
 from ..utils.exceptions import (AstropyDeprecationWarning,
                                 AstropyPendingDeprecationWarning)
 
@@ -29,7 +29,7 @@ from .runner import TestRunner  # pylint: disable=W0611
 
 __all__ = ['raises', 'enable_deprecations_as_exceptions', 'remote_data',
            'treat_deprecations_as_exceptions', 'catch_warnings',
-           'assert_follows_unicode_guidelines', 'quantity_allclose',
+           'assert_follows_unicode_guidelines',
            'assert_quantity_allclose', 'check_pickling_recovery',
            'pickle_protocol', 'generic_recursive_equality_test']
 
@@ -133,47 +133,34 @@ _modules_to_ignore_on_import = set([
     'setuptools'])
 _warnings_to_ignore_entire_module = set([])
 _warnings_to_ignore_by_pyver = {
-    (3, 5): set([
+    None: set([  # Python version agnostic
         # py.test reads files with the 'U' flag, which is
         # deprecated.
         r"'U' mode is deprecated",
+        # https://github.com/astropy/astropy/pull/7372
+        r"Importing from numpy\.testing\.decorators is deprecated, "
+        r"import from numpy\.testing instead\.",
+        # Deprecation warnings ahead of pytest 4.x
+        r"MarkInfo objects are deprecated"]),
+    (3, 5): set([
         # py.test raised this warning in inspect on Python 3.5.
         # See https://github.com/pytest-dev/pytest/pull/1009
         # Keeping it since e.g. lxml as of 3.8.0 is still calling getargspec()
         r"inspect\.getargspec\(\) is deprecated, use "
-        r"inspect\.signature\(\) instead",
-        # https://github.com/astropy/astropy/pull/7372
-        r"Importing from numpy\.testing\.decorators is deprecated, import from numpy\.testing instead\.",
-        # Deprecation warnings ahead of pytest 4.x
-        r"MarkInfo objects are deprecated"]),
+        r"inspect\.signature\(\) instead"]),
     (3, 6): set([
-        # py.test reads files with the 'U' flag, which is
-        # deprecated.
-        r"'U' mode is deprecated",
         # inspect raises this slightly different warning on Python 3.6-3.7.
         # Keeping it since e.g. lxml as of 3.8.0 is still calling getargspec()
         r"inspect\.getargspec\(\) is deprecated, use "
-        r"inspect\.signature\(\) or inspect\.getfullargspec\(\)",
-        # https://github.com/astropy/astropy/pull/7372
-        r"Importing from numpy\.testing\.decorators is deprecated, import from numpy\.testing instead\.",
-        # Deprecation warnings ahead of pytest 4.x
-        r"MarkInfo objects are deprecated"]),
+        r"inspect\.signature\(\) or inspect\.getfullargspec\(\)"]),
     (3, 7): set([
-        # py.test reads files with the 'U' flag, which is
-        # deprecated.
-        r"'U' mode is deprecated",
         # inspect raises this slightly different warning on Python 3.6-3.7.
         # Keeping it since e.g. lxml as of 3.8.0 is still calling getargspec()
         r"inspect\.getargspec\(\) is deprecated, use "
         r"inspect\.signature\(\) or inspect\.getfullargspec\(\)",
-        # https://github.com/astropy/astropy/pull/7372
-        r"Importing from numpy\.testing\.decorators is deprecated, import from numpy\.testing instead\.",
-        # Deprecation warnings ahead of pytest 4.x
-        r"MarkInfo objects are deprecated",
         # Deprecation warning for collections.abc, fixed in Astropy but still
         # used in lxml, and maybe others
-        r"Using or importing the ABCs from 'collections'",
-    ]),
+        r"Using or importing the ABCs from 'collections'"])
 }
 
 
@@ -204,8 +191,9 @@ def enable_deprecations_as_exceptions(include_astropy_deprecations=True,
 
     warnings_to_ignore_by_pyver : dict
         Dictionary mapping tuple of ``(major, minor)`` Python version to
-        a list of deprecation warning messages to ignore. This is in
-        addition of those already ignored by default
+        a list of deprecation warning messages to ignore.
+        Python version-agnostic warnings should be mapped to `None` key.
+        This is in addition of those already ignored by default
         (see ``_warnings_to_ignore_by_pyver`` values).
 
     """
@@ -287,7 +275,7 @@ def treat_deprecations_as_exceptions():
             warnings.filterwarnings('ignore', category=w, module=m)
 
     for v in _warnings_to_ignore_by_pyver:
-        if sys.version_info[:2] == v:
+        if v is None or sys.version_info[:2] == v:
             for s in _warnings_to_ignore_by_pyver[v]:
                 warnings.filterwarnings("ignore", s, DeprecationWarning)
 
@@ -312,11 +300,11 @@ class catch_warnings(warnings.catch_warnings):
     """
 
     def __init__(self, *classes):
-        super(catch_warnings, self).__init__(record=True)
+        super().__init__(record=True)
         self.classes = classes
 
     def __enter__(self):
-        warning_list = super(catch_warnings, self).__enter__()
+        warning_list = super().__enter__()
         treat_deprecations_as_exceptions()
         if len(self.classes) == 0:
             warnings.simplefilter('always')
@@ -340,7 +328,7 @@ class ignore_warnings(catch_warnings):
     """
 
     def __init__(self, category=None):
-        super(ignore_warnings, self).__init__()
+        super().__init__()
 
         if isinstance(category, type) and issubclass(category, Warning):
             self.category = [category]
@@ -359,7 +347,7 @@ class ignore_warnings(catch_warnings):
         return wrapper
 
     def __enter__(self):
-        retval = super(ignore_warnings, self).__enter__()
+        retval = super().__enter__()
         if self.category is not None:
             for category in self.category:
                 warnings.simplefilter('ignore', category)
@@ -488,11 +476,3 @@ def assert_quantity_allclose(actual, desired, rtol=1.e-7, atol=None,
     from ..units.quantity import _unquantify_allclose_arguments
     np.testing.assert_allclose(*_unquantify_allclose_arguments(
         actual, desired, rtol, atol), **kwargs)
-
-
-# TODO: This is a workaround to preserve API compatibility for the bugfix
-def quantity_allclose(*args, **kwargs):
-    return _quantity_allclose(*args, **kwargs)
-
-
-quantity_allclose.__doc__ = _quantity_allclose.__doc__
