@@ -40,7 +40,7 @@ def pytest_report_header(config):
     # TESTED_VERSIONS can contain the affiliated package version, too
     if len(TESTED_VERSIONS) > 1:
         for pkg, version in TESTED_VERSIONS.items():
-            if pkg != 'Astropy':
+            if pkg not in ['Astropy', 'astropy_helpers']:
                 s = "\nRunning tests with {0} version {1}.\n".format(
                     pkg, version)
     else:
@@ -95,11 +95,15 @@ def pytest_report_header(config):
             s += "{0}: {1}\n".format(module_display, version)
 
     # Helpers version
-    try:
-        from ...version import astropy_helpers_version
-    except ImportError:
-        pass
+    if 'astropy_helpers' in TESTED_VERSIONS:
+        astropy_helpers_version = TESTED_VERSIONS['astropy_helpers']
     else:
+        try:
+            from ...version import astropy_helpers_version
+        except ImportError:
+            astropy_helpers_version = None
+
+    if astropy_helpers_version:
         s += "astropy_helpers: {0}\n".format(astropy_helpers_version)
 
     special_opts = ["remote_data", "pep8"]
