@@ -5,7 +5,7 @@ import operator
 import pytest
 import numpy as np
 
-from .. import Time, TimeDelta, OperandTypeError
+from .. import Time, TimeDelta
 
 
 class TestTimeComparisons():
@@ -19,17 +19,15 @@ class TestTimeComparisons():
         """
         If an incompatible object is compared to a Time object, == should
         return False and != should return True. All other comparison
-        operators should raise an OperandTypeError.
+        operators should raise a TypeError.
         """
         t1 = Time('J2000', scale='utc')
         for op, op_str in ((operator.ge, '>='),
                            (operator.gt, '>'),
                            (operator.le, '<='),
                            (operator.lt, '<')):
-            with pytest.raises(OperandTypeError) as err:
+            with pytest.raises(TypeError) as err:
                 op(t1, None)
-            assert str(err).endswith("Unsupported operand type(s) for {0}: 'Time' and 'NoneType'"
-                                     .format(op_str))
         # Keep == and != as they are specifically meant to test Time.__eq__
         # and Time.__ne__
         assert (t1 == None) is False  # nopep8
@@ -66,7 +64,7 @@ class TestTimeComparisons():
 
     def test_timedelta(self):
         dt = self.t2 - self.t1
-        with pytest.raises(OperandTypeError):
+        with pytest.raises(TypeError):
             self.t1 > dt
         dt_gt_td0 = dt > TimeDelta(0., format='sec')
         assert np.all(dt_gt_td0 == np.array([False, False, False, False, False,
