@@ -53,8 +53,8 @@ def _initialize_module():
         (['lyr'], u.lyr, "Light year"),
         (['alpha'], _si.alpha, "Fine structure constant"),
         ((['AA', 'Å'], ['Angstrom', 'Angstroem']), u.AA, "Angstrom"),
-        (['arcm', 'arcmin'], u.arcminute, "minute of arc"),
-        (['arcs', 'arcsec'], u.arcsecond, "second of arc"),
+        (['arcmin', 'arcm'], u.arcminute, "minute of arc"),
+        (['arcsec', 'arcs'], u.arcsecond, "second of arc"),
         (['atm'], _si.atm, "atmosphere"),
         (['AU', 'au'], u.au, "astronomical unit"),
         (['bar'], u.bar, "bar"),
