@@ -7,6 +7,7 @@ import warnings
 
 # LOCAL
 from ..constants import si as _si
+from ..utils.misc import isiterable
 from . import si
 from . import cgs
 from . import astrophys
@@ -17,10 +18,10 @@ from .core import UnitsError, Unit
 
 __all__ = ['parallax', 'spectral', 'spectral_density', 'doppler_radio',
            'doppler_optical', 'doppler_relativistic', 'mass_energy',
-           'brightness_temperature', 'beam_angular_area',
-           'dimensionless_angles', 'logarithmic', 'temperature',
-           'temperature_energy', 'molar_mass_amu', 'pixel_scale',
-           'plate_scale']
+           'brightness_temperature', 'thermodynamic_temperature',
+           'beam_angular_area', 'dimensionless_angles', 'logarithmic',
+           'temperature', 'temperature_energy', 'molar_mass_amu',
+           'pixel_scale', 'plate_scale', 'with_H0']
 
 
 def dimensionless_angles():
@@ -46,8 +47,23 @@ def parallax():
     Returns a list of equivalence pairs that handle the conversion
     between parallax angle and distance.
     """
+
+    def parallax_converter(x):
+        x = np.asanyarray(x)
+        d = 1 / x
+
+        if isiterable(d):
+            d[d < 0] = np.nan
+            return d
+
+        else:
+            if d < 0:
+                return np.array(np.nan)
+            else:
+                return d
+
     return [
-        (si.arcsecond, astrophys.parsec, lambda x: 1. / x)
+        (si.arcsecond, astrophys.parsec, parallax_converter)
     ]
 
 
@@ -541,6 +557,7 @@ def brightness_temperature(frequency, beam_area=None):
 
         return [(astrophys.Jy/si.sr, si.K, convert_JySr_to_K, convert_K_to_JySr)]
 
+
 def beam_angular_area(beam_area):
     """
     Convert between the ``beam`` unit, which is commonly used to express the area
@@ -558,6 +575,69 @@ def beam_angular_area(beam_area):
             (astrophys.Jy/astrophys.beam, astrophys.Jy/Unit(beam_area)),
            ]
 
+
+def thermodynamic_temperature(frequency, T_cmb=None):
+    r"""Defines the conversion between Jy/beam and "thermodynamic temperature",
+    :math:`T_{CMB}`, in Kelvins.  The thermodynamic temperature is a unit very
+    commonly used in cosmology. See eqn 8 in [1]
+
+    :math:`K_{CMB} \equiv I_\nu / \left(2 k \nu^2 / c^2  f(\nu) \right)`
+
+    with :math:`f(\nu) = \frac{ x^2 e^x}{(e^x - 1 )^2}`
+    where :math:`x = h \nu / k T`
+
+    Parameters
+    ----------
+    frequency : `~astropy.units.Quantity` with spectral units
+        The observed `spectral` equivalent `~astropy.units.Unit` (e.g.,
+        frequency or wavelength)
+    T_cmb :  `~astropy.units.Quantity` with temperature units or None
+        The CMB temperature at z=0.  If `None`, the default cosmology will be
+        used to get this temperature.
+
+    Notes
+    -----
+    For broad band receivers, this conversion do not hold
+    as it highly depends on the frequency
+
+    References
+    ----------
+    .. [1] Planck 2013 results. IX. HFI spectral response
+       https://arxiv.org/abs/1303.5070
+
+    Examples
+    --------
+    Planck HFI 143 GHz::
+
+        >>> from astropy import units as u
+        >>> from astropy.cosmology import Planck15
+        >>> freq = 143 * u.GHz
+        >>> equiv = u.thermodynamic_temperature(freq, Planck15.Tcmb0)
+        >>> (1. * u.mK).to(u.MJy / u.sr, equivalencies=equiv)  # doctest: +FLOAT_CMP
+        <Quantity 0.37993172 MJy / sr>
+
+    """
+    nu = frequency.to(si.GHz, spectral())
+
+    if T_cmb is None:
+        from ..cosmology import default_cosmology
+        T_cmb = default_cosmology.get().Tcmb0
+
+    def f(nu, T_cmb=T_cmb):
+        x = _si.h * nu / _si.k_B / T_cmb
+        return x**2 * np.exp(x) / np.expm1(x)**2
+
+    def convert_Jy_to_K(x_jybm):
+        factor = (f(nu) * 2 * _si.k_B * si.K * nu**2 / _si.c**2).to_value(astrophys.Jy)
+        return x_jybm / factor
+
+    def convert_K_to_Jy(x_K):
+        factor = (astrophys.Jy / (f(nu) * 2 * _si.k_B * nu**2 / _si.c**2)).to_value(si.K)
+        return x_K / factor
+
+    return [(astrophys.Jy/si.sr, si.K, convert_Jy_to_K, convert_K_to_Jy)]
+
+
 def temperature():
     """Convert between Kelvin, Celsius, and Fahrenheit here because
     Unit and CompositeUnit cannot do addition or subtraction properly.
@@ -625,3 +705,29 @@ def plate_scale(platescale):
                          "distance/angle")
 
     return [(si.m, si.radian, lambda d: d*platescale_val, lambda rad: rad/platescale_val)]
+
+
+def with_H0(H0=None):
+    """
+    Convert between quantities with little-h and the equivalent physical units.
+
+    Parameters
+    ----------
+    H0 : `None` or `~astropy.units.Quantity`
+        The value of the Hubble constant to assume. If a `~astropy.units.Quantity`,
+        will assume the quantity *is* ``H0``.  If `None` (default), use the
+        ``H0`` attribute from the default `astropy.cosmology` cosmology.
+
+    References
+    ----------
+    For an illuminating discussion on why you may or may not want to use
+    little-h at all, see https://arxiv.org/pdf/1308.4150.pdf
+    """
+
+    if H0 is None:
+        from .. import cosmology
+        H0 = cosmology.default_cosmology.get().H0
+
+    h100_val_unit = Unit(H0.to((si.km/si.s)/astrophys.Mpc).value/100 * astrophys.littleh)
+
+    return [(h100_val_unit, None)]
