@@ -2,8 +2,7 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import numpy as np
 
-from .. import CompositeUnit, UnitsError, dimensionless_unscaled
-from . import magnitude_zero_points as mag0
+from .. import CompositeUnit, UnitsError, dimensionless_unscaled, photometric
 from .core import FunctionUnitBase, FunctionQuantity
 from .units import dex, dB, mag
 
@@ -117,11 +116,6 @@ class MagUnit(LogUnit):
         By default, this is ``mag``, but this allows one to use an equivalent
         unit such as ``2 mag``.
     """
-    def __init__(self, *args, **kwargs):
-        # Ensure we recognize magnitude zero points here.
-        with mag0.enable():
-            super().__init__(*args, **kwargs)
-
     @property
     def _default_function_unit(self):
         return mag
@@ -319,16 +313,16 @@ dB._function_unit_class = DecibelUnit
 mag._function_unit_class = MagUnit
 
 
-STmag = MagUnit(mag0.ST)
+STmag = MagUnit(photometric.STflux)
 STmag.__doc__ = "ST magnitude: STmag=-21.1 corresponds to 1 erg/s/cm2/A"
 
-ABmag = MagUnit(mag0.AB)
+ABmag = MagUnit(photometric.ABflux)
 ABmag.__doc__ = "AB magnitude: ABmag=-48.6 corresponds to 1 erg/s/cm2/Hz"
 
-M_bol = MagUnit(mag0.Bol)
+M_bol = MagUnit(photometric.Bol)
 M_bol.__doc__ = ("Absolute bolometric magnitude: M_bol=0 corresponds to "
-                 "L_bol0={0}".format(mag0.Bol.si))
+                 "L_bol0={0}".format(photometric.Bol.si))
 
-m_bol = MagUnit(mag0.bol)
+m_bol = MagUnit(photometric.bol)
 m_bol.__doc__ = ("Apparent bolometric magnitude: m_bol=0 corresponds to "
-                 "f_bol0={0}".format(mag0.bol.si))
+                 "f_bol0={0}".format(photometric.bol.si))
