@@ -10,7 +10,7 @@ from numpy.testing import assert_allclose
 
 # LOCAL
 from ... import units as u
-from ... import constants
+from ... import constants, cosmology
 from ...tests.helper import assert_quantity_allclose
 
 
@@ -227,6 +227,12 @@ def test_parallax():
     b = u.au.to(u.arcminute, a, u.parallax())
     assert_allclose(b, 1)
 
+    val = (-1 * u.mas).to(u.pc, u.parallax())
+    assert np.isnan(val.value)
+
+    val = (-1 * u.mas).to_value(u.pc, u.parallax())
+    assert np.isnan(val)
+
 
 def test_parallax2():
     a = u.arcsecond.to(u.pc, [0.1, 2.5], u.parallax())
@@ -574,6 +580,17 @@ def test_beam():
     np.testing.assert_almost_equal(flux_density.value, 13.5425483146382)
 
 
+def test_thermodynamic_temperature():
+    nu = 143 * u.GHz
+    tb = 0.0026320518775281975 * u.K
+    np.testing.assert_almost_equal(
+        tb.value, (1 * u.MJy/u.sr).to_value(
+            u.K, equivalencies=u.thermodynamic_temperature(nu, T_cmb=2.7255 * u.K)))
+    np.testing.assert_almost_equal(
+        1.0, tb.to_value(
+            u.MJy / u.sr, equivalencies=u.thermodynamic_temperature(nu, T_cmb=2.7255 * u.K)))
+
+
 def test_equivalency_context():
     with u.set_enabled_equivalencies(u.dimensionless_angles()):
         phase = u.Quantity(1., u.cycle)
@@ -730,3 +747,26 @@ def test_plate_scale():
 
     assert_quantity_allclose(asec.to(u.mm, u.plate_scale(platescale)), mm)
     assert_quantity_allclose(asec.to(u.mm, u.plate_scale(platescale2)), mm)
+
+
+def test_littleh():
+    H0_70 = 70*u.km/u.s/u.Mpc
+    h100dist = 100 * u.Mpc/u.littleh
+
+    assert_quantity_allclose(h100dist.to(u.Mpc, u.with_H0(H0_70)), 70*u.Mpc)
+
+    # make sure using the default cosmology works
+    H0_default_cosmo = cosmology.default_cosmology.get().H0
+    assert_quantity_allclose(h100dist.to(u.Mpc, u.with_H0()),
+                             H0_default_cosmo.value*u.Mpc)
+
+    # Now try a luminosity scaling
+    h1lum = 1 * u.Lsun * u.littleh**-2
+    assert_quantity_allclose(h1lum.to(u.Lsun, u.with_H0(H0_70)), .49*u.Lsun)
+
+    # And the trickiest one: magnitudes.  Using H0=10 here for the round numbers
+    H0_10 = 10*u.km/u.s/u.Mpc
+    # assume the "true" magnitude M = 12.
+    # Then M - 5*log_10(h)  = M + 5 = 17
+    withlittlehmag = 17 * (u.mag + u.MagUnit(u.littleh**2))
+    assert_quantity_allclose(withlittlehmag.to(u.mag, u.with_H0(H0_10)), 12*u.mag)
