@@ -9,8 +9,9 @@ import numpy as np
 from numpy.testing import assert_allclose
 
 from ... import units as u
+from .. import quantity_helper as qh
+from ..._erfa import ufunc as erfa_ufunc
 from ...tests.helper import raises
-from ...utils.compat import NUMPY_LT_1_13
 
 try:
     import scipy  # pylint: disable=W0611
@@ -51,27 +52,41 @@ def test_testwarn(tw):
         tw.f(*tw.q_in)
 
 
-class TestUfuncCoverage:
-    """Test that we cover all ufunc's"""
-
-    # Ignore possible scipy ufuncs; for scipy in particular, we have support
-    # for some, but for others it still has to be decided whether we can
-    # support them or not.
-    @pytest.mark.skipif(HAS_SCIPY,
-                        reason='scipy.special coverage is incomplete')
+class TestUfuncHelpers:
+    # Note that this test should work even if scipy is present, since
+    # the scipy.special ufuncs are only loaded on demand.
+    # The test passes independently of whether erfa is already loaded
+    # (which will be the case for a full test, since coordinates uses it).
     def test_coverage(self):
+        """Test that we cover all ufunc's"""
 
-        all_extern_ufuncs = set([])
         all_np_ufuncs = set([ufunc for ufunc in np.core.umath.__dict__.values()
-                             if type(ufunc) == np.ufunc])
-        all_extern_ufuncs |= all_np_ufuncs
-        from .. import quantity_helper as qh
+                             if isinstance(ufunc, np.ufunc)])
 
         all_q_ufuncs = (qh.UNSUPPORTED_UFUNCS |
                         set(qh.UFUNC_HELPERS.keys()))
+        # Check that every numpy ufunc is covered.
+        assert all_np_ufuncs - all_q_ufuncs == set()
+        # Check that all ufuncs we cover come from numpy or erfa.
+        # (Since coverage for erfa is incomplete, we do not check
+        # this the other way).
+        all_erfa_ufuncs = set([ufunc for ufunc in erfa_ufunc.__dict__.values()
+                               if isinstance(ufunc, np.ufunc)])
+        assert (all_q_ufuncs - all_np_ufuncs - all_erfa_ufuncs == set())
 
-        assert all_extern_ufuncs - all_q_ufuncs == set([])
-        assert all_q_ufuncs - all_extern_ufuncs == set([])
+    def test_scipy_registered(self):
+        # Should be registered as existing even if scipy is not available.
+        assert 'scipy.special' in qh.UFUNC_HELPERS.modules
+
+    def test_removal_addition(self):
+        assert np.add in qh.UFUNC_HELPERS
+        assert np.add not in qh.UNSUPPORTED_UFUNCS
+        qh.UFUNC_HELPERS[np.add] = None
+        assert np.add not in qh.UFUNC_HELPERS
+        assert np.add in qh.UNSUPPORTED_UFUNCS
+        qh.UFUNC_HELPERS[np.add] = qh.UFUNC_HELPERS[np.subtract]
+        assert np.add in qh.UFUNC_HELPERS
+        assert np.add not in qh.UNSUPPORTED_UFUNCS
 
 
 class TestQuantityTrigonometricFuncs:
@@ -334,12 +349,11 @@ class TestQuantityMathFuncs:
         with pytest.raises(TypeError):
             dividend % u.km
 
-        if hasattr(np, 'divmod'):  # not NUMPY_LT_1_13
-            quotient4, remainder4 = np.divmod(dividend, divisor)
-            assert np.all(quotient4 == quotient)
-            assert np.all(remainder4 == remainder)
-            with pytest.raises(TypeError):
-                np.divmod(dividend, u.km)
+        quotient4, remainder4 = np.divmod(dividend, divisor)
+        assert np.all(quotient4 == quotient)
+        assert np.all(remainder4 == remainder)
+        with pytest.raises(TypeError):
+            np.divmod(dividend, u.km)
 
     def test_sqrt_scalar(self):
         assert np.sqrt(4. * u.m) == 2. * u.m ** 0.5
@@ -362,15 +376,12 @@ class TestQuantityMathFuncs:
         assert np.all(np.reciprocal(np.array([1., 2., 4.]) * u.m)
                       == np.array([1., 0.5, 0.25]) / u.m)
 
-    # heaviside only introduced in numpy 1.13
-    @pytest.mark.skipif("not hasattr(np, 'heaviside')")
     def test_heaviside_scalar(self):
         assert np.heaviside(0. * u.m, 0.5) == 0.5 * u.dimensionless_unscaled
         assert np.heaviside(0. * u.s,
                             25 * u.percent) == 0.25 * u.dimensionless_unscaled
         assert np.heaviside(2. * u.J, 0.25) == 1. * u.dimensionless_unscaled
 
-    @pytest.mark.skipif("not hasattr(np, 'heaviside')")
     def test_heaviside_array(self):
         values = np.array([-1., 0., 0., +1.])
         halfway = np.array([0.75, 0.25, 0.75, 0.25]) * u.dimensionless_unscaled
@@ -565,12 +576,10 @@ class TestQuantityMathFuncs:
 
 class TestInvariantUfuncs:
 
-    # np.positive was only added in numpy 1.13.
     @pytest.mark.parametrize(('ufunc'), [np.absolute, np.fabs,
                                          np.conj, np.conjugate,
                                          np.negative, np.spacing, np.rint,
-                                         np.floor, np.ceil] +
-                             [np.positive] if hasattr(np, 'positive') else [])
+                                         np.floor, np.ceil, np.positive])
     def test_invariant_scalar(self, ufunc):
 
         q_i = 4.7 * u.m
@@ -719,7 +728,7 @@ class TestInplaceUfuncs:
         v_copy = v.copy()
         tmp = v.copy()
         check = v
-        np.modf(v, tmp, v)  # cannot use out1,out2 keywords with numpy 1.7
+        np.modf(v, tmp, v)
         assert check is v
         assert check.unit == u.dimensionless_unscaled
         v2 = v_copy.to(u.dimensionless_unscaled)
@@ -733,17 +742,13 @@ class TestInplaceUfuncs:
         np.modf(v3, v3, tmp)
         assert check3 is v3
         assert check3.unit == u.dimensionless_unscaled
-        # in np<1.13, without __array_ufunc__, one cannot replace input with
+        # And now, with numpy >= 1.13, one can also replace input with
         # first output when scaling
         v4 = v_copy.copy()
-        if NUMPY_LT_1_13:
-            with pytest.raises(TypeError):
-                np.modf(v4, v4, tmp)
-        else:
-            check4 = v4
-            np.modf(v4, v4, tmp)
-            assert check4 is v4
-            assert check4.unit == u.dimensionless_unscaled
+        check4 = v4
+        np.modf(v4, v4, tmp)
+        assert check4 is v4
+        assert check4.unit == u.dimensionless_unscaled
 
     @pytest.mark.parametrize(('value'), [1., np.arange(10.)])
     def test_two_argument_ufunc_inplace_1(self, value):
@@ -789,7 +794,6 @@ class TestInplaceUfuncs:
         assert_allclose(s.value, np.arctan2(1., 2.))
         assert s.unit is u.radian
 
-    @pytest.mark.skipif(NUMPY_LT_1_13, reason="numpy >=1.13 required.")
     @pytest.mark.parametrize(('value'), [1., np.arange(10.)])
     def test_two_argument_two_output_ufunc_inplace(self, value):
         v = value * u.m
@@ -844,7 +848,6 @@ class TestInplaceUfuncs:
             a4 += u.Quantity(10, u.mm, dtype=np.int64)
 
 
-@pytest.mark.xfail("NUMPY_LT_1_13")
 class TestUfuncAt:
     """Test that 'at' method for ufuncs (calculates in-place at given indices)
 
@@ -950,7 +953,6 @@ class TestUfuncReduceReduceatAccumulate:
         with pytest.raises(ValueError):
             np.sin.reduceat(s, i)
 
-    @pytest.mark.xfail("NUMPY_LT_1_13")
     def test_two_argument_ufunc_reduce_accumulate(self):
         s = np.arange(10.) * u.m
         i = np.array([0, 5, 1, 6])
@@ -1021,7 +1023,6 @@ class TestUfuncOuter:
         with pytest.raises(ValueError):
             np.sin.outer(s)
 
-    @pytest.mark.xfail("NUMPY_LT_1_13")
     def test_two_argument_ufunc_outer(self):
         s1 = np.arange(10.) * u.m
         s2 = np.arange(2.) * u.s
@@ -1053,6 +1054,12 @@ class TestUfuncOuter:
 if HAS_SCIPY:
     from scipy import special as sps
 
+    def test_scipy_registration():
+        """Check that scipy gets loaded upon first use."""
+        assert sps.erf not in qh.UFUNC_HELPERS
+        sps.erf(1. * u.percent)
+        assert sps.erf in qh.UFUNC_HELPERS
+
     class TestScipySpecialUfuncs:
 
         erf_like_ufuncs = (
