@@ -63,24 +63,13 @@ def test_find_mod_objs():
     assert namedtuple not in objs
 
 
-def _minversion_test():
+def test_minversion():
     from types import ModuleType
     test_module = ModuleType(str("test_module"))
     test_module.__version__ = '0.12.2'
-    good_versions = ['0.12', '0.12.1', '0.12.0.dev']
+    good_versions = ['0.12', '0.12.1', '0.12.0.dev', '0.12dev']
     bad_versions = ['1', '1.2rc1']
     for version in good_versions:
         assert minversion(test_module, version)
     for version in bad_versions:
         assert not minversion(test_module, version)
-
-
-def test_minversion():
-    import sys
-    if 'pkg_resources' in sys.modules:
-        pkg_resources_saved = sys.modules['pkg_resources']
-        # Force ImportError for pkg_resources in minversion()
-        sys.modules['pkg_resource'] = None
-        _minversion_test()
-        sys.modules['pkg_resource'] = pkg_resources_saved
-    _minversion_test()
