@@ -80,14 +80,26 @@ def test_inherit_docstrings():
             "FOO"
             pass
 
+        @property
+        def bar(self):
+            "BAR"
+            pass
+
     class Subclass(Base):
         def __call__(self, *args):
             pass
 
+        @property
+        def bar(self):
+            return 42
+
     if Base.__call__.__doc__ is not None:
         # TODO: Maybe if __doc__ is None this test should be skipped instead?
         assert Subclass.__call__.__doc__ == "FOO"
 
+    if Base.bar.__doc__ is not None:
+        assert Subclass.bar.__doc__ == "BAR"
+
 
 def test_set_locale():
     # First, test if the required locales are available
