@@ -8,7 +8,6 @@
 
 """
 
-
 # STDLIB
 import time
 
@@ -17,6 +16,7 @@ import pytest
 import numpy as np
 
 # LOCAL
+from ..exceptions import AstropyUserWarning
 from ..timer import RunTimePredictor
 from ...modeling.fitting import ModelsError
 
@@ -48,7 +48,9 @@ def test_timer():
 
     # --- These must run next to set up data points. ---
 
-    p.time_func([2.02, 2.04, 2.1, 'a', 2.3])
+    with pytest.warns(AstropyUserWarning, match="ufunc 'multiply' did not "
+                      "contain a loop with signature matching types"):
+        p.time_func([2.02, 2.04, 2.1, 'a', 2.3])
     p.time_func(2.2)  # Test OrderedDict
 
     assert p._funcname == 'func_to_time'
