@@ -112,7 +112,7 @@ class MinMaxInterval(BaseInterval):
     """
 
     def get_limits(self, values):
-        return np.min(values), np.max(values)
+        return np.nanmin(values), np.nanmax(values)
 
 
 class AsymmetricPercentileInterval(BaseInterval):
@@ -150,8 +150,9 @@ class AsymmetricPercentileInterval(BaseInterval):
         values = values[np.isfinite(values)]
 
         # Determine values at percentiles
-        vmin, vmax = np.percentile(values, (self.lower_percentile,
-                                            self.upper_percentile))
+        vmin, vmax = np.nanpercentile(values,
+                                      (self.lower_percentile,
+                                       self.upper_percentile))
 
         return vmin, vmax
 
