@@ -5,6 +5,7 @@ from collections import defaultdict
 
 import numpy as np
 
+from matplotlib import rcParams
 from matplotlib.artist import Artist
 from matplotlib.axes import Axes, subplot_class_factory
 from matplotlib.transforms import Affine2D, Bbox, Transform
@@ -16,7 +17,7 @@ from ...wcs.utils import wcs_to_celestial_frame
 from .transforms import (WCSPixel2WorldTransform, WCSWorld2PixelTransform,
                          CoordinateTransform)
 from .coordinates_map import CoordinatesMap
-from .utils import get_coord_meta
+from .utils import get_coord_meta, transform_contour_set_inplace
 from .frame import EllipticalFrame, RectangularFrame
 
 __all__ = ['WCSAxes', 'WCSAxesSubplot']
@@ -73,7 +74,9 @@ class WCSAxes(Axes):
         ``longitude``, ``latitude``, or ``scalar``, the ``wrap`` entries should
         give, for the longitude, the angle at which the coordinate wraps (and
         `None` otherwise), and the ``unit`` should give the unit of the
-        coordinates as :class:`~astropy.units.Unit` instances.
+        coordinates as :class:`~astropy.units.Unit` instances. This can
+        optionally also include a ``format_unit`` entry giving the units to use
+        for the tick labels (if not specified, this defaults to ``unit``).
     transData : `~matplotlib.transforms.Transform`, optional
         Can be used to override the default data -> pixel mapping.
     slices : tuple, optional
@@ -133,8 +136,8 @@ class WCSAxes(Axes):
 
         world = coords._transform.transform(np.array([pixel]))[0]
 
-        xw = coords[self._x_index].format_coord(world[self._x_index])
-        yw = coords[self._y_index].format_coord(world[self._y_index])
+        xw = coords[self._x_index].format_coord(world[self._x_index], format='ascii')
+        yw = coords[self._y_index].format_coord(world[self._y_index], format='ascii')
 
         if self._display_coords_index == 0:
             system = "world"
@@ -175,9 +178,12 @@ class WCSAxes(Axes):
         All arguments are passed to :meth:`~matplotlib.axes.Axes.imshow`.
         """
 
-        origin = kwargs.get('origin', 'lower')
+        origin = kwargs.pop('origin', 'lower')
 
-        if origin == 'upper':
+        # plt.imshow passes origin as None, which we should default to lower.
+        if origin is None:
+            origin = 'lower'
+        elif origin == 'upper':
             raise ValueError("Cannot use images with origin='upper' in WCSAxes.")
 
         # To check whether the image is a PIL image we can check if the data
@@ -192,9 +198,66 @@ class WCSAxes(Axes):
         else:
             if isinstance(X, Image) or hasattr(X, 'getpixel'):
                 X = X.transpose(FLIP_TOP_BOTTOM)
-                kwargs['origin'] = 'lower'
 
-        return super().imshow(X, *args, **kwargs)
+        return super().imshow(X, *args, origin=origin, **kwargs)
+
+    def contour(self, *args, **kwargs):
+        """
+        Plot contours.
+
+        This is a custom implementation of :meth:`~matplotlib.axes.Axes.contour`
+        which applies the transform (if specified) to all contours in one go for
+        performance rather than to each contour line individually. All
+        positional and keyword arguments are the same as for
+        :meth:`~matplotlib.axes.Axes.contour`.
+        """
+
+        # In Matplotlib, when calling contour() with a transform, each
+        # individual path in the contour map is transformed separately. However,
+        # this is much too slow for us since each call to the transforms results
+        # in an Astropy coordinate transformation, which has a non-negligible
+        # overhead - therefore a better approach is to override contour(), call
+        # the Matplotlib one with no transform, then apply the transform in one
+        # go to all the segments that make up the contour map.
+
+        transform = kwargs.pop('transform', None)
+
+        cset = super().contour(*args, **kwargs)
+
+        if transform is not None:
+            # The transform passed to self.contour will normally include
+            # a transData component at the end, but we can remove that since
+            # we are already working in data space.
+            transform = transform - self.transData
+            cset = transform_contour_set_inplace(cset, transform)
+
+        return cset
+
+    def contourf(self, *args, **kwargs):
+        """
+        Plot filled contours.
+
+        This is a custom implementation of :meth:`~matplotlib.axes.Axes.contourf`
+        which applies the transform (if specified) to all contours in one go for
+        performance rather than to each contour line individually. All
+        positional and keyword arguments are the same as for
+        :meth:`~matplotlib.axes.Axes.contourf`.
+        """
+
+        # See notes for contour above.
+
+        transform = kwargs.pop('transform', None)
+
+        cset = super().contourf(*args, **kwargs)
+
+        if transform is not None:
+            # The transform passed to self.contour will normally include
+            # a transData component at the end, but we can remove that since
+            # we are already working in data space.
+            transform = transform - self.transData
+            cset = transform_contour_set_inplace(cset, transform)
+
+        return cset
 
     def plot_coord(self, *args, **kwargs):
         """
@@ -330,6 +393,9 @@ class WCSAxes(Axes):
                     self.coords[coord_index].set_ticklabel_position('')
                     self.coords[coord_index].set_ticks_position('')
 
+        if rcParams['axes.grid']:
+            self.grid()
+
     def draw_wcsaxes(self, renderer):
 
         # Here need to find out range of all coordinates, and update range for
@@ -575,6 +641,94 @@ class WCSAxes(Axes):
         else:
             raise ValueError('axis should be one of x/y/both')
 
+    def tick_params(self, axis='both', **kwargs):
+        """
+        Method to set the tick and tick label parameters in the same way as the
+        :meth:`~matplotlib.axes.Axes.tick_params` method in Matplotlib.
+
+        This is provided for convenience, but the recommended API is to use
+        :meth:`~astropy.visualization.wcsaxes.CoordinateHelper.set_ticks`,
+        :meth:`~astropy.visualization.wcsaxes.CoordinateHelper.set_ticklabel`,
+        :meth:`~astropy.visualization.wcsaxes.CoordinateHelper.set_ticks_position`,
+        :meth:`~astropy.visualization.wcsaxes.CoordinateHelper.set_ticklabel_position`,
+        and :meth:`~astropy.visualization.wcsaxes.CoordinateHelper.grid`.
+
+        Parameters
+        ----------
+        axis : int or str, optional
+            Which axis to apply the parameters to. This defaults to 'both'
+            but this can also be set to an `int` or `str` that refers to the
+            axis to apply it to, following the valid values that can index
+            ``ax.coords``. Note that ``'x'`` and ``'y``' are also accepted in
+            the case of rectangular axes.
+        which : {'both', 'major', 'minor'}, optional
+            Which ticks to apply the settings to. By default, setting are
+            applied to both major and minor ticks. Note that if ``'minor'`` is
+            specified, only the length of the ticks can be set currently.
+        direction : {'in', 'out'}, optional
+            Puts ticks inside the axes, or outside the axes.
+        length : float, optional
+            Tick length in points.
+        width : float, optional
+            Tick width in points.
+        color : color, optional
+            Tick color (accepts any valid Matplotlib color)
+        pad : float, optional
+            Distance in points between tick and label.
+        labelsize : float or str, optional
+            Tick label font size in points or as a string (e.g., 'large').
+        labelcolor : color, optional
+            Tick label color (accepts any valid Matplotlib color)
+        colors : color, optional
+            Changes the tick color and the label color to the same value
+             (accepts any valid Matplotlib color).
+        bottom, top, left, right : bool, optional
+            Where to draw the ticks. Note that this can only be given if a
+            specific coordinate is specified via the ``axis`` argument, and it
+            will not work correctly if the frame is not rectangular.
+        labelbottom, labeltop, labelleft, labelright : bool, optional
+            Where to draw the tick labels. Note that this can only be given if a
+            specific coordinate is specified via the ``axis`` argument, and it
+            will not work correctly if the frame is not rectangular.
+        grid_color : color, optional
+            The color of the grid lines (accepts any valid Matplotlib color).
+        grid_alpha : float, optional
+            Transparency of grid lines: 0 (transparent) to 1 (opaque).
+        grid_linewidth : float, optional
+            Width of grid lines in points.
+        grid_linestyle : string, optional
+            The style of the grid lines (accepts any valid Matplotlib line
+            style).
+        """
+
+        if not hasattr(self, 'coords'):
+            # Axes haven't been fully initialized yet, so just ignore, as
+            # Axes.__init__ calls this method
+            return
+
+        if axis == 'both':
+
+            for pos in ('bottom', 'left', 'top', 'right'):
+                if pos in kwargs:
+                    raise ValueError("Cannot specify {0}= when axis='both'".format(pos))
+                if 'label' + pos in kwargs:
+                    raise ValueError("Cannot specify label{0}= when axis='both'".format(pos))
+
+            for coord in self.coords:
+                coord.tick_params(**kwargs)
+
+        elif axis in self.coords:
+
+            self.coords[axis].tick_params(**kwargs)
+
+        elif axis in ('x', 'y'):
+
+            if self.frame_class is RectangularFrame:
+                for coord_index in range(len(self.slices)):
+                    if self.slices[coord_index] == axis:
+                        self.coords[coord_index].tick_params(**kwargs)
+
+
 # In the following, we put the generated subplot class in a temporary class and
 # we then inherit it - if we don't do this, the generated class appears to
 # belong in matplotlib, not in WCSAxes, from the API's point of view.
