@@ -6,8 +6,9 @@ from numpy.testing import assert_almost_equal
 
 from matplotlib import rc_context
 
-from ....tests.helper import assert_quantity_allclose
 from .... import units as u
+from ....tests.helper import assert_quantity_allclose
+from ....units import UnitsError
 from ..formatter_locator import AngleFormatterLocator, ScalarFormatterLocator
 
 
@@ -83,7 +84,8 @@ class TestAngleFormatterLocator:
         values, spacing = fl.locator(34.3, 36.1)
         assert_almost_equal(values.to_value(u.degree), [34.5, 35., 35.5, 36.])
 
-        fl.format = 'dd'
+        with pytest.warns(UserWarning, match='Spacing is too small'):
+            fl.format = 'dd'
         values, spacing = fl.locator(34.3, 36.1)
         assert_almost_equal(values.to_value(u.degree), [35., 36.])
 
@@ -121,20 +123,21 @@ class TestAngleFormatterLocator:
                                                     ('hh:mm:ss', '1h01m34s'),
                                                     ('hh:mm:ss.s', '1h01m34.1s'),
                                                     ('hh:mm:ss.ssss', '1h01m34.1354s'),
-                                                    ('d', '15'),
-                                                    ('d.d', '15.4'),
-                                                    ('d.dd', '15.39'),
-                                                    ('d.ddd', '15.392'),
-                                                    ('m', '924'),
-                                                    ('m.m', '923.5'),
-                                                    ('m.mm', '923.53'),
-                                                    ('s', '55412'),
-                                                    ('s.s', '55412.0'),
-                                                    ('s.ss', '55412.03'),
+                                                    ('d', '15\xb0'),
+                                                    ('d.d', '15.4\xb0'),
+                                                    ('d.dd', '15.39\xb0'),
+                                                    ('d.ddd', '15.392\xb0'),
+                                                    ('m', '924\''),
+                                                    ('m.m', '923.5\''),
+                                                    ('m.mm', '923.53\''),
+                                                    ('s', '55412"'),
+                                                    ('s.s', '55412.0"'),
+                                                    ('s.ss', '55412.03"'),
                                                     ])
     def test_format(self, format, string):
         fl = AngleFormatterLocator(number=5, format=format)
-        assert fl.formatter([15.392231] * u.degree, None)[0] == string
+        print(fl.formatter([15.392231] * u.degree, None, format='ascii')[0], string)
+        assert fl.formatter([15.392231] * u.degree, None, format='ascii')[0] == string
 
     @pytest.mark.parametrize(('separator', 'format', 'string'), [(('deg', "'", '"'), 'dd', '15deg'),
                                                                  (('deg', "'", '"'), 'dd:mm', '15deg24\''),
@@ -143,8 +146,8 @@ class TestAngleFormatterLocator:
                                                                  (':', 'dd:mm:ss.s', '15:23:32.0'),
                                                                  ((':', ":", 's'), 'hh', '1:'),
                                                                  (('-', "-", 's'), 'hh:mm:ss.ssss', '1-01-34.1354s'),
-                                                                 (('d', ":", '"'), 'd', '15'),
-                                                                 (('d', ":", '"'), 'd.d', '15.4'),
+                                                                 (('d', ":", '"'), 'd', '15\xb0'),
+                                                                 (('d', ":", '"'), 'd.d', '15.4\xb0'),
                                                                  ])
     def test_separator(self, separator, format, string):
         fl = AngleFormatterLocator(number=5, format=format)
@@ -155,7 +158,7 @@ class TestAngleFormatterLocator:
         fl = AngleFormatterLocator(number=5, format="dd:mm:ss")
         assert fl.formatter([15.392231] * u.degree, None)[0] == '15\xb023\'32"'
         with rc_context(rc={'text.usetex': True}):
-            assert fl.formatter([15.392231] * u.degree, None)[0] == "15$^\\circ$23'32\""
+            assert fl.formatter([15.392231] * u.degree, None)[0] == "$15^\\circ23{}^\\prime32{}^{\\prime\\prime}$"
 
     @pytest.mark.parametrize(('format'), ['x.xxx', 'dd.ss', 'dd:ss', 'mdd:mm:ss'])
     def test_invalid_formats(self, format):
@@ -190,7 +193,8 @@ class TestAngleFormatterLocator:
     def test_incorrect_spacing(self):
         fl = AngleFormatterLocator()
         fl.spacing = 0.032 * u.deg
-        fl.format = 'dd:mm:ss'
+        with pytest.warns(UserWarning, match='Spacing is not a multiple of base spacing'):
+            fl.format = 'dd:mm:ss'
         assert_almost_equal(fl.spacing.to_value(u.arcsec), 115.)
 
     def test_decimal_values(self):
@@ -203,6 +207,34 @@ class TestAngleFormatterLocator:
         assert_quantity_allclose(fl.locator(266.9730, 266.9750)[0],
                                  [266.9735, 266.9740, 266.9745, 266.9750] * u.deg)
 
+        fl = AngleFormatterLocator(decimal=True, format_unit=u.hourangle, number=4)
+        assert_quantity_allclose(fl.locator(266.9730, 266.9750)[0],
+                                 [17.79825, 17.79830] * u.hourangle)
+
+    def test_values_unit(self):
+
+        # Make sure that the intrinsic unit and format unit are correctly
+        # taken into account when using the locator
+
+        fl = AngleFormatterLocator(unit=u.arcsec, format_unit=u.arcsec, decimal=True)
+        assert_quantity_allclose(fl.locator(850, 2150)[0],
+                                 [1000., 1200., 1400., 1600., 1800., 2000.] * u.arcsec)
+
+        fl = AngleFormatterLocator(unit=u.arcsec, format_unit=u.degree, decimal=False)
+        assert_quantity_allclose(fl.locator(850, 2150)[0],
+                                 [15., 20., 25., 30., 35.] * u.arcmin)
+
+        fl = AngleFormatterLocator(unit=u.arcsec, format_unit=u.hourangle, decimal=False)
+        assert_quantity_allclose(fl.locator(850, 2150)[0],
+                                 [60., 75., 90., 105., 120., 135.] * (15 * u.arcsec))
+
+        fl = AngleFormatterLocator(unit=u.arcsec)
+        fl.format = 'dd:mm:ss'
+        assert_quantity_allclose(fl.locator(0.9, 1.1)[0], [1] * u.arcsec)
+
+        fl = AngleFormatterLocator(unit=u.arcsec, spacing=0.2 * u.arcsec)
+        assert_quantity_allclose(fl.locator(0.3, 0.9)[0], [0.4, 0.6, 0.8] * u.arcsec)
+
     @pytest.mark.parametrize(('spacing', 'string'), [(2 * u.deg, '15\xb0'),
                                                      (2 * u.arcmin, '15\xb024\''),
                                                      (2 * u.arcsec, '15\xb023\'32"'),
@@ -211,6 +243,50 @@ class TestAngleFormatterLocator:
         fl = AngleFormatterLocator()
         assert fl.formatter([15.392231] * u.degree, spacing)[0] == string
 
+    @pytest.mark.parametrize(('format_unit', 'decimal', 'show_decimal_unit', 'spacing', 'ascii', 'latex'),
+                             [(u.degree, False, True, 2 * u.degree, '15\xb0', r'$15^\circ$'),
+                              (u.degree, False, True, 2 * u.arcmin, '15\xb024\'', r'$15^\circ24{}^\prime$'),
+                              (u.degree, False, True, 2 * u.arcsec, '15\xb023\'32"', r'$15^\circ23{}^\prime32{}^{\prime\prime}$'),
+                              (u.degree, False, True, 0.1 * u.arcsec, '15\xb023\'32.0"', r'$15^\circ23{}^\prime32.0{}^{\prime\prime}$'),
+                              (u.hourangle, False, True, 15 * u.degree, '1h', r'$1^\mathrm{h}$'),
+                              (u.hourangle, False, True, 15 * u.arcmin, '1h02m', r'$1^\mathrm{h}02^\mathrm{m}$'),
+                              (u.hourangle, False, True, 15 * u.arcsec, '1h01m34s', r'$1^\mathrm{h}01^\mathrm{m}34^\mathrm{s}$'),
+                              (u.hourangle, False, True, 1.5 * u.arcsec, '1h01m34.1s', r'$1^\mathrm{h}01^\mathrm{m}34.1^\mathrm{s}$'),
+                              (u.degree, True, True, 15 * u.degree, '15\xb0', r'$15\mathrm{^\circ}$'),
+                              (u.degree, True, True, 0.12 * u.degree, '15.39\xb0', r'$15.39\mathrm{^\circ}$'),
+                              (u.degree, True, True, 0.0036 * u.arcsec, '15.392231\xb0', r'$15.392231\mathrm{^\circ}$'),
+                              (u.arcmin, True, True, 15 * u.degree, '924\'', r'$924\mathrm{^\prime}$'),
+                              (u.arcmin, True, True, 0.12 * u.degree, '923.5\'', r'$923.5\mathrm{^\prime}$'),
+                              (u.arcmin, True, True, 0.1 * u.arcmin, '923.5\'', r'$923.5\mathrm{^\prime}$'),
+                              (u.arcmin, True, True, 0.0002 * u.arcmin, '923.5339\'', r'$923.5339\mathrm{^\prime}$'),
+                              (u.arcsec, True, True, 0.01 * u.arcsec, '55412.03"', r'$55412.03\mathrm{^{\prime\prime}}$'),
+                              (u.arcsec, True, True, 0.001 * u.arcsec, '55412.032"', r'$55412.032\mathrm{^{\prime\prime}}$'),
+                              (u.mas, True, True, 0.001 * u.arcsec, '55412032mas', r'$55412032\mathrm{mas}$'),
+                              (u.degree, True, False, 15 * u.degree, '15', '15'),
+                              (u.degree, True, False, 0.12 * u.degree, '15.39', '15.39'),
+                              (u.degree, True, False, 0.0036 * u.arcsec, '15.392231', '15.392231'),
+                              (u.arcmin, True, False, 15 * u.degree, '924', '924'),
+                              (u.arcmin, True, False, 0.12 * u.degree, '923.5', '923.5'),
+                              (u.arcmin, True, False, 0.1 * u.arcmin, '923.5', '923.5'),
+                              (u.arcmin, True, False, 0.0002 * u.arcmin, '923.5339', '923.5339'),
+                              (u.arcsec, True, False, 0.01 * u.arcsec, '55412.03', '55412.03'),
+                              (u.arcsec, True, False, 0.001 * u.arcsec, '55412.032', '55412.032'),
+                              (u.mas, True, False, 0.001 * u.arcsec, '55412032', '55412032'),
+                              # Make sure that specifying None defaults to
+                              # decimal for non-degree or non-hour angles
+                              (u.arcsec, None, True, 0.01 * u.arcsec, '55412.03"', r'$55412.03\mathrm{^{\prime\prime}}$')])
+    def test_formatter_no_format_with_units(self, format_unit, decimal, show_decimal_unit, spacing, ascii, latex):
+        # Check the formatter works when specifying the default units and
+        # decimal behavior to use.
+        fl = AngleFormatterLocator(unit=u.degree, format_unit=format_unit, decimal=decimal, show_decimal_unit=show_decimal_unit)
+        assert fl.formatter([15.392231] * u.degree, spacing, format='ascii')[0] == ascii
+        assert fl.formatter([15.392231] * u.degree, spacing, format='latex')[0] == latex
+
+    def test_incompatible_unit_decimal(self):
+        with pytest.raises(UnitsError) as exc:
+            AngleFormatterLocator(unit=u.arcmin, decimal=False)
+        assert exc.value.args[0] == 'Units should be degrees or hours when using non-decimal (sexagesimal) mode'
+
 
 class TestScalarFormatterLocator:
 
@@ -280,7 +356,8 @@ class TestScalarFormatterLocator:
         values, spacing = fl.locator(34.3, 36.1)
         assert_almost_equal(values.value, [34.5, 35., 35.5, 36.])
 
-        fl.format = 'x'
+        with pytest.warns(UserWarning, match='Spacing is too small'):
+            fl.format = 'x'
         values, spacing = fl.locator(34.3, 36.1)
         assert_almost_equal(values.value, [35., 36.])
 
@@ -346,5 +423,19 @@ class TestScalarFormatterLocator:
     def test_incorrect_spacing(self):
         fl = ScalarFormatterLocator(unit=u.m)
         fl.spacing = 0.032 * u.m
-        fl.format = 'x.xx'
+        with pytest.warns(UserWarning, match='Spacing is not a multiple of base spacing'):
+            fl.format = 'x.xx'
         assert_almost_equal(fl.spacing.to_value(u.m), 0.03)
+
+    def test_values_unit(self):
+
+        # Make sure that the intrinsic unit and format unit are correctly
+        # taken into account when using the locator
+
+        fl = ScalarFormatterLocator(unit=u.cm, format_unit=u.m)
+        assert_quantity_allclose(fl.locator(850, 2150)[0],
+                                 [1000., 1200., 1400., 1600., 1800., 2000.] * u.cm)
+
+        fl = ScalarFormatterLocator(unit=u.cm, format_unit=u.m)
+        fl.format = 'x.x'
+        assert_quantity_allclose(fl.locator(1, 19)[0], [10] * u.cm)
