@@ -19,6 +19,8 @@ from ....tests.image_tests import IMAGE_REFERENCE_DIR
 
 class DistanceToLonLat(CurvedTransform):
 
+    has_inverse = True
+
     def __init__(self, R=6e3):
         super().__init__()
         self.R = R
@@ -58,7 +60,6 @@ class TestTransformCoordMeta(BaseImageTests):
 
     @pytest.mark.remote_data(source='astropy')
     @pytest.mark.mpl_image_compare(baseline_dir=IMAGE_REFERENCE_DIR,
-                                   filename='coords_overlay.png',
                                    tolerance=0, style={})
     def test_coords_overlay(self):
 
@@ -92,14 +93,14 @@ class TestTransformCoordMeta(BaseImageTests):
         overlay['lon'].grid(color='red', linestyle='solid', alpha=0.3)
         overlay['lat'].grid(color='blue', linestyle='solid', alpha=0.3)
 
-        overlay['lon'].set_ticklabel(size=7)
-        overlay['lat'].set_ticklabel(size=7)
+        overlay['lon'].set_ticklabel(size=7, exclude_overlapping=True)
+        overlay['lat'].set_ticklabel(size=7, exclude_overlapping=True)
 
         overlay['lon'].set_ticklabel_position('brtl')
         overlay['lat'].set_ticklabel_position('brtl')
 
-        overlay['lon'].set_ticks(spacing=10. * u.deg, exclude_overlapping=True)
-        overlay['lat'].set_ticks(spacing=10. * u.deg, exclude_overlapping=True)
+        overlay['lon'].set_ticks(spacing=10. * u.deg)
+        overlay['lat'].set_ticks(spacing=10. * u.deg)
 
         ax.set_xlim(-0.5, 1215.5)
         ax.set_ylim(-0.5, 1791.5)
@@ -108,7 +109,6 @@ class TestTransformCoordMeta(BaseImageTests):
 
     @pytest.mark.remote_data(source='astropy')
     @pytest.mark.mpl_image_compare(baseline_dir=IMAGE_REFERENCE_DIR,
-                                   filename='coords_overlay_auto_coord_meta.png',
                                    tolerance=0, style={})
     def test_coords_overlay_auto_coord_meta(self):
 
@@ -133,7 +133,6 @@ class TestTransformCoordMeta(BaseImageTests):
 
     @pytest.mark.remote_data(source='astropy')
     @pytest.mark.mpl_image_compare(baseline_dir=IMAGE_REFERENCE_DIR,
-                                   filename='direct_init.png',
                                    tolerance=0, style={})
     def test_direct_init(self):
 
@@ -152,14 +151,14 @@ class TestTransformCoordMeta(BaseImageTests):
         ax.coords['lon'].grid(color='red', linestyle='solid', alpha=0.3)
         ax.coords['lat'].grid(color='blue', linestyle='solid', alpha=0.3)
 
-        ax.coords['lon'].set_ticklabel(size=7)
-        ax.coords['lat'].set_ticklabel(size=7)
+        ax.coords['lon'].set_ticklabel(size=7, exclude_overlapping=True)
+        ax.coords['lat'].set_ticklabel(size=7, exclude_overlapping=True)
 
         ax.coords['lon'].set_ticklabel_position('brtl')
         ax.coords['lat'].set_ticklabel_position('brtl')
 
-        ax.coords['lon'].set_ticks(spacing=10. * u.deg, exclude_overlapping=True)
-        ax.coords['lat'].set_ticks(spacing=10. * u.deg, exclude_overlapping=True)
+        ax.coords['lon'].set_ticks(spacing=10. * u.deg)
+        ax.coords['lat'].set_ticks(spacing=10. * u.deg)
 
         ax.set_xlim(-400., 500.)
         ax.set_ylim(-300., 400.)
