@@ -235,7 +235,7 @@ def test_celestial():
 def test_wcs_to_celestial_frame():
 
     # Import astropy.coordinates here to avoid circular imports
-    from ...coordinates.builtin_frames import ICRS, FK5, FK4, Galactic
+    from ...coordinates.builtin_frames import ICRS, ITRS, FK5, FK4, Galactic
 
     mywcs = WCS(naxis=2)
     mywcs.wcs.set()
@@ -278,6 +278,14 @@ def test_wcs_to_celestial_frame():
     frame = wcs_to_celestial_frame(mywcs)
     assert isinstance(frame, Galactic)
 
+    mywcs = WCS(naxis=2)
+    mywcs.wcs.ctype = ['TLON-CAR', 'TLAT-CAR']
+    mywcs.wcs.dateobs = '2017-08-17T12:41:04.430'
+    mywcs.wcs.set()
+    frame = wcs_to_celestial_frame(mywcs)
+    assert isinstance(frame, ITRS)
+    assert frame.obstime == Time('2017-08-17T12:41:04.430')
+
     for equinox in [np.nan, 1987, 1982]:
         mywcs = WCS(naxis=2)
         mywcs.wcs.ctype = ['RA---TAN', 'DEC--TAN']
@@ -335,7 +343,7 @@ def test_wcs_to_celestial_frame_extend():
 def test_celestial_frame_to_wcs():
 
     # Import astropy.coordinates here to avoid circular imports
-    from ...coordinates import ICRS, FK5, FK4, FK4NoETerms, Galactic, BaseCoordinateFrame
+    from ...coordinates import ICRS, ITRS, FK5, FK4, FK4NoETerms, Galactic, BaseCoordinateFrame
 
     class FakeFrame(BaseCoordinateFrame):
         pass
@@ -391,6 +399,18 @@ def test_celestial_frame_to_wcs():
     mywcs.wcs.set()
     assert_allclose((mywcs.wcs.lonpole, mywcs.wcs.latpole), (180, 60))
 
+    frame = ITRS(obstime=Time('2017-08-17T12:41:04.43'))
+    mywcs = celestial_frame_to_wcs(frame, projection='CAR')
+    assert tuple(mywcs.wcs.ctype) == ('TLON-CAR', 'TLAT-CAR')
+    assert mywcs.wcs.radesys == 'ITRS'
+    assert mywcs.wcs.dateobs == '2017-08-17T12:41:04.430'
+
+    frame = ITRS()
+    mywcs = celestial_frame_to_wcs(frame, projection='CAR')
+    assert tuple(mywcs.wcs.ctype) == ('TLON-CAR', 'TLAT-CAR')
+    assert mywcs.wcs.radesys == 'ITRS'
+    assert mywcs.wcs.dateobs == Time('J2000').utc.isot
+
 
 def test_celestial_frame_to_wcs_extend():
 
