@@ -17,7 +17,9 @@ from .. import _wcs
 from ...utils.data import (
     get_pkg_data_filenames, get_pkg_data_contents, get_pkg_data_filename)
 from ...utils.misc import NumpyRNGContext
+from ...utils.exceptions import AstropyUserWarning
 from ...io import fits
+from ...coordinates import SkyCoord
 
 
 class TestMaps:
@@ -390,7 +392,9 @@ def test_validate():
         results = wcs.validate(get_pkg_data_filename("data/validate.fits"))
         results_txt = repr(results)
         version = wcs._wcs.__version__
-        if version[0] == '5':
+        if version[0] == '6':
+            filename = 'data/validate.6.txt'
+        elif version[0] == '5':
             if version >= '5.13':
                 filename = 'data/validate.5.13.txt'
             else:
@@ -593,11 +597,12 @@ def test_footprint_to_file(tmpdir):
     From github issue #1912
     """
     # Arbitrary keywords from real data
-    w = wcs.WCS({'CTYPE1': 'RA---ZPN', 'CRUNIT1': 'deg',
-                 'CRPIX1': -3.3495999e+02, 'CRVAL1': 3.185790700000e+02,
-                 'CTYPE2': 'DEC--ZPN', 'CRUNIT2': 'deg',
-                 'CRPIX2': 3.0453999e+03, 'CRVAL2': 4.388538000000e+01,
-                 'PV2_1': 1., 'PV2_3': 220.})
+    hdr = {'CTYPE1': 'RA---ZPN', 'CRUNIT1': 'deg',
+           'CRPIX1': -3.3495999e+02, 'CRVAL1': 3.185790700000e+02,
+           'CTYPE2': 'DEC--ZPN', 'CRUNIT2': 'deg',
+           'CRPIX2': 3.0453999e+03, 'CRVAL2': 4.388538000000e+01,
+           'PV2_1': 1., 'PV2_3': 220., 'NAXIS1': 2048, 'NAXIS2': 1024}
+    w = wcs.WCS(hdr)
 
     testfile = str(tmpdir.join('test.txt'))
     w.footprint_to_file(testfile)
@@ -621,6 +626,12 @@ def test_footprint_to_file(tmpdir):
     with pytest.raises(ValueError):
         w.footprint_to_file(testfile, coordsys='FOO')
 
+    del hdr['NAXIS1']
+    del hdr['NAXIS2']
+    w = wcs.WCS(hdr)
+    with pytest.warns(AstropyUserWarning):
+        w.footprint_to_file(testfile)
+
 
 def test_validate_faulty_wcs():
     """
@@ -1042,14 +1053,19 @@ def test_naxis():
     w.wcs.cdelt = [0.1, 0.1]
     w.wcs.crpix = [1, 1]
     w._naxis = [1000, 500]
+    assert w.pixel_shape == (1000, 500)
+    assert w.array_shape == (500, 1000)
 
-    assert w._naxis1 == 1000
-    assert w._naxis2 == 500
-
-    w._naxis1 = 99
-    w._naxis2 = 59
+    w.pixel_shape = (99, 59)
     assert w._naxis == [99, 59]
 
+    w.array_shape = (45, 23)
+    assert w._naxis == [23, 45]
+    assert w.pixel_shape == (23, 45)
+
+    w.pixel_shape = None
+    assert w.pixel_bounds is None
+
 
 def test_sip_with_altkey():
     """
@@ -1094,7 +1110,6 @@ def test_keyedsip():
     assert w.sip.crpix[0] == 2048
     assert w.sip.crpix[1] == 1026
 
-
 def test_zero_size_input():
     with fits.open(get_pkg_data_filename('data/sip.fits')) as f:
         w = wcs.WCS(f[0].header)
@@ -1125,3 +1140,46 @@ def test_scalar_inputs():
     result = wcsobj.all_pix2world([2], 1)
     assert_array_equal(result, [np.array([2.])])
     assert result[0].shape == (1,)
+
+def test_footprint_contains():
+    """
+    Test WCS.footprint_contains(skycoord)
+    """
+
+    header = """
+WCSAXES =                    2 / Number of coordinate axes
+CRPIX1  =               1045.0 / Pixel coordinate of reference point
+CRPIX2  =               1001.0 / Pixel coordinate of reference point
+PC1_1   =    -0.00556448550786 / Coordinate transformation matrix element
+PC1_2   =   -0.001042120133257 / Coordinate transformation matrix element
+PC2_1   =    0.001181477028705 / Coordinate transformation matrix element
+PC2_2   =   -0.005590809742987 / Coordinate transformation matrix element
+CDELT1  =                  1.0 / [deg] Coordinate increment at reference point
+CDELT2  =                  1.0 / [deg] Coordinate increment at reference point
+CUNIT1  = 'deg'                / Units of coordinate increment and value
+CUNIT2  = 'deg'                / Units of coordinate increment and value
+CTYPE1  = 'RA---TAN'           / TAN (gnomonic) projection + SIP distortions
+CTYPE2  = 'DEC--TAN'           / TAN (gnomonic) projection + SIP distortions
+CRVAL1  =      250.34971683647 / [deg] Coordinate value at reference point
+CRVAL2  =      2.2808772582495 / [deg] Coordinate value at reference point
+LONPOLE =                180.0 / [deg] Native longitude of celestial pole
+LATPOLE =      2.2808772582495 / [deg] Native latitude of celestial pole
+RADESYS = 'ICRS'               / Equatorial coordinate system
+MJD-OBS =      58612.339199259 / [d] MJD of observation matching DATE-OBS
+DATE-OBS= '2019-05-09T08:08:26.816Z' / ISO-8601 observation date matching MJD-OB
+NAXIS   =                    2 / NAXIS
+NAXIS1  =                 2136 / length of first array dimension
+NAXIS2  =                 2078 / length of second array dimension
+    """
+
+    header = fits.Header.fromstring(header.strip(),'\n')
+    test_wcs = wcs.WCS(header)
+
+    hasCoord = test_wcs.footprint_contains(SkyCoord(254,2,unit='deg'))
+    assert hasCoord == True
+
+    hasCoord = test_wcs.footprint_contains(SkyCoord(240,2,unit='deg'))
+    assert hasCoord == False
+
+    hasCoord = test_wcs.footprint_contains(SkyCoord(24,2,unit='deg'))
+    assert hasCoord == False
