@@ -1,6 +1,6 @@
 /*============================================================================
 
-  WCSLIB 5.18 - an implementation of the FITS WCS standard.
+  WCSLIB 5.19 - an implementation of the FITS WCS standard.
   Copyright (C) 1995-2018, Mark Calabretta
 
   This file is part of WCSLIB.
@@ -22,7 +22,7 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: spc.c,v 5.18 2018/01/10 08:32:14 mcalabre Exp $
+  $Id: spc.c,v 5.19.1.1 2018/07/26 15:41:40 mcalabre Exp mcalabre $
 *===========================================================================*/
 
 #include <math.h>
@@ -207,13 +207,13 @@ int spcprt(const struct spcprm *spc)
   }
 
   wcsprintf("     spxX2P: %s\n",
-    wcsutil_fptr2str((int (*)(void))spc->spxX2P, hext));
+    wcsutil_fptr2str((void (*)(void))spc->spxX2P, hext));
   wcsprintf("     spxP2S: %s\n",
-    wcsutil_fptr2str((int (*)(void))spc->spxP2S, hext));
+    wcsutil_fptr2str((void (*)(void))spc->spxP2S, hext));
   wcsprintf("     spxS2P: %s\n",
-    wcsutil_fptr2str((int (*)(void))spc->spxS2P, hext));
+    wcsutil_fptr2str((void (*)(void))spc->spxS2P, hext));
   wcsprintf("     spxP2X: %s\n",
-    wcsutil_fptr2str((int (*)(void))spc->spxP2X, hext));
+    wcsutil_fptr2str((void (*)(void))spc->spxP2X, hext));
 
   return 0;
 }
